// Notice 
//  
// The contents of this file are subject to the
//  same license as the source code for ImageJ.  
// FracLac is in the public domain.
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLAnalyzer;
/**Calculates regression statistics on passed arrays or their 
 * transformations.
 *   
 * <h3>Inputs:</h3>
 * integer or float arrays, integer for number of values to assess
 * <h3>Outputs:</h3>
 * <p>Regression statistics
 * The stats are the following five values for the original or
 * transformed arrays:
 * <ul><LI>slope of the regression line
 * =((n*&#8721;SC)-(&#8721;S*&#8721;C))
 * /((n*&#8721;S<sup>2</sup>)-(&#8721;S*&#8721;S)),
 * <LI> r<sup>2</sup> correlation coefficient
 * =(n*(&#8721;SC)-(&#8721;S*&#8721;C))
 * /&#8730;(((n*&#8721;Ssq)-(&#8721;S*&#8721;S))
 * ((n*&#8721;Csq)-((&#8721;C*&#8721;C))));
 * <LI> y intercept= (&#8721;C - (slope*&#8721;S))/n;,
 * <LI> inverse of prefactor FractalStatsF = Euler's e<sup>
 * y-int</sup>
 * <LI> and standard error of estimate as
 * &#8730;((&#8721;C<sup>2</sup>-y-int*&#8721;
 * C-slope*&#8721;SC)/(n-2))
 * </ul>
 * <h3>Needs:</h3><LI>not dependent</LI>
 *
 * The arrays are transformed differently depending on the
 * specific methods used.  For untransformed data, use the
 * {@link #PlainLinearRegression plain linear regression}
 * method or the
 * {@link #PowerRegression power regression} method. For fractal
 * analysis, use {@link #FindDfStats Find Df stats method} which
 * uses the log of C and the log of 1/S  for passed arrays C and S.
 * Constructors call {@link #GetStats GetStats method}
 * to initialize variables for fractal analysis but other methods
 * must be called explicitly to return the proper statistics.
 *
 * <p> For futher information or to send comments, bugs, 
 * and feature requests
 * contact{@link <a href="mailto:akarpe@postoffice.csu.edu.au"> 
 * the author</a>}
 *
 * <p>file: FracStats.java</p>
 * <p>IDE: NetBeans 4.1</p>
 * @version FracLac v2.4e FracStats 2006
 * @author Audrey Karperien (akarpe01@postoffice.csu.edu.au),
 * Charles Sturt University
 * @since J2SKD1.5.0_03
 */

public class FracStats
{
    //////////////////////////////////////////////////////////////////////
    //////////////////////////// Variables  //////////////////////////////
    //////////////////////////////////////////////////////////////////////
    /*Variables set in {@link #PlainLinearRegression 
     linear regression} */
    public float PlainsumSC=0,
    PlainsumS=0,
    PlainsumC=0,
    PlainsumSsq = 0,
    PlainsumCsq=0;
    public float[] oldcount;
    /**Float array of values for size in a box count*/
    public float[] oldsize;
    /**Static ints for defining array positions*/
    final public static int SLOPEindex=0,
            DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4;
/**Used for {@link #LinearSmoothStats smoothed} arrays
@see #LinearSmoothStats
@see FLAnalyzer.FLMain#BoxCountDataWriter*/
public int smoothedNumBoxes = 0;
    
    public int oldnum;    
    public float[] SmoothedSize;    
    public float[] SmoothedCount;
    
    /**Slope of the linear regression line.
     * Is changed in {@link #GetStats get stats} to the
     * first element of the array returned by
     * {@link #PlainLinearRegression linear regression}.*/
    public float Pslope;
    /**Correlation coefficient squared or r<sup>2</sup>*/
    public float Prsq;
    /**y-intercept*/
    public float Pyint;
    /**Inverse of prefactor.  This variable is set in the
     *{@link #PlainLinearRegression} method, from index 3 (item 4)
     *of the returned array from
     * {@link #PlainLinearRegression regression}
     *method.
     */
    public float PF;
  
    /**Standard error returned by 
     * {@link #PlainLinearRegression linear regression}.*/
    public float Pstderr;
    /**Fractal dimension or negative slope of log-log regression line
     * calculated in {@link #FindDfStats fractal} analysis method.*/
    public float df;
    /**Y intercept of log-log regression line, index 2 (item 3)
     * returned by
     * {@link #FindDfStats fractal dimension} method.*/
    public float FractalStatsYint;
    /**FractalStatsF parameter or inverse of 
     * prefactor A in AX<sup>d</sup>
     * index 3 (element 4) returned by 
     * {@link #FindDfStats fractal dimension}
     * function.*/
    public float FractalStatsF;
    /**{@link #LinearSmoothStats Compressed} fractal dimension.*/
    public float smootheddf;
    /**{@link #LinearSmoothStats Compressed} y-intercept.*/
    public float Smoothedyint;
    /**When {@link #GetStats get stats} method is invoked,
     * this is set to index 3 (item 4)
     * from the {@link #LinearSmoothStats linearly smoothed method}
     * .*/
    public float SmoothedF;
    /**R<sup>2</sup> for log-log regression 
     * line, set from index 1 (item 2) from
     * {@link #FindDfStats fractal dimension} 
     * method when {@link #GetStats get stats}
     * method is invoked.*/
    public float FractalStatsRsq;
    /**R squared for  {@link #LinearSmoothStats Compressed} 
     * regression line.*/
    public  float Smoothedrsq;
    /**Slope of linear regression line.*/
    public float ldf;
    /**Y-intercept of linear regression line.*/
    public float lyint;
    /**R squared for linear regression line.*/
    public  float lrsq;
    /**FractalStatsF parameter for linear regression line.*/
    public float lF;
    /**Standard error of log-log regression line.*/
    public float stderr;
    /**Standard error of linear regression line.*/
    public float lstderr;
    /**Standard error linearly smoothed log-log regression line.*/
    public   float Smoothedstderr;
    /**Linear regression line linearly smoothed slope.*/
    public  float lcdf;
    /**R squared for linear
     * @link #LinearSmoothStats Compressed} regression line.*/
    public float lcrsq;
    /**Y-intercept for linear
     * {@link #LinearSmoothStats Compressed} regression line.*/
    public float lcyint;
    /**FractalStatsF parameter for linear
     * {@link #LinearSmoothStats Compressed} regression line.*/
    public float lcF;
    /**Standard error for
     * {@link #LinearSmoothStats Compressed} 
     * linear regression line.*/
    public float lcstderr;
    //////////////////////////////////////////////////////////////////////
    ////////////////////////////Constructors//////////////////////////////
    //////////////////////////////////////////////////////////////////////
    /**Constructor accepts arrays and calls 
     * {@link #GetStats GetStats} method.
     *@param C integer array for Y axis
     * (corresponds to counts for fractal analysis
     * @param S integer array for X axis
     * (corresponds to sizes for fractal analysis
     * @param originalNumSizes number of values
     * to use in passed arrays
     */
    public FracStats (float[] C, int[] S, int originalNumSizes)
    {
        oldcount = new float [originalNumSizes];
        oldsize = new float [originalNumSizes];
        for (int w = 0; w<originalNumSizes; w++)
        {oldcount[w]=(float)C[w];
         oldsize[w]=(float)S[w];}
        
        this.oldnum=originalNumSizes;
        GetStats (oldcount, oldsize, originalNumSizes);
        
    }
    /**Constructor .*/
    public FracStats ()
    {};
    /**Constructor accepts arrays and calls
     * {@link #GetStats GetStats} method.
     * @param C float array
     * (corresponds to counts for fractal analysis)
     * @param S float array (corresponds to sizes for fractal analysis)
     * @param originalNumSizes int for values to use in passed arrays
     */
    public FracStats (double[] C, double[] S, int originalNumSizes)
    {
        oldcount = new float [originalNumSizes];
        oldsize = new float [originalNumSizes];
        for (int w = 0; w<originalNumSizes; w++)
        {oldcount[w]=(float)C[w];
         oldsize[w]=(float)S[w];}
        
        this.oldnum=originalNumSizes;
        GetStats (oldcount, oldsize, originalNumSizes);
        
    }
    /**Constructor accepts arrays and calls
     * {@link #GetStats GetStats} method.
     * @param C float array (corresponds to
     * counts for fractal analysis)
     * @param S float array (corresponds to sizes for fractal analysis)
     * @param originalNumSizes int for values to use in passed arrays
     */
    public FracStats (float[] C, float[] S, int originalNumSizes)
    {
        oldcount = new float [originalNumSizes];
        oldsize = new float [originalNumSizes];
        for (int w = 0; w<originalNumSizes; w++)
        {
            oldcount[w]=C[w];
            oldsize[w]=S[w];
        }        
        this.oldnum=originalNumSizes;
        GetStats (oldcount, oldsize, originalNumSizes);
        
    }
    /**Constructor accepts arrays and calls 
     * {@link #GetStats GetStats} method.
     * @param C int array (corresponds to counts for fractal analysis)
     * @param S float array (corresponds to sizes for fractal analysis)
     * @param originalNumSizes int for values to use in passed arrays
     */
    public FracStats (int[] C, float[] S, int originalNumSizes)
    {
        oldcount = new float [originalNumSizes];
        oldsize = new float [originalNumSizes];
        for (int w = 0; w<originalNumSizes; w++)
        {oldcount[w]=(float)C[w];
         oldsize[w]=(float)S[w];}
        
        this.oldnum=originalNumSizes;
        GetStats (oldcount, oldsize, originalNumSizes);
        
    }

    /**Constructor accepts arrays and calls
     * {@link #GetStats GetStats} method.
     * @param C int array (corresponds to counts for fractal analysis)
     * @param S int array (corresponds to sizes for fractal analysis)
     * @param originalNumSizes int for values to use in passed arrays
     */
    public FracStats (int[] C, int[] S, int originalNumSizes)
    {
        oldcount = new float [originalNumSizes];
        oldsize = new float [originalNumSizes];
        for (int w = 0; w<originalNumSizes; w++)
        {oldcount[w]=(float)C[w];
         oldsize[w]=(float)S[w];}
        
        this.oldnum=originalNumSizes;
        GetStats (oldcount, oldsize, originalNumSizes);
        
    }

//////////////////////////////////////////////////////////////////////
////////////////////////////  Methods   //////////////////////////////
//////////////////////////////////////////////////////////////////////
/**Calculates stats but returns nothing so
* individual variables are
* accessed to get the stats.
* Is automatically called by constructors but must be
* called again for any changes. Calls
* {@link #FindDfStats FindDfStats}, 
* {@link #LinearSmoothStats FindCompressed},
* and {@link #InvLinearRegression LinearRegression} methods.
* Variables set are <p>stats for power regression for
* count and 1/size: <p>{@link #df slope or fractal dimension},
* <p> {@link #FractalStatsRsq r<sup>2</sup>}, 
* <p> {@link #FractalStatsYint y intercept},
* <p> {@link #FractalStatsF prefactor},<p>  
* {@link #stderr standard error}, <p>
*stats for array after horizontal slopes removed:
* {@link #smootheddf slope or fractal dimension},
* <p>  {@link #Smoothedrsq r<sup>2</sup>}, <p>
* {@link #Smoothedyint y intercept},
* <p> {@link #SmoothedF prefactor}, <p> 
* {@link #Smoothedstderr standard error},
* stats for linear regression: <p> {@link #ldf},
* <p> {@link #lrsq},<p>  {@link #lyint},
* <p> {@link #lF}, <p> {@link #lstderr}, <p> {@link #lcdf},
* <p> {@link #lcrsq}, <p> {@link #lcyint},
* <p> {@link #lcF}, <p> {@link #lcstderr}.
*
* @param y float array of counts
* @param x integer array of sizes
* @param n integer for number of values to use
*/
public void GetStats (float[] y, float[] x, int n)
{
    float [] reg = FindDfStats (y, x, n);
    df= reg[0];
    FractalStatsRsq=reg[1];
    FractalStatsYint=reg[2];
    FractalStatsF=reg[3];
    stderr=reg[4];
    float [] comp =   LinearSmoothStats (y, x, n);
    smootheddf= comp[0];
    Smoothedrsq=comp[1];
    Smoothedyint=comp[2];
    SmoothedF=comp[3];
    Smoothedstderr=comp[4];
    float [] linear= InvLinearRegression  (y, x, n);
    ldf=linear[0];
    lrsq=linear[1];
    lyint=linear[2];
    lF=linear[3];
    lstderr=linear[4];
    float [] lcompress = 
            InvLinearRegression (SmoothedCount, SmoothedSize, smoothedNumBoxes);
    lcdf=lcompress[0];
    lcrsq=lcompress[1];
    lcyint=lcompress[2];
    lcF=lcompress[3];
    lcstderr=lcompress[4];
    float[] regularstats =
            PlainLinearRegression
            (y, x, n);
    Pslope=regularstats[0];
    Prsq=regularstats[1];
    Pyint=regularstats[2];
    PF=regularstats[3];
    Pstderr=regularstats[4];
}

    /**
     * Finds a compensated scaling relationship from box counting
     * data by compressing periods
     * of horizontal slope to the smallest box at which the
     * interval started. This is done
     * because FracLac does box counting
     * without prior knowledge of the scaling
     * in a pattern and uses a linear series.
     * After removing areas of horizontal slope, returns an array of
     * {@link #FindDfStats fractal dimension information} for the
     * negative slope of the regression line (box count fractal
     * dimension), y intercept, and correlation coefficient
     * from a log-log plot of the corrected arrays. Calls an overloaded
     * version of {@link #LinearSmoothStats linearly smoothed method}
     * that accepts a float array
     * after converting the Count array of integers to floats.
     * @param intCount is the first integer array to assess
     * (e.g., of counts
     * of boxes containing pixels in a box count)
     * @param intSizes is an integer array, the second array to
     * assess in calculating regression line
     * @param oldNmSzs int for the number of entries to assess
     * in the passed arrays
     * @return float array with 3 entries for negative
     * slope of regression line (box count fractal dimension),
     * y intercept,
     * and correlation coefficient from regression line for
     * log-log plot of linearly smoothed arrays
     */
    public float[] LinearSmoothStats (int[] intCount, 
            float[] intSizes, int oldNmSzs)
    {/**converts counts to floats array then calls stats 
     * to make and use linearly smoothed array*/
        float []floatcount = new float [oldNmSzs];
        for (int w = 0; w<oldNmSzs; w++)
        {floatcount[w]=(float)intCount[w];}
        return LinearSmoothStats (floatcount, intSizes, oldNmSzs);
    }
    
     
    
    
/**Performs a linear smoothing of the relationship between testarray
 * and Sizes, for the first <i>oldNumSizes</i> and returns a 
 * float of statistics about the resultant arrays.  
 * Removes areas of horizontal slope from testarray versus Sizes 
 * by retaining from all  intervals
 * where testarray does not change, the corresponding smallest 
*  value in Sizes (i.e., the value in Sizes at the index that marks the 
 * start of the interval in testarray).  Returns an array of
 * {@link #FindDfStats stats} based on the smoothed set of arrays. 
 * Is usually called by an overloaded version of itself that received 
 * integer data. This function assumes that increases
* should be ignored, assuming that the smallest possible box
*at a count holds density most efficiently and that
*that is desirable.
* @param testarray is the first float 
* array to assess (e.g., of counts
* of boxes containing pixels in a box count)
* @param Sizes is an integer array, the second array to
* assess in calculating regression line
* @param n int for the number of entries to assess
* in the passed arrays
* @return float array from 
* {@link #FindDfStats FindDfStats method} 
accessible using DFSLOPEindex=0, RSQindex=1, 
    YINTindex=2, PREFACTORindex=3,
    STDERRindex=4*/
    public float[] LinearSmoothStats (float[] testarray,
            float[] Sizes, int n)
{
    //shrinks array then calls function to find stats 
    //calculates corrected fractal dimension without horizontal slopes
    //uses lowest box size in horizontal part
        smoothedNumBoxes = 1;
        float []c = new float [n];
        float []s = new float [n];
        s[0]=Sizes[0];
        c[0]=testarray[0];
        for (int q=1; q<n; q++)           
        {
            boolean proceed = false;            
            if (testarray[q]!=testarray[q-1])proceed=true;            
            if (proceed==true)
            {
                s[smoothedNumBoxes]=Sizes[q];
                c[smoothedNumBoxes]=testarray[q];
                smoothedNumBoxes++;
            }            
        }
        SmoothedSize=new float [smoothedNumBoxes];
        SmoothedCount = new float [smoothedNumBoxes];
        for (int i = 0; i < smoothedNumBoxes; i++)
        {
            SmoothedSize[i]=s[i];
            SmoothedCount[i]=c[i];
        }
        return 
 FindDfStats (SmoothedCount, SmoothedSize, smoothedNumBoxes);
        
    }
    
    /**Calculates the log-log plot of Counts and Sizes/Biggest Size.
     *See {@link #FindDfStats FindDfStats}
     *for further calculations.
     * @param Counts float array
     * @param Sizes integer array
     * @param NumSizes integer for number of values to use
     * @return float array with 5 {@link #CalculateStats statistics}
     *accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4
     */
    public float[] ratioFindDfStats (float[] Counts, float[] Sizes,
            int NumSizes)
    {  /*takes an array for box sizes
     and an array for Counts and
     *the number of box sizes to check 
     and calculates stats using those arrays*/
        float n=(float)NumSizes;
        float biggestSize = (float)Sizes[0];
        for (int q = 0; q<NumSizes; q++)
        {
            biggestSize = Math.max ((float)Sizes[q], biggestSize);
        }
        float DsumSC=0, DsumS=0, DsumC=0, 
                DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<NumSizes; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE LOG OF THE COUNT*THE LOG OF THE SIZE\
            ////use the scale factor, calculated as the
            ////box size divided by the largest size
            //That is, the maximum size is treated as 1 now
            DsumSC
            = DsumSC + (float)((float)Math.log (Sizes[q]/biggestSize)
            *(float)Math.log (Counts[q]));
            ///SUM THE LOG OF THE SIZE
            DsumS = DsumS 
            + (float)Math.log (Sizes[q]/biggestSize);
            //SUM THE LOG OF THE COUNTS
            DsumC = DsumC + (float)Math.log (Counts[q]);
            //SUM THE LOG OF COUNTS TIMES LOG OF COUNTS
            DsumCsq
            =DsumCsq+(float)((float)Math.log (Counts[q])
            *(float)Math.log (Counts[q]));
            //SUM THE LOG OF SIZE TIMES LOG SIZE
            DsumSsq
            = DsumSsq
            + (float)((float)Math.log (Sizes[q]/biggestSize)
            *(float)Math.log (Sizes[q]/biggestSize));
        }
        return CalculateStats (DsumSC,
        DsumS,
        DsumC,
        DsumSsq,
        DsumCsq,
        n);
        
        
    }
    
    /**
     * When called in fractal analysis, finds the fractal dimension
     * and related statistics from box counting data gathered from
     * a pattern.  The passed arrays are
     * usually the number or count of covering
     * objects at each size of covering objects that was required to
     * efficiently cover the pattern being analyzed.
     * The returned array holds power law
     * {@link #CalculateStats regression}
     * statistics typically used in fractal
     * analysis, calculated
     * from the log of the passed Counts array and the log of
     * the inverse
     * of the Sizes area (i.e., 1/Sizes). 
     * Thus, FindDfStats sums and transforms
     * the data and passes the 
     * {@link #CalculateStats regression} method the
     * following values:
     * <ul>
     * <LI>&#8721;SC = &#8721;
     * (ln (Sizes[q]<sup>-1</sup>)*ln Counts[q])</LI>
     * <LI>&#8721;S = &#8721;ln (Sizes[q]<sup>-1</sup>)</LI>
     * <LI>&#8721;C = &#8721;ln Counts[q]</LI>
     * <LI>&#8721;C<sup>2</sup> =
     * &#8721;ln (Counts[q]<sup>2</sup>)</LI>
     * <LI>&#8721;S<sup>2</sup> = 
     * &#8721;(ln (Sizes[q]<sup>-1</sup>)<sup>2</sup>)</LI>
     * </ul>
     *
     * @param Counts array of floats
     * @param Sizes array of integers; 1/Sizes[i] is used
     * @param NumSizes an integer for the number of values to use
     * from the passed arrays
     * @return float array with 5 {@link #CalculateStats statistics}
     accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4*/
    public float[] FindDfStats
            (float[] Counts, float[] Sizes, int NumSizes)
    {  //takes an array for box sizes and an array for Counts and
        //the number of box sizes to check and 
        //calculates stats using those arrays*/
        
        float n=(float)NumSizes;
        float DsumSC=0, DsumS=0, DsumC=0,
                DsumSsq = 0, DsumCsq=0;
        
        for (int q=0; q<NumSizes; q++)
        {//go through array and sum values for regression line*/
            
            /////SUM THE LOG OF THE COUNT*THE LOG OF THE SIZE\
            ////use the scale factor, calculated as the
            ////box size given divided by the largest size
            
            DsumSC = DsumSC
            + ((float)(Math.log (1.0f/Sizes[q])
            *(float)Math.log (Counts[q])));
            
            ///SUM THE LOG OF THE SIZE
            
            DsumS = DsumS + (float)Math.log (1.0f/Sizes[q]);
            
            //SUM THE LOG OF THE COUNTS
            
            DsumC = DsumC + (float)Math.log (Counts[q]);
            
            //SUM THE LOG OF COUNTS TIMES LOG OF COUNTS
            
            DsumCsq
            =DsumCsq+
                    ((float)Math.log (Counts[q])*(float)Math.log (Counts[q]));
            
            //SUM THE LOG OF SIZE TIMES LOG SIZE
            
            DsumSsq = DsumSsq
            +((float)
            (Math.log (1.0f/Sizes[q])*(float)Math.log (1.0f/Sizes[q])));
        }
        
        return CalculateStats (
        DsumSC,
        DsumS,
        DsumC,
        DsumSsq,
        DsumCsq,
        n);
        
        
    }
    
    /**Returns an array of floats that are calculated using
     *{@link #CalculateStats statistical calculator}.
     *Calculates values to pass to this function as follows:
     *<li>DSumSC =&#8721;<sub>0 
     * to NumSizes</sub>[log(Sizes)*log(Counts)]
     *<li>DSumC=&#8721;<sub>0
     * to NumSizes</sub>[log(Counts)]
     *<li>DSumS=&#8721;<sub>0 to NumSizes</sub>[log(Sizes)]  
     *<li>DSumCsq = &#8721;<sub>0
     * to NumSizes</sub>[log(Counts)*log(Counts)]
     *<li>DSumSsq=&#8721;<sub>0 
     * to NumSizes</sub>[log(Sizes)*log(Sizes)]
     * @param Counts float [] of values
     * @param Sizes float [] of values
     * @param NumSizes int for elements to use in arrays
     * @return float [] of values for the slope or fractal dimension
     *accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4
     */
    public float[] FindLogLogRegressionStats (float[] Counts, 
            float[] Sizes, int NumSizes)
    {  //takes an array for box sizes and an array for Counts and
        //the number of box sizes to check and 
        //calculates stats using those arrays*/
        
        float n=(float)NumSizes;
        float DsumSC=0, DsumS=0, DsumC=0,
                DsumSsq = 0, DsumCsq=0;
        
        for (int q=0; q<NumSizes; q++)
        {//go through array and sum values for regression line*/
            
            /////SUM THE LOG OF THE COUNT*THE LOG OF THE SIZE\
            ////use the scale factor, calculated as the
            ////box size given divided by the largest size
            
            DsumSC = DsumSC
            + (float)((float)Math.log (Sizes[q])
            *(float)Math.log (Counts[q]));
            
            ///SUM THE LOG OF THE SIZE
            
            DsumS = DsumS + (float)Math.log (Sizes[q]);
            
            //SUM THE LOG OF THE COUNTS
            
            DsumC = DsumC + (float)Math.log (Counts[q]);
            
            //SUM THE LOG OF COUNTS TIMES LOG OF COUNTS
            
            DsumCsq
            =DsumCsq+
                    ((float)Math.log (Counts[q])*(float)Math.log (Counts[q]));
            
            //SUM THE LOG OF SIZE TIMES LOG SIZE
            
            DsumSsq = DsumSsq
            +((float)Math.log (Sizes[q])*(float)Math.log (Sizes[q]));
        }
        
        return CalculateStats (
        DsumSC,
        DsumS,
        DsumC,
        DsumSsq,
        DsumCsq,
        n);
        
        
    }
   /**Returns the slope of the 
    * {@link #PowerRegression power law regression}
    * line from the log of Y versus the 
    *log of X.
    *<LI>y=Ax<sup>slope</sup></LI>
     @param Y float [] of y-values for the regression
     *@param X float [] of x-values for the regression
     *@param NumSizes int for the number of 
    * entries to use in the passed X and Y
     *@return float Slope of the regression line*/ 
   public float SlopeOfPowerRegression(float[] Y, float[] X,
           int NumSizes)
   {
       float [] V=PowerRegression(Y, X, NumSizes);
       return V[0];
   }
   
   
    /**Returns the y-intercept of the 
     * {@link #PowerRegression power law regression} line
     * from the log of Y versus the 
    *log of X
      @param Y float [] of y-values for the regression
     *@param X float [] of x-values for the regression
     *@param NumSizes int for the number of entries
     * to use in the passed X and Y
     *@return float Y-intercept for the power regression*/   
   public float YInterceptOfPowerRegression(float[] Y,
           float[] X, int NumSizes)
   {
       float [] V=PowerRegression(Y, X, NumSizes);
       return V[2];
   }
     /**Returns the correlation of the 
      * {@link #PowerRegression power law regression} line
      * from the log of Y versus the 
    *log of X.
       @param Y float [] of y-values for the regression
     *@param X float [] of x-values for the regression
     *@param NumSizes int for the number of entries
      * to use in the passed X and Y
     *@return float r<sup>2</sup> correlation coefficient*/ 
   public float RSquaredOfPowerRegression(float[] Y,
           float[] X, int NumSizes)
   {
       float [] V=PowerRegression(Y, X, NumSizes);
       return V[1];
   }
    /**Returns the Standard Error of the 
     * {@link #PowerRegression power law regression} line
     * from the log of Y versus the 
    *log of X
     @param Y float [] of y-values for the regression
     *@param X float [] of x-values for the regression
     *@param NumSizes int for the number of entries
     * to use in the passed X and Y
     *@return float standard error for regression line
     */ 
   public float StandardErrorOfPowerRegression(float[] Y, 
           float[] X, int NumSizes)
   {
       float [] V=PowerRegression(Y, X, NumSizes);
       return V[4];
   }
   /**Returns the prefactor A=Euler's e^y-intercept of the 
    * {@link #PowerRegression power law regression} line
    * from the log of Y versus the 
    *log of X. 
    * <LI> Note that A=<i>e</i><sup>y-intercept</sup>
    * and y=Ax<sup>slope</sup>
     @param Y float [] of y-values for the regression
     *@param X float [] of x-values for the regression
     *@param NumSizes int for the number of entries
    * to use in the passed X and Y
     *@return float PreFactor A for the regression line
    * (y=Ax<sup>slope</sup>*/ 
   public float PreFactorOfPowerRegression(float[] Y,
           float[] X, int NumSizes)
   {
       float [] V=PowerRegression(Y, X, NumSizes);
       return V[3];
   }
/**Returns {@link #CalculateStats stats}
* array of power law regression data, using
* log of Counts (Y) array and log of Sizes (X) array.
*
* @param Counts array of floats
* @param Sizes array of integers
* @param NumSizes an integer for the number of values to use
* from the passed arrays
* @return float array with 5 {@link #CalculateStats statistics}
0=slope, 1=r squared, 2=y intercept, 3=prefactor, 4=stderr
* accessible using DFSLOPEindex=0, RSQindex=1, 
    YINTindex=2, PREFACTORindex=3,
    STDERRindex=4.*/
public float[] PowerRegression (float[] Counts, float[] Sizes,
            int NumSizes)
{  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and 
     calculates stats using those arrays*/
        float n=(float)NumSizes;
        float DsumSC=0, DsumS=0, DsumC=0, 
                DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<NumSizes; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE LOG OF THE COUNT*THE LOG OF THE SIZE\
            DsumSC
            = DsumSC
            + ((float)Math.log (Sizes[q])*((float)Math.log (Counts[q])));
            ///SUM THE LOG OF THE SIZE
            DsumS = DsumS + ((float)Math.log (Sizes[q]));
            //SUM THE LOG OF THE COUNTS
            DsumC = DsumC + ((float)Math.log (Counts[q]));
            //SUM THE LOG OF COUNTS TIMES LOG OF COUNTS
            DsumCsq
            =DsumCsq+(float)((float)Math.log (Counts[q])
            *(float)Math.log (Counts[q]));
            //SUM THE LOG OF SIZE TIMES LOG SIZE
            DsumSsq
            = DsumSsq +
                    (float)((float)Math.log
                    (Sizes[q])*((float)Math.log (Sizes[q])));
        }
        return CalculateStats (DsumSC,
        DsumS,
        DsumC,
        DsumSsq,DsumCsq,
        n);
        
        
    }
    
    /****************************************************************/
    /** Converts to log base 10.
     * @param number float to convert
     * @return float log base 10 of number*/
    public float log10 (float number)
    {
        return ((float)Math.log (number)/(float)Math.log (10.0f));
    }
    
    /** Converts to log base 10.
     * @param number int to convert
     * @return float log base 10 of number*/
    public float log10 (int number)
    {
        return (float)Math.log ((float)number)/(float)Math.log (10.0f);
    }
    
    /** Converts to log base 10.
     * @param       number float to convert
     * @return          float log base 10 of number*/
    public float log10 (double number)
    {
        return (float)Math.log ((float)number)/(float)Math.log (10.0f);
    }
    
    /**Returns array of statistics. Operates on the inverse of Sizes
     * array.
     * @param Counts float array for x axis (e.g., number of boxes that
     * contained pixels in fractal analysis)
     * @param Sizes float array for y axis (e.g., the corresponding
     * array of box sizes)
     * @param NumSizes integer for the number of elements to
     * use in both arrays
     * @return float array with 5 {@link #CalculateStats statistics}
     *accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4
     */
    public float[] InvLinearRegression (float[] Counts, 
            float[] Sizes, int NumSizes)
    {  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and calculates
     stats using those arrays*/
        float n=(float)NumSizes;
        float DsumSC=0, 
                DsumS=0, DsumC=0, DsumSsq = 0, DsumCsq=0;
        for (int q=0; q<NumSizes; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE COUNT*THE SIZE
            DsumSC
            = DsumSC + (float)((1.0f/Sizes[q])*Counts[q]);
            ///SUM THE SIZE
            DsumS = DsumS + (float)(1.0f/Sizes[q]);
            //SUM THE COUNTS
            DsumC = DsumC + Counts[q];
            //SUM THE COUNTS TIMES COUNTS
            DsumCsq
            =DsumCsq+((Counts[q])*(Counts[q]));
            //SUM THE SIZE TIMES SIZE
            DsumSsq
            = DsumSsq + (float)((1.0f/Sizes[q])*(1.0f/Sizes[q]));
        }
        return CalculateStats (DsumSC,
        DsumS,
        DsumC,
        DsumSsq,
        DsumCsq,
        n);
    }
    
    /***************************************************************/
    /**Returns array of regression statistics. 
     * Operates on the passed arrays.
     *
     * @param        Y float array for x axis (e.g., count of boxes that
     * contained pixels in fractal analysis)
     * @param       X float array for y axis (e.g., the corresponding
     * array of box sizes)
     * @param        Num integer for the number of elements to
     * use in both arrays
     * @return           float array with 5 
     * {@link #CalculateStats statistics}
     * accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4
     */
    public float[] PlainLinearRegression (float[] Y, float[] X, int Num)
    {  /*takes an array for box sizes and an array for Counts and
     *the number of box sizes to check and 
     calculates stats using those arrays*/
        float n=(float)Num;
        PlainsumSC=0;
        PlainsumS=0;
        PlainsumC=0;
        PlainsumSsq = 0;
        PlainsumCsq=0;
        for (int q=0; q<Num; q++)
        {/*go through array and sum values for regression line*/
            /////SUM THE COUNT*THE SIZE
            PlainsumSC
            = PlainsumSC + (X[q]*Y[q]);
            ///SUM THE SIZE
            PlainsumS = PlainsumS + (X[q]);
            //SUM THE COUNTS
            PlainsumC = PlainsumC + (Y[q]);
            //SUM THE COUNTS TIMES COUNTS
            PlainsumCsq
            =PlainsumCsq+(Y[q]*Y[q]);
            //SUM THE SIZE TIMES SIZE
            PlainsumSsq
            = PlainsumSsq + (X[q]*X[q]);
        }
        
        return CalculateStats (PlainsumSC,
        PlainsumS,
        PlainsumC,
        PlainsumSsq,
        PlainsumCsq,
        n);
    }
    
    /***************************************************************/
    /**Returns an array holding the slope of the regression line,
     * the r<sup>2</sup>,
     * the y-intercept, the prefactor<sup>-1</sup> (prefactor is A
     * in y=Ax<sup>b</sup>) for power
     * law regressions, and the standard error
     * for a regression line calculated from the passed values.
     * This method is called by this class's other methods that
     * prepare (for example, transform and sum)the original arrays.
     *
     *<UL>
     * <LI>
     * CALCULATE THE SLOPE OF THE REGRESSION LINE
        <li>
     *  b=
        ((n*&#8721;SC)-(&#8721;S*&#8721;C))
        /((n*&#8721;S<sup>2</sup>)-(&#8721;S*&#8721;S));
        </li>
        </LI>
        <li>CALCULATE CORRELATION 
     * COEFFICIENT OF REGRESSION LINE
       <li>correl= (n*(&#8721;SC)-(&#8721;S*&#8721;C))
        /&#8730;
     * (((n*&#8721;S<sup>2</sup>)-(&#8721;S*&#8721;S))*
        *((n*&#8721;C<sup>2</sup>)-((&#8721;C*&#8721;C))));
       </li>
     * <li>correl= correl<sup>2</sup>;
     *</li></LI>
        
        <LI>CALCULATE Y INTERCEPT OF REGRESSION LINE
        <li>yint
        = (&#8721;C-(b*&#8721;S))/n
       </li></LI>
     <LI>CALCULATE THE PREFACTOR FOR
     * THE REGRESSION LINE
       <li>A=1/e<sup>y-intercept</sup></li>
     * <li>
        for a power law: y=Ax<sup>b</sup></LI>
        </LI></li>
     <LI>CALCULATE STANDARD ERROR
     *<li>se=
     * &#8730; ((&#8721;C<sup>2</sup>
     * -y-intercept*&#8721;C-b*&#8721;SC)/(n-2))
     *</li></li></UL>
     *@see      #FindDfStats
     *@param        DsumSC float for sum of all S times C array
     *@param        DsumS float for sum of all S array
     *@param        DsumC float for sum of all C array
     *@param        DsumSsq float for sum of all S<sup>2</sup>
     *@param        DsumCsq float for sum of all C<sup>2</sup>
     *@param        n float for number of values
     * that were summed in one array
     *@return          float array holding slope, 
     * correlation r<sup>2</sup>,
     *          y-intercept,
     *          prefactor's inverse, and standard error for regression line,
     * accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4.
     */
    public float [] CalculateStats (
    float DsumSC,
    float DsumS,
    float DsumC,
    float DsumSsq,
    float DsumCsq,
    float n)
    {
        float [] fivestats=new float[5];        
        
        //CALCULATE THE SLOPE OF THE REGRESSION LINE
        float b=
        ((n*DsumSC)-(DsumS*DsumC))
        /((n*DsumSsq)-(DsumS*DsumS));
        fivestats[DFSLOPEindex]=b;//-1*b;
        
        //CALCULATE CORRELATION 
        //COEFFICIENT OF REGRESSION LINE
        float correl=(n*(DsumSC)-(DsumS*DsumC))
        /(float)Math.sqrt (((n*DsumSsq)-(DsumS*DsumS))
        *((n*DsumCsq)-((DsumC*DsumC))));
        fivestats[RSQindex] = correl*correl;
        
        //CALCULATE Y INTERCEPT OF REGRESSION LINE
        float ayint
        = (DsumC-(b*DsumS))/n;
        /*y intercept for linear regression by least squares*/
        //System.out.println(ayint);
        fivestats[YINTindex]=ayint;
        //CALCULATE THE PREFACTOR FOR THE REGRESSION LINE
        //IF THIS WERE A POWER REGRESSION
        fivestats[PREFACTORindex]=1f/(float)(Math.exp (ayint));
//Prefactor A = e to the FractalStatsYint
        //for a power law: y=Ax^Df
        //standarderror
        fivestats[STDERRindex]=(float)Math.sqrt
                ((DsumCsq-ayint*DsumC-b*DsumSC)/(n-2f));
        
        return fivestats;
    }
   
    /**Returns a {@link #FindDfStats FindDfStats instance} from
     * integers arrays.   Calls FindDfStats for floats
     * after converting passed Counts array to floats.
     *
     * @param       Counts integer array
     * @param       Sizes integer array
     * @param       NumSizes integer for number of values to use
     * @return          float array of values based on passed arrays
     *accessible using DFSLOPEindex=0, RSQindex=1, 
            YINTindex=2, PREFACTORindex=3,
            STDERRindex=4
     */
    public float[] FindDfStats (int[] Counts, float[] Sizes, int NumSizes)
    {/*converter for int array instead of float array*/
        float[]c=new float[NumSizes];
        for (int w = 0; w<NumSizes; w++)  c[w] = (float)Counts[w];
        return FindDfStats (c,Sizes,NumSizes);
    }
    
}

