// Notice
//
// The contents of this file are subject to the
//  same license as the source code for ImageJ. 
//  FracLac is in the public domain.
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLUtilities;

import ij.IJ;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Map.Entry;

/**Methods for manipulating numbers and arrays in FracLac plugin.
 * @author  Audrey_Karperien
 */
public class FLutil {
    /**Commonly used string for tab character "\t".*/
    public static String tab = "\t";
    
    /**Constructor creates a new instance of util but takes
     * no parameters.*/
    public FLutil() {    }
     /**
     * Returns an array containing the mean of all arrays at each 
      * index.  When called after a fractal box count, seeks a 
      * representative covering from a sample of coverings laid over a
      * 2-dimensional pattern. The arrays passed should be one array
      * for each each location, each holding at the position 
      * corresponding to the position in the original list of box sizes, 
      * one value.  That value is the number of covering objects at that
      * &epsilon;. The returned array holds the mean number of 
      * objects needed to cover the pattern at each &epsilon; 
      * (covering box size), from all the scans. For other applications, 
      * this is a float array that is the average value of each position in
      * the passed array of arrays. <p>To finish the analysis, pass the
      * results to a {@link FLAnalyzer.FracStats#FindDfStats fractal 
      * dimension} class. </p> <p>The passed ints control how many 
      * arrays, and how many &epsilon;s in them, are used to find the
      * mean, starting from index 0 in each array.   The number of 
      * &epsilon;s must be the same for all arrays and all the arrays 
      * must have at least as  many members as the numBoxes or an
      * OutOfBounds Exception occurs. </p>
     * @param   TestArray array of arrays of integers to be averaged
     *          at each position
     * @param   Boxes the number of values in the arrays to use
     * @param   Locs the number of arrays to assess
     * @return  a float array of the average at each position over all 
      * the arrays  */
public static float [] MeanMatchedArray(float [][] TestArray,
        int Locs, int Boxes) 
{
    float  avcover[]= newArray (Boxes, 0.0f);
    for (int j = 0; j<Locs; j++) 
    {
        for (int i = 0; i < Boxes; i++) 
        {
            avcover[i] =  avcover[i] + TestArray[j][i];
        }
    }        
    for (int i = 0; i < Boxes; i++) 
    {
        avcover[i] =  avcover[i]/(float)Locs;
    }        
    return avcover;
}
    
/**Returns an array of means for all elements at the index in array 
 * A[g][] that match a condition.  The condition is that they must 
 *be at the index where a value from the unique sizes array is found 
 *in the sizes array.  The first array in the returned 2d array is 
 the set of unique sizes, the following array is the mean at each size.*/
public static BoxCountHolder MeanOfUnMatchedArrays
        (float [][] T, int [][] B)
{
    int G = T.length;
    float a [][] = new float [G][];
    int U [] = GetUniqueValues (B);
    int [] c = newArray(U.length, 0);
    for (int i = 0; i < G; i++)a[i]=newArray(U.length,0f);
   for (int g = 0; g <G; g++)
   {
        for (int u = 0 ; u<U.length; u++)
        {
            for (int b=0; b < B[g].length; b++)
                if (U[u]==B[g][b])
                    a[g][u]=T[g][b];
        }
   }
    //  NOW a HOLDS THE SUM OF EACH VALUE IN T THAT 
    // CORRESPONDS TO A UNIQUE B:  FIND THE MEAN
    float [] m= new float [U.length];
    for (int g = 0; g < G; g++)
    {
        for (int i = 0; i < U.length; i++)    
        {        
            m[i]=m[i]+a[g][i];
            if(a[g][i]>=1)c[i]++;      
        }        
    }
    for (int i = 0; i < c.length; i++)m[i]=m[i]/(float)c[i];
            
    return new BoxCountHolder(U, m);  
}
 
/**Returns the passed array with 1 added to each element* 
* @param array float [][] to +1
* @return float [][] of the passed array +1 
*/
public static float[][] Plus1 (float [][] array)
     {
         float [][] f = new float [array.length] [];
         for (int i= 0; i<array.length; i++)
         {
             f[i]=new float [array[i].length];
             for (int j = 0; j < array[i].length; j++)
             f[i][j]=array[i][j]+1.0f;
         }
         return f;
     }
/**Adds 1 to each element of the passed array and
*returns a new array with the value added.
* @param array  float [] array of floats to add to
* @return float [] the new array having
* the new value at each position up to "length"*/
public static float[] Plus1 (float [] array)
 {
     int length = array.length;
     float [] f = new float [length];
     for (int i = 0; i < length; i++)  f[i]=array[i]+1.0f;
     return f;
 }  

/**Returns the number of boxes of size that will fit across width for a
 * sliding box lacunarity scan when moved slide each move.
*@param width int for the total distance
*@param size int for the sliding box's size
*@param slide int for the distance to slide the box each time
*@return float the number of times it will go*/
public float NumBoxesInASlideAcrossOrDown
       (float width, float  size, float slide)
{
    return 1+(int)Math.ceil ((width-size)/slide);
}

/**Returns a one dimensional array that has at each element the 
 * lowest value at that position from all of the 1d arrays in the passed
 * 2d array.  Only the first <i>numBoxes</i> elements of the 1d 
 * arrays in the passed 2d array are assesssed.  All of the  arrays  in 
 * the 2d array must be at least numBoxes long. When called after a 
 * fractal box count scan over multiple origins, seeks the set of 
 * coverings that most efficiently covered an object in a plane.  Takes
 * an array of arrays, one array at each scan location holding the 
 * number of coverings used for each object size.  That is, the arrays
 *passed should be one array for each location, each holding at the 
 * position corresponding to the position in the original list of box 
 * sizes, one value.  That value is the number of covering objects at 
 * that &epsilon;. Returns a one-dimensional array holding one value
 * for each &epsilon;,equal to the minimum number, drawn from all 
 * the locations, of covering objects at that &epsilon;. 
* <p>To finish the analysis, pass the results to a {@link 
 * FLAnalyzer.FracStats#FindDfStats fractal dimension} class.</p>
 *<p>The passed integers control how many sizes to use,  starting 
 * from index 0, in each array, but the number of &epsilon;s  must 
 * be the same for all arrays and all the arrays must have at least  as
 * many members as the numBoxes or an OutOfBounds Exception 
 * occurs. </p>
* @param   TestArray integer array of arrays, one for each location,
 *that is as long as the number of sizes
 * @param   numLocs integer for the number of location array to use
 * @param   numBoxes integer for the number of counts from each 
 * location array to use
 * @return  mcover integer array holding the lowest value at each 
 * size*/

public static float [] findMinCover(float [][] TestArray, int numLocs,
        int numBoxes) 
{
    float  mcover[]= new float[numBoxes];
    for (int a = 0; a < numBoxes; a++)mcover[a] = TestArray[0][a];
    for (int j = 0; j<numLocs; j++) 
    {
        for (int i = 0; i < numBoxes; i++) 
        {
            if (TestArray[j][i]<mcover[i]) mcover[i] = TestArray[j][i];
        }
    }
    return mcover;
}

/**Goes through a 3d array and returns an array synthesized
*from the indices that had the lowest count.  First determines
*the length of the first array at [0][0], compares it to the length
* of [1][0], saves the index of the shorter one, compares that
*length to [2][0], etc.  For each [g][][], finds the shortest [g][b],
*and stores the array at that position.  The result is an array 
*of "minimum cover" for box counting.  If two arrays have the 
*same length at an index, then the array with the lower standard
*deviation is selected.
@param a [][][] float array to minimize
@param es int for the number of elements at a[][e][] to assess
- that is, all of the arrays must have at least es elements or an 
*out of bounds exception is thrown*/
public static float [][] minMassCover(float a [][][], int es)
{
float [][] m=new float [es][];      
int g = 0;
for (int b=0; b < es; b++)
{m[b]=a[0][b];
    for (g = 0; g < a.length; g++)
    {

        if(a[g][b].length<=m[b].length)
        {

            if(a[g][b].length<m[b].length)
            {
                m[b]=a[g][b];

            }
            else if (a[g][b].length==m[b].length)
            {
                float s=FLstatistics.Variance (m[b], m[b].length);
                float s2=FLstatistics.Variance(a[g][b], a[g][b].length);
                if(s2<s)
                {
                    m[b]=a[g][b];
                    //System.out.println("std dev "+s2 +" < "+s);

                }                       
            }
        }
    }               
}
return m;

} 

/**Returns L if L contains m, or else adds m at the next position
 at the end of L and returns the new array.*/
static int [] ListContains(int m, int [] L)
{
int index=L.length;
for (int i=0; i < L.length; i++) if (L[i]==m) return L;        
int [] n = new int [index+1];
for (int i =0; i<L.length; i++) n[i]=L[i];
n[n.length-1]=m;
return n;       
}

/**Returns the unique numeric values in the passed 
 array, in ascending order.
 *@see #ListContains
 *@see #Sort
 @param B int [][] of values
 @return int [] of unique values from all arrays*/
public static int [] GetUniqueValues(int B[][])
{            
int [] L = new int [1];
L[0]=B[0][0];
for (int i  = 0; i < B.length; i++)
{
  for (int j = 0; j < B[i].length; j++)
        L=ListContains(B[i][j], L);
}
int []m = Sort
     (L);
return m;
}

/**Returns the passed array sorted ascendingly.
 @see #get2ndHighest
 @see #maxArray*/
static int [] Sort(int [] a)
{
int num=a.length;
int [] b = new int [num];
b[num-1]=maxArray(a);
for (int i =num-2; i >=0; i--)
{
b[i]=get2ndHighest(b[i+1], a);
}        
return b;
}

/**Returns the next highest value in the array compared to the 
 passed value of <i>highest</i>.
 @param highest int to compare
 @param array [] int array to select the next value from*/
public static int get2ndHighest(int highest, int [] array)
{    
    int n = array[0];
    for (int i = 0; i < array.length; i++)
        if (array[i]<highest){n=array[i];i=array.length;}
    for (int i=0; i < array.length; i++)
    {      
        if (array[i]<highest) {n=(int)Math.max(n, array[i]);}        
    }
    return n;
}

/**Copies the <i>N</i> elements of a float array, <i>f</i>.*/   
public static float [] copyFloat (float[] f, int N)
{
    float [] g = new float[N];
    for (int i = 0; i < N; i++) g[i]=f[i];
    return g;
}
/**Copies the <i>N</i> elements of a float array, <i>f</i>.*/   
public static int [] copyInt (int[] f, int N)
{
    int [] g = new int[N];
    for (int i = 0; i < N; i++) g[i]=f[i];
    return g;
}
/**Goes through a 3d array and returns an array synthesized
*from the indices that had the lowest count at a particular 
* &epsilon;.  The arrays do not all have to have the same number 
* of  &epsilon;s. First makes a list of all unique values for box size, 
*then checks for that value in the &epsilon; position for 
* all of the arrays, and returns the mass array that is shortest.
* If two mass arrays at an &epsilon; are the same length, 
* chooses the one with the lower standard deviation.
@param a float array [grid][&epsilon;][mass] to minimize
@param B [][] int array of &epsilon;s holding the corresponding  
&epsilon; for a[grid][&epsilon;]
 @return BoxCountHolder an array holding an array of  box sizes
 *and one of corresponding masses*/
public static BoxCountHolder minCover (
float a [][][], int [][] B, boolean MultipleBSets)
{        
   int grids=a.length;
   if (grids!=B.length)IJ.showMessage ("Error in array");
   int [] U = MultipleBSets ? GetUniqueValues(B) : 
       copyInt (B[0], B[0].length);
   float [][] mins =new float [U.length][];
    for (int u =0; u < U.length; u++)
    {
        boolean did=false;
        int boxsize = U[u];
        for (int g =0; g < grids; g++)
        {            
            for (int b=0; b < B[g].length; b++)    
            {
            //if there is a B that matches that boxsize, 
            //use its grid and box indices to find the 
            //corresponding array of masses in a
            if (B[g][b]==boxsize) 
            {
                mins[u]=a[g][b];
                b=B[g].length;
                did=true;
            }         
            }
            if (did)g=grids;
        }  
        //get a value from a that corresponds to an index 
        //from B that is stored in each U
    }
    //initialize mins
    for (int u=0; u<U.length; u++)
    {
        for (int g = 0; g <B.length; g++)
        {
            for (int b = 0; b <B[g].length; b++)
             {
                if (B[g][b]==U[u])
                {
                    if(a[g][b].length<mins[u].length)  
                        mins[u]=a[g][b];                    
                    else if(a[g][b].length==mins[u].length)
                    {
                        float s=FLstatistics.Variance (mins[u], mins[u].length);
                        float s2=FLstatistics.Variance(a[g][b], a[g][b].length);
                        if(s2<s)mins[u]=a[g][b];
                    }
                }
            }
        }
    }
    
    return new BoxCountHolder(U, mins);
} 


/**Performs a linear smoothing on the relationship between the 
 * length of each array in masses and boxsizes.  It is important to 
 * note that the smoothing is not for the arrays in the masses 
 * array, but for their lengths only.  Returns an array of arrays that 
 * holds at [0] a contracted series 
* from the passed array for boxsizes, and for the rest of the array a 
* contracted series based on the passed masses [][].  The 
* contractions are made whenever the masses array has two 
* consecutive entries of the same length (regardless of the values 
* themselves).  Starting from index 0, if two consecutive arrays have
* the same number elements, only the first one (which is assumed to
* be the smaller box size) is retained.  At the same time, only the 
* corresponding element in the boxsizes array is retained. In fractal 
* analysis, this function removes areas of horizontal slope from pixel
* count data. For example, if masses = {{3,4,3}, {2,3,1}, {2,78}, 
* {3,3,3,3},{4,4,4,4}}, then the returned array will lose the entry 
* after {3,4,3} because both have 3 elements.  {4,4,4,4} will also be
* lost, so the result will be {{3,4,3}, {2,78}, {3,3,3,3}}.  If the 
* passed boxsizes array was {1,2,3,4,5}, the returned boxsizes array
* (stored at index [0][]) will be {1,3,4}.To use the returned data: 
* <pre> 
* float [][]B=SmoothedArray
* (masses, BOXSIZES, N);  
* float [][] M=new float [B.length-1][];
* for (int j=0; j<M.length; j++)M[j]=B[j+1]; 
* //now B[0] is the new boxsizes, and M is the new masses for B. 
* </pre>
* @param masses  is the array to assess (e.g., of counts of boxes 
* containing pixels in a box count)
* @param boxsizes is the integer array of box sizes
* @return 2d array of masses with new box size array*/
public static BoxCountHolder SmoothedArray 
        (float[][] masses, int[] boxsizes) 
{
    int N=boxsizes.length;
    float [][]m = new float [N][];
    int []s = new int [N];
    s[0]=boxsizes[0];
    m[0]=masses[0];
    int d = 1;
    int lastcheck=m[0].length;
    for (int q=1; q<N; q++)            
    {
        boolean proceed = true;            
        if (masses[q].length==m[d-1].length)proceed=false;
        if (proceed) 
        {
            s[d]=boxsizes[q];
            m[d]=masses[q];
            d++;
        }
    }    
    
   BoxCountHolder jh =new BoxCountHolder();   
   jh.masses=new float [d+1][];
   jh.boxes=new int [d];
    for (int j=0; j < d; j++)
    {
       jh.boxes[j]=s[j];
       jh.masses[j]=m[j];
    }
    return jh;

   
}
    


/**Tests the functioning of the findMinCover and findAvgCover
* methods.
* @param   args String array of arguments; is a dud
*          because this class is only for testing
*/
public static void main(String args[])
{
       
//    int [][] B = {
//        {2,3,5},{2,3,4,5},{3,5}
//    }; 
//    float [][] T = {
//        {1,12,6},{1,2,4,3},{5,9}};
//    
//    float [][] m = MeanOfUnMatchedArrays (T, B);
//    for (int i = 0; i < m[0].length; i++) System.out.print(m[0][i]+"  ");
//    System.out.println();
//     for (int i = 0; i < m[1].length; i++) System.out.print(m[1][i]+"  ");
//    
    testminCover();
   
}
static void testminCover(){
   int [][] B = 
{{33, 4,3}, 
{2,3,3}, 
{2,78}, 
{3,3,3,3} };
float [][][]a = 
{
{{1,1,1}, {1,1,1,1,2}, {2,2}}, 
{{2,4}, {2,2,2,2,2}, {2,3}},
{{5,6}, {2,2,4,2,2}},
{{5,6}, {2,2,4,2,2}, {2,3}, {3,45}}
};
BoxCountHolder c =minCover (a, B, true);
int bs [] =c.boxes; 
float ms [] [] =c.masses;
for (int k = 0; k<bs.length; k++)
{
 System.out.print("size "+bs[k]+": " +               tab);    
 for (int i = 0; i< ms[k].length; i++)
     System.out.print(tab+" "+ms[k][i]);
 System.out.println();
}
}    
static void test()
{
float [][] A = {
{4, 1, 2, 3, 5, 6, 3}, 
{5, 4, 2, 1, 2, 3, 4}};    
float []B = findMinCover(A, A.length, A[0].length);
for (int i = 0; i< B.length; i++) System.out.print(B[i]+",  ");
System.out.println("Should be 4,1,2,1,2,3,3");

float [][] D = {{1, 1, 1, 1, 1, 1, 1}, {4,5,6,7,8,9,10}};       
float [] C = MeanMatchedArray(D, D.length, D[0].length );
for (int i = 0; i< C.length; i++) System.out.print(C[i]+",  ");
System.out.println("should be 2.5, 3, 3.5, 4, 4.5, 5, 5.5");

float [][] g = {{2f,5f,9f}, {.5f, .3f, .2f}};
String [] S=MassAndProbabilityStrings (g);
for (int i = 0; i< S.length; i++)  System.out.println(S[i]+"  ");
System.out.println(); 

float [][][]a = 
{
{{1,1,1}, {1,1,1,1,2}, {2,2}}, 
{{2,4}, {2,2,2,2,2}, {2,3}},
{{5,6}, {2,2,4,2,2}, {2,3}}
};
int es =a[0].length;
float b [][]=minMassCover (a, es); 

System.out.println(FLstatistics.Variance (a[1][0], 2));
System.out.println(FLstatistics.Variance (a[2][0], 2));
for (int h=0; h < b.length; h++)
{ System.out.println();
for (int i =0; i < b[h].length; i++)
{System.out.print(b[h][i]+"   ");}
}
}
    
 
/**Utility creates and initializes an array using the passed 
* parameters.
* @param   length int for the length of the new array
* @param   value float for the value to
* initialize the array members to
* @return  float array of passed length, initialed to passed value*/
public static float[] newArray(int length, float value) 
{
        float [] Array=new float [length];
        for (int i =0; i < length; i++)
            Array[i]=value;
        return Array;
}
/**Returns an array of size <i>length</i> initialized to <i>value
 * </i> at each element.
 * @param length int for first elements of array to process
 * @param value double to initialize array
 * @return double [] of value */
 public static double[] newArray(int length, double value) 
 {
     double [] Array=new double [length];
     for (int i =0; i < length; i++) Array[i]=value;
     return Array;
}
    

    
/** Creates and initialized an array using the passed parameters.
 *@param   length int for length of new array
 * @param   value int for value to initialize new array elements to
 * @return  int array of passed length, initialized to passed value*/
 public static int[] newArray(int length, int value) 
 {
     int [] Array=new int [length];
     for (int i =0; i < length; i++) Array[i]=value;
     return Array;
 }    
    
/** Returns a new 2dimensional array initialized to the passed value,
* and of the passed dimensions.
* @param   length1 int for length of the first dimension
* @param   length2 int for length of the second dimension
* @param   value double value to initialize elements to
* @return  double[][] initialized to passed value, 
* of passed dimensions */
public static double[][] 
    new2dArray(int length1, int length2, double value) 
{
    double [][] Array=new double [length1][length2];
    for (int i =0; i < length1; i++) 
    {
        for (int j = 0; j<length2; j++) Array[i][j]=value;
    }
    return Array;
}
    
/**Returns a 2 dimensional array of size [length1][length2], 
 * initialized to <i>value</i>.
 * @param length1 dimension of 1st degree of array
 * @param length2 dimension of second degree of array
 * @param value value to initialize array elements to 
 * @return float [][] initialized to the passed value */
 public static float[][] new2dArray (int length1, 
         int length2, float value) 
 {
     float [][] Array=new float [length1][length2];        
     for (int i =0; i < length1; i++) 
     {
            for (int j = 0; j<length2; j++) Array[i][j]=value;
        }
     return Array;
 }
    
  public static int countGreaterThan(float []a,int length, int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]>b)c++;
    return c;
}
public static int countLessThan(float []a, int length,int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]<b)c++;
    return c;
}  
 public static  int countGreaterOrEqual(float []a,int length, int b)
{
    int c = 0;
    for (int i = 0; i < length;i++)
        if (a[i]>=b)c++;
    return c;
}
public static int countLessOrEqual(float []a,int length, int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]<=b)c++;
    return c;
}  
public static int countGreaterThan(int []a,int length, int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]>b)c++;
    return c;
}
public static int countLessThan(int []a,int length, int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]<b)c++;
    return c;
}  
 public static  int countGreaterOrEqual(int []a, int length, int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]>=b)c++;
    return c;
}
public static int countLessOrEqual(int []a, int length, int b)
{
    int c = 0;
    for (int i = 0; i < length; i++)
        if (a[i]<=b)c++;
    return c;
}  
/** Returns a new 2dimensional array initialized to
* the passed value, and of the passed dimensions.
*
* @param   length1 int for length of first dimension
* @param   length2 int for length of second dimension
* @param   value int to initialize elements to
* @return  int[][] initialized to passed value, of passed dimensions
*/
public static int[][] new2dArray(int length1, 
    int length2, int value) {
int [][] Array=new int [length1][length2];

for (int i =0; i < length1; i++) {
    for (int j = 0; j<length2; j++) Array[i][j]=value;
}
return Array;
}
/** Returns a float array holding the mean, standard deviation,
* coefficient of variation, and lacunarity stat for the probability
* that is the passed array, which does not specify the value
* of each probability.  This method assumes that the value is the
* same as the position in the array.
*
* <ul><LI>The mean of the probability distribution
* is: &#8721;(i*mprob[i])
* where i is the index and value, and mprob[i] is the weight</LI>
* <LI>the std dev of the probability distribution is:
* &#8730;
* &#8721;(((mprob[i]-meanProb)*(mprob[i]-meanProb))*i);</LI>
* <LI>the Probability Density lacunarity at this epsilon is:
* (&#8721;M<sup>2</sup>-
* (&#8721;M*&#8721;M))/(&#8721;M*&#8721;M);
* </LI><LI>where &#8721;M=&#8721;(i*mprob[i])
* and &#8721;M<sup>2</sup> =
* &#8721;(i*i*mprob[i]);</LI></ul>
*
*
* @return  float array of mean, stddev, cv, and lacunarity
*          <LI>[0]=meanProb</LI><LI>
*          [1]=&#8730;(variance) =standard deviation
* </LI><LI>
*          [2]=&#8730;(variance)/meanProb
* =coefficient of variation
* </LI><LI>[3]=the variance/the mean squared = lacunarity-1
*          (&#8721;Msq-(meanProbability*meanProbability))
*          /(meanProbability*meanProbability)</LI>
*
* @param   mprob array of floats as a probability distribution
* @param   LOC integer for this location
*/
public static float [] probmass(float [] mprob, int LOC) {
float[] stuff=new float[4];
if (mprob==null);
else {
    //int numberOfBoxesWithPixels=boxes;
    float meanProbability = 0;
    float sumMsq = 0;
    for (int i = 0; i < mprob.length; i++) {
//calculate moments as per chap. 8.4.10 , p. 238,  Innaccone
        //This is the summed densities of pixels or mean
        meanProbability=meanProbability + ((float)i*mprob[i]);
        //This is the squared density or variance
        sumMsq = sumMsq + ((float)i*(float)i*mprob[i]);
    }
    float meanProb=meanProbability;
    float variance=0;
    for (int i=0; i<mprob.length; i++) {
        if (mprob[i]>0) {
            variance
                    =variance+
                    ((((float)i-meanProb)*
                    ((float)i-meanProb))*mprob[i]);}
    }

    stuff[0]=meanProb;
    stuff[1]=(float)Math.sqrt(variance);
//standard deviation
    stuff[2]=(float)Math.sqrt(variance)/meanProb;
//coefficient of variation
    stuff[3]=//the variance/the mean squared = lacunarity-1
            (sumMsq-
            (meanProbability*meanProbability))/
            (meanProbability*meanProbability);
}
return stuff;

}


/**
* Formats a double to a string using
* DecimalFormat and 4 decimal places..
*
* @param   sum double to format
* @return  string from the passed double
*/
public static String fnum(double sum) {

DecimalFormat myFormatter = 
        new DecimalFormat("####.####");

String output = myFormatter.format(sum);
return output;
}
/**
* Returns the string of the passed float, to 4 decimal places.
* @param sum float to make into a string
* @return String for the passed float
*/
public static String fnum(float sum) {

DecimalFormat myFormatter =
        new DecimalFormat("####.####");

String output = myFormatter.format(sum);
return output;
}

/**Formats numbers as strings using the
* passed number of decimal
* places.
*
* @param   sum number to format
* @param   decimals number of decimal places
* @return  String of formatted number */
public static String fnum(double sum, int decimals) {

DecimalFormat myFormatter =
        new DecimalFormat("####.####");
switch (decimals) {
    case 0: myFormatter = 
            new DecimalFormat("####.");
    case 1: myFormatter = 
            new DecimalFormat("####.#");
    case 2: myFormatter = 
            new DecimalFormat("####.##");
    case 3: myFormatter = 
            new DecimalFormat("####.###");
    case 4: myFormatter = 
            new DecimalFormat("####.####");
    case 5: myFormatter = 
            new DecimalFormat("####.#####");
    case 6: myFormatter = 
            new DecimalFormat("####.######");
    default: myFormatter = 
            new DecimalFormat("####.##");
}       String output = myFormatter.format(sum);
return output;
}

public static String fnum(float sum, int decimals) {

DecimalFormat myFormatter =
        new DecimalFormat("####.####");
switch (decimals) {
    case 0: myFormatter = 
            new DecimalFormat("####.");
    case 1: myFormatter = 
            new DecimalFormat("####.#");
    case 2: myFormatter = 
            new DecimalFormat("####.##");
    case 3: myFormatter = 
            new DecimalFormat("####.###");
    case 4: myFormatter = 
            new DecimalFormat("####.####");
    case 5: myFormatter = 
            new DecimalFormat("####.#####");
    case 6: myFormatter = 
            new DecimalFormat("####.######");
    default: myFormatter = 
            new DecimalFormat("####.##");
}       String output = myFormatter.format(sum);
return output;
}

/**Returns the length of the longest array*/
public static int LongestArray(String [][] a)
{ 
int L=a[0].length;

for (int i = 0; i < a.length; i++)
L= Math.max (a[i].length, L);

return L;
}
/**Returns the length of the longest array*/
public static int LongestArray(int [][] a)
{ 
int L=a[0].length;              
for (int i = 0; i < a.length; i++)
 L= Math.max (a[i].length, L);                    
return L;
}
public static int LongestArray(float [][] a)
{ 
int L=a[0].length;              
for (int i = 0; i < a.length; i++)
 L= Math.max (a[i].length, L);                    
return L;
}
/**Returns the length of the longest array*/
public static int LongestArray(float[][][] a)
{ 
int L=a[0][0].length;      
for (int j =0; j <a.length; j++)
for (int i = 0; i < a[j].length; i++)
 L= Math.max (a[j][i].length, L);                    
return L;
}
/**Returns a double array holding the mean, standard deviation,
* coefficient of variation, and lacunarity statistic for the
* passed probability distribution.
* <ul><LI>The mean of the probability distribution
* is: &#8721;(i*mprob[i])
* where i is the mass value m[i], and 
* mprob[i] is the weight</LI>
* <LI>the std dev of the probability distribution is:
* &#8730;&#8721;(((mprob[i]-meanProb)
*(mprob[i]-meanProb))*i);</LI>
* <LI>the Probability Density lacunarity
* at this epsilon is:
* (&#8721;M<sup>2</sup>
* -(&#8721;M*&#8721;M))/(&#8721;M*&#8721;M);
* </LI><LI>where &#8721;M=&#8721;(i*mprob[i])
* and &#8721;M<sup>2</sup>
* = &#8721;(i*i*mprob[i]);</LI></ul>
*
*
* @param   mprob double array of arrays holding the value
*          of the bin for the frequency distribution
* at [0] and the corresponding
*          probability at [1]
* @param   LOC int specifying which location this scan is from
* @return  double array of mean, stddev, cv,  lacunarity, 
* and cv<sup>2</sup>
*/
public static float [] 
    BinnedProbabilityStats(float [][] mprob, int LOC) {
if (mprob==null)return(null);
if (mprob.length==0)return(null);
else {//bins at 0 and probs at 1
    //int numberOfBoxesWithPixels=boxes;
    float sumM = 0;
    float sumMsq = 0;
    // double sum = 0;
    for (int i = 0; i < mprob[0].length; i++) 
    {//calculate moments as per chap. 8.4.10 , p. 238,  Innaccone

        //This is the summed densities of pixels
        sumM=sumM + (mprob[0][i]*mprob[1][i]);
        //This is the squared density
        sumMsq = sumMsq +
                (mprob[0][i]*mprob[0][i]*mprob[1][i]);
    }
    float meanProb=sumM;
    float variance=0;

    for (int i=0; i<mprob[0].length; i++) {
        if (mprob[1][i]>0) {
            variance=variance
                    +(((mprob[0][i]-meanProb)*
                    (mprob[0][i]-meanProb))*mprob[1][i]);}

    }

    float[] stuff=new float[5];
    stuff[0]=meanProb;
    stuff[1]=(float)Math.sqrt(variance);
    stuff[2]=(float)Math.sqrt(variance)/meanProb;
    stuff[3]= (sumMsq-(sumM*sumM))/(sumM*sumM);
    stuff[4]=variance/(meanProb*meanProb);
    return stuff;
}
}

/**Returns a probability distribution as three arrays, [0] for the
* values or masses, [1] for the probability of the values, and [2]
* for the frequencies.  Invokes
* {@link #GetFrequencyTable frequency table method} and
* {@link #MeanProbArray mean probability method}.
*
* @param   PpB integer array to distribute
* @param   boxes int for the number of 
* elements to distribute from
*          the array, starting at index 0
* @return  double [][] with masses at [0] and
*          their probabilities at [1] and Frequencies at [2]
* @throws  Exception during call to 
* {@link #GetFrequencyTable method}
*          to get FrequencyTable
*/
public static float [][] 
    GetProbabilityDistribution(int [] PpB, int boxes)
throws Exception {
int [][] FREQUENCIES = GetFrequencyTable(PpB);

float [] masses = IntToFloatArray(FREQUENCIES[0]);
float [] frequencies = IntToFloatArray(FREQUENCIES[1]);

float [] probabilities
        = MeanProbArray(FREQUENCIES[1],
        FREQUENCIES[1].length);
//prob is now each frequency divided 
//by the sum of all frequencies
//or the probability distribution of all P at i such that the
//sum of all mprob = 1 and the sum {prob*mass} = the mean
return new float [][]
{masses, probabilities, frequencies};
}

/**Returns the minimum of the passed three doubles using
* Math.min(Math.min(a,b), c)
*
* @param a double to assess
* @param b double to assess
* @param c double to assess
* @return double that is the 
* minimum of the three numbers passed
*/
public static double min(double a, double b, double c) {
double min = Math.min(a, b);
min = Math.min(min, c);
return min;

}
/**
* Returns the minium of the three passed floats.
* @param a float to compare
* @param b float to compare
* @param c float to compare
* @return float for the minimum of the three passed floats
*/
public static float min(float a, float b, float c) {
float min = (float)Math.min(a, b);
min = (float)Math.min(min, c);
return min;

}
/**Returns the array of the logarithms of the passed array
* using Math.log(array[i]) in a loop of i=0 to array.length.
*
* @param   array of doubles
* @return  array of logarithms of those doubles
*/
public static double [] LogArray(double array[]) {
double [] N = new double[array.length];

for (int i = 0; i < array.length; i++)
    N[i]=Math.log(array[i]);

return N;
}



/**Returns the minimum of the array from
* a loop of i=0 to array.length given min=Math.min(array[i], min).
*
* @param   A array of doubles
* @return  double the minimum value
*/
public static double minArray(double [] A) {
double min = A[0];
for (int w= 0; w<A.length; w++)
    min = Math.min(A[w], min);
return min;
}

/**Returns the minimum of the array from
* a loop of i=0 to array.length given min=Math.min(array[i], min).
*
* @param   A array of doubles
* @return  double the minimum value
*/
public static int minArray(int [] A) {
int min = A[0];
for (int w= 0; w<A.length; w++)
    min = (int)Math.min(A[w], min);
return min;
}
/**Returns the minimum of the array from
* a loop of i=0 to array.length given min=Math.min(array[i], min).
*
* @param   A array of floats
* @return  float the minimum value
*/
public static float minArray(float [] A) {
float min = A[0];
for (int w= 0; w<A.length; w++)
    min = (float)Math.min(A[w], min);
return min;
}

/**Returns the maximum of the array from
* a loop of i=0 to array.length given min=
* Math.max(array[i], max).
*
* @param   A array of doubles
* @return  double the minimum value
*/
public static double maxArray(double [] A) {
double max = A[0];
for (int w= 0; w<A.length; w++)
    max = Math.max(A[w], max);
return max;
}
/**Returns the maximum of the array from
* a loop of i=0 to array.length given min=
* Math.max(array[i], max).
*
* @param   A array of doubles
* @return  double the minimum value
*/
public static int maxArray(int [] A) {
int max = A[0];
for (int w= 0; w<A.length; w++)
    max = (int)Math.max(A[w], max);
return max;
}
/**Returns the maximum of the array from
* a loop of i=0 to array.length where max=
* Math.max(array[i], max).
*
* @param   A array of floats
* @return  float the minimum value
*/
public static float maxArray(float [] A) {
float max = A[0];
for (int w= 0; w<A.length; w++)
    max =(float) Math.max(A[w], max);
return max;
}
/**Returns the maximum of the three passed values.
* Uses Math.max(Math.max(a,b), c)
*
* @param   a double to compare to other passed values
* @param   b double to compare to other passed values
* @param   c double to compare to other passed values
* @return  minimum of three passed doubles
*/
public static double max(double a, double b, double c) {
double max = Math.max(a, b);
max = Math.max(max, c);
return max;

}
/**Returns the maximum of the passed three floats.
*Uses (float)Math.max(a,b) then (float)Math.max(max, c);
* 
* @param a float to compare
* @param b float to compare
* @param c float to compare
* @return float for the maximum of the three values
*/
public static float max(float a, float b, float c) {
float max = (float)Math.max(a, b);
max = (float)Math.max(max, c);
return max;

}


/**Returns the array of floats based on the (float)doubles, up to
* <i>length</i> elements of the passed array, A..
*
* @param   A array of doubles
* @param   length length of array to convert
* @return  float array
*/
public static float[] DoubleArrayToFloatArray(double[] A,
    int length) {
float[]N=new float[length];
for (int i = 0; i < length; i++)
    N[i]=(float)(A[i]);
return N;
}
//    /**Returns the array of ints based on the (int)doubles for
//     *<i>length</i> of the elements in A.
//     *
//     * @param   A double [] array of doubles
//     * @param   length int length of array to convert
//     * @return  int array [] of integer casts of the double array, 
//     *for the first <i>length</i> elements of A
//     */
//    public static int[] IntArray(double[] A, int length) {
//        int[]N=new int[length];
//        for (int i = 0; i < length; i++)
//            N[i]=(int)(A[i]);
//        return N;
//    }

//    /**
//     * Returns the passed float array cast to ints.
//     * @param A float [] to cast to int
//     * @param length int for number of elements to return
//     * @return int [] of casts to int
//     */
//    public static int[] IntArray(float[] A, int length) {
//        int[]N=new int[length];
//        for (int i = 0; i < length; i++)
//            N[i]=(int)(A[i]);
//        return N;
//    }
/**Returns the array of the float casts of
* the logarithms of the doubles.
*
* @param   A array of doubles
* @param   length int for length of array to convert
* @return  float array that is logarithms cast to floats
* from passed array
*/
public static float[] LogFloatArray(double[] A,
    int length) {
float[]N=new float[length];
for (int i = 0; i < length; i++)
    N[i]=(float)Math.log(A[i]);
return N;
}
/**
* Returns Math.log(A) for the first <i>length</i> elements of A.
* @param A float [] to calculate
* @param length int for number of elements to use
* @return float [] of log A
*/
public static float[] 
    LogFloatArray(float[] A, int length) {
float[]N=new float[length];
for (int i = 0; i < length; i++)
    N[i]=(float)Math.log(A[i]);
return N;
}
/**
* Returns Math.log(A) for the first <i>length</i> elements of A.
* @param A float [] to calculate
* @param length int for number of elements to use
* @return float [] of log A
*/
public static float[] 
    LogArray(float[] A, int length) {
float[]N=new float[length];
for (int i = 0; i < length; i++)
    N[i]=(float)Math.log(A[i]);
return N;
}
/**Returns the array of the logarithms of the passed doubles.
*Returns only the first <i>length</i> elements of A.
* @param   A array of doubles
* @param   length int for length of array to convert
* @return  double array that is logarithms from passed array
*/
public static double[] LogDoubleArray(double[] A,
    int length) {
double[]N=new double[length];
for (int i = 0; i < length; i++)
    N[i]=Math.log(A[i]);
return N;
}


/**Returns the logarithm of the inverses of the
* array passed using Math.log(1.0/A[i]), for 
* <i>length</i> elements
* of A..
*
* @param   A array of doubles
* @param   length int for length of array to convert
* @return  double array that is logarithms of
* 1/array from passed array
*/
public static double[] 
    LogInverseDoubleArray(double[] A,
    int length) {
double[]N=new double[length];
for (int i = 0; i < length; i++)
    N[i]=Math.log(1.0/A[i]);
return N;
}

/**Returns Math.log(1/A) 
* for the first <i>length</i> elements of A.
* 
* @param A float [] to analyze
* @param length int for number of elements to return
* @return float [] of log(1/A)
*/
public static float[] LogInverseFloatArray(float[] A,
    int length) {
float[]N=new float[length];
for (int i = 0; i < length; i++)
    N[i]=(float)Math.log(1.0f/A[i]);
return N;
}

static double[] LogInverseArray(int[] A, int length) {
double[]N=new double[length];
for (int i = 0; i < length; i++)
    N[i]=Math.log(1.0f/(float)A[i]);
return N;
}
/**Returns the array of doubles as 1.0/Array[i] for all i in Array.
*
* @param   A [] doubles
* @param   length int for number of elements to assess
* @return  double array of inverses
*/
public static double[] 
    InverseDoubleArray(double[] A,
    int length) {
double[]N=new double[length];
for (int i = 0; i < length; i++)
    N[i]=(1.0/A[i]);
return N;
}
/**Returns the inverse of the passed array.
* Returns 1/A, using the first <i>length</i> elements of A.
* @param A float [] to calculate
* @param length int for number of elements to return
* @return float [] of 1/A
*/
public static float[] InverseFloatArray(float[] A,
    int length) {
float[]N=new float[length];
for (int i = 0; i < length; i++)
    N[i]=(1.0f/A[i]);
return N;
}
/*** Returns an array of 1/A
* @param A double [] of values
* @return float [] of 1/A*/
public static float[] InverseFloatArray(float [] A) 
{
    int length = A.length;
    float[]N=new float[length];
    for (int i = 0; i < length; i++)
        N[i]=(1.0f/A[i]);
    return N;
}

/**Returns an array of two random ints that are 
*within x and y, corrected if either is negative to 
* so that the negative one is 0.
@param x int for x margin
@param y int for y margin
@return an int array holding the x [0] and y[1]*/
public static int[] RandomXY(int x, int y) 
{
    int [] xy = new int [2];
    Random rnd = new Random ();      
    xy[1]=y<=0?0:rnd.nextInt(y);
    xy[0]=x<=0?0:rnd.nextInt(x);         
    return xy;
}

static int abs(int q){return q<0?q*-1:q;} 
/**Returns the mean of the passed array, using the
* number of elements specified starting at index 0.
*<li>&mu;=&#8721;[a<sub>0 to a.length</sub>]
* &#247;a.length</li>*
* @param   a double array to find the arithmetic mean of
* @return  double as sum of the array divided
* by the number of values summed*/
public static double MeanOfArray(double[] a)
{
    double sum = SumArray(a, a.length);
    return (sum/a.length);
}
/**Returns the mean of the passed array, using the
* number of elements in the array starting at index 0.
**<li>&mu;=&#8721;[a<sub>0 to a.length</sub>]
* &#247;a.length</li>
* @param   a float array to find the arithmetic mean of
* @return  float as &#8721; of the array divided
* by the number of values summed*/
public static float MeanOfArray(float[] a) 
{
    float sum = SumArray(a, a.length);
    return (sum/a.length);
}
//    /**Returns the passed array of floats as a string of
//     *tab deliminated values*/
//    public static String ArrayToTabString(float [] a)
//    {
//        String s= "";
//        for (int i = 0; i < a.length; i++)
//            s=s+a[i]+tab;
//        return s;
//    }
/**Returns two strings, one holding masses and one
*holding frequencies, based on the passed array
* that has the index as the value of the element
*and the element as that value's frequency.  
*@param f float [] where the index is the value and
*the element is the frequency of that value
*@return String [] two strings where [1] is the 
*value string and [0] is the frequency of each value,
*with frequencies of 0 removed for both.
*/
public static String [] MassAndFrequencyStrings(float [] f)
{ 
//make a string array to hold each nonzero value 
String [] s = new String [2];
s[0]="";
s[1]="";
boolean first=true;
for (int i = 0; i < f.length; i++)
{
  if(f[i]>0)
  {    
      if (first)
      {
            first=false;
            s[0]=fnum(f[i]);
            s[1]=fnum(i);
       }
       else
       {
            s[0]=s[0]+tab+fnum(f[i]);
            s[1]=s[1]+tab+ fnum(i);               
        }
}}

return s;
}
/**Returns two strings, one holding masses and one
*holding frequencies, based on the passed arrays
* that have the bins and probabilities.  
*@param f float [][] of bins [0] and probabilities [1]
*@return String [] two strings where [1] is the 
*value string and [0] is the frequency of each value,
*with frequencies of 0 removed for both.
*/
public static String [] MassAndProbabilityStrings(float [][] f)
{ 
//make a string array to hold each nonzero value 
String [] s = new String [2];
int B=0;
int P=1;
s[B]="";
s[P]="";
boolean first=true;
for (int i = 0; i < f[0].length; i++)
{  
  {    
      if (first)
      {
            first=false;
            s[P]=fnum(f[P][i]);
            s[B]=fnum(f[B][i]);
       }
       else
       {
            s[P]=s[P]+tab+fnum(f[P][i]);
            s[B]=s[B]+tab+ fnum(f[B][i]);               
        }
}}

return s;
}         
/**Returns the double mean of the passed integer array.
**<li>&mu;=&#8721;[a<sub>0 to a.length</sub>]
* &#247;a.length</li>
* @param   a array of integers
* @return  double for the sum of the array divided by the
* number of integers added*/
public static double MeanOfArray(int[] a) 
{
    double sum = SumArray(a, a.length);
    return (sum/a.length);
}

/**Returns a map that has keys and their corresponding mapped
* values, from a passed integer array.  
* The array is sorted into bins and
* the bins tallied.
* ClassCastException, IllegalArgumentException,
* NullPointerException, UnsupportedOperationException,
* getting and putting values in a HashMap.
* @return      java.util.Map with a key for every integer that 
* appeared in the list, and corresponding mapped members that
*  are the number of times that integer  appeared in the original 
* array
* @param       args the list of arguments*/
public static Map LoadFrequencyTable(int[] args) {
final Integer ONE = new Integer(1);
Integer ARGS [] = new Integer[args.length];
//           Convert the incoming array to an Integer array
for (int i = 0; i < args.length ; i++)
    ARGS[i]=new Integer(args[i]);

Map m = new HashMap();
Integer freq=null;

for (int i=0; i<args.length; i++) {
    try{
        freq = (Integer) m.get(ARGS[i]);//Map keys to values in the
        //                                          new map by
        //                                checking each value in the array
        //                                        for a value mapped to it
    }catch(UnsupportedOperationException n){} 
    catch(ClassCastException cc){} 
    catch(IllegalArgumentException g){} 
    catch(NullPointerException hg){}
    try{m.put(ARGS[i],
            (freq==null ? ONE : new Integer(freq.intValue() + 1)));
    //                     then if there is no value, making a new key
    //                     and giving it a value of 1 in the
    //                     returned map or else adding 1 to the
    //                     number counted so far and assigning that
    //                               new count as the value
    }catch(UnsupportedOperationException n){}
    catch(ClassCastException cc){} 
    catch(IllegalArgumentException g){}
    catch(NullPointerException hg){}
}

return m;
}




/**Utility that returns a 2d integer array with keys at [0][i] and
* their frequencies at [1][i].
* @return an int [][] that is a mapped list of the unique integers
* that were in the passed array, where the array at index [0] is the
* integers (masses) and the array at index [1]
* is the corresponding frequency
* of each integer (the frequency of the mass)
* @param args the array of integers
* @throws Exception
*/
public static int [][] GetFrequencyTable(int args[]) throws 
    Exception {
return ReadFrequencyTable(LoadFrequencyTable(args));
}

/**Makes a frequency table from the passed Map,
* which must translate
* into Integers.
* @param m java.util.Map to assess
* @return int [][] array of the keys at [0] and values at [1]
*/
public static int[][] ReadFrequencyTable(Map m) {
Integer [] KEYS = new Integer [m.size()];
Integer [] VALUES = new Integer[m.size()];
int c = 0;
for (Iterator i=m.entrySet().iterator(); i.hasNext(); ) {
    Entry e = (Entry) i.next();
    KEYS[c]=(Integer)e.getKey();
    VALUES[c]=(Integer)e.getValue();
    c++;
}

int [] Keys = new int [c];
int [] Values = new int [c];
for (int i = 0; i < m.size(); i++) {
    Keys[i]=KEYS[i].intValue();
    Values[i]=VALUES[i].intValue();

}
return new int[][]
{Keys, Values};

}


/** Returns the array with the lower coefficient of variation of
* the passed two arrays.
* @param A [] int array to compare
* @param B [] int array to compare
* @return the array having the lower coefficient of variation*/
public static int[] MinimumVariationArray(int[] A, int[] B)
{
    boolean returnA=false;
    {
        if (FLstatistics.CoefVar(IntToFloatArray(A), A.length)<
                FLstatistics.CoefVar(IntToFloatArray(B), B.length))
            returnA=true;
        else returnA=false;
    }
    if (returnA)return A;
    else return B;
}
/**Returns a double array from the passed integer array.*
* @param   array of integers
* @return  double []*/
public static double[] IntToDoubleArray(int[] array) 
{
    double [] N = new double[array.length];
    for (int i = 0; i<array.length; i++)
        N[i]=(double)array[i];
    return N;
}
/**Returns the passed array as a float array
* @param array int [] to convert
* @return float [] of the passed integer array*/
public static float[] IntToFloatArray(int[] array) 
{
    float [] N = new float[array.length];
    for (int i = 0; i<array.length; i++)
        N[i]=(float)array[i];
    return N;
}
/**Returns the passed array as a float array
* @param array int [] to convert
* @return float [] of the passed integer array*/
public static float[][] IntToFloatArray(int[][] array) 
{
    float [][] N = new float[array.length][];
    for (int i = 0; i<array.length; i++) N[i]=IntToFloatArray(array[i]);
    return N;
}
/** Returns the array with the lower mean.
*<li>mean<sub>A</sub>
* =&mu;=&#8721;[A<sub>0 to A.length</sub>]
* &#247;A.length</li>
* @param A int [] array to assess
* @param B int [] array to assess
* @return array with lower mean*/
public static int[] MinimumMeanArray(int[] A, int[] B)
{
    boolean returnA=false;
    {
        if  (FLstatistics.Mean(IntToFloatArray(A), A.length)
                <
                FLstatistics.Mean(IntToFloatArray(B), B.length) )
        returnA=true;
        else returnA=false;
    }
    if (returnA)return A;
    else return B;
}


/** Returns whichever of the passed arrays has the greater mean.
*<li>mean<sub>A</sub>
* =&mu;=&#8721;[A<sub>0 to A.length</sub>]
* &#247;A.length</li>
* @param A int [] array to assess
* @param B int [] array to assess
* @return int [] array having greater mean
*/
public static int[] MaximumMeanArray(int[] A, int[] B)
{
    boolean returnA=false;
    {
        if ( FLstatistics.Mean(IntToFloatArray(A), A.length)
            >
            FLstatistics.Mean(IntToFloatArray(B), B.length) )
        returnA=true;
        else returnA=false;
    }
    if (returnA)return A;
    else return B;
}

/** Returns the array having the lower  minimum of two passed 
 * arrays.<li>mean<sub>A</sub> =&mu;=&#8721;[A<sub>
 * 0 to A.length</sub>] &#247;A.length</li>
* @param A int [] array to assess
* @param B int [] array to assess
* @return Array that had lower minimum.*/
public static int[] LowerPeakArray(int[] A, int[] B) 
{
    double maxa = (double)A[0];
    double maxb= (double)B[0];

    for (int i = 0; i < A.length; i++)
    {
        maxa=Math.max(A[i], maxa);
    }
    for (int i = 0; i < B.length; i++)
        maxb=Math.max(B[i],maxb);
    if (maxa>maxb)return B;
    else return A;
}

/**Writes a string from the passed array with tabs between.*
* @param Probabilities double array
* @return String for array*/
public static String ArrayToString(double[] Probabilities)
{
    String strings= "";
    for (int i = 0; i < Probabilities.length; i++)strings=strings + tab +
                java.lang.Double.toString(Probabilities[i]);
    return strings;
}

/**Returns an array whose elements are the original elements
*divided by the sum of the array.
* @param array an integer array to assess
* @param NumElements an integer for the number of elements to 
 * assess
* @return double array that contains each element divided by
* the sum of the entire array, so that the sum of the new array is 1*/    
public static float[] MeanProbArray(int[] array, int NumElements) 
{
    float sum = (float)SumArray(array, NumElements);
    float [] N= new float [NumElements];
    for (int i = 0; i < NumElements; i++)
    {
        N[i]=(((float)array[i])/sum); 
    }
    return N;
}


/**Returns array[i]/sum of array over the first NumElements in the
 * array. 
* @param array of floats to sum and divide
* @param NumElements int for number of elements to use
* @return float [] of values for element/sum
* for the first NumElements in array*/
public static float[] ElementOverSumOfArray(float[] array, 
    int NumElements) 
{
    float sum = SumArray(array, NumElements);
    float [] N= new float [NumElements];
    for (int i = 0; i < NumElements; i++) 
    {
        N[i]=((array[i])/sum);
    }
    return N;
}

/**Returns an array whose elements are the original elements
* divided by the passed value of total foreground pixels.
* @return double []array that contains each element divided by
* the total foreground pixels, for the first 
* NumElements in the array.
* @param TotalForegroundPixels
* @param array a double array to assess
* @param NumElements  int for the number
* of elements to assess*/
public static double[] MeanMassProbArray(double[] array,
    int NumElements, int TotalForegroundPixels)
{
    double sum = TotalForegroundPixels;
    double [] N= new double [NumElements];
    for (int i = 0; i < NumElements; i++) 
    {
        N[i]=((array[i])/sum);
    }
    return N;
}

/** Returns the sum of the elements in the passed array, up
* to the number passed.
* @return double for sum of all elements
* @param Nums value for the number of values
* to add in the passed array
* @param array integer array to sum
*/
public static double SumArray(int[] array, int Nums) 
{
    double sum=0;
    for (int i =0; i < Nums; i++)sum=sum+(double)array[i];
    return sum;
}
/**Returns sum of first Nums elements of array.
* @param array array to sum
* @param Nums int for number of elements to sum
* @return int for sum of first Nums elements of array*/
public static int SumInts(int[] array, int Nums)
{
    int sum=0;
    for (int i =0; i < Nums; i++)sum=sum+array[i];
    return sum;
}

/** Returns the sum of the elements in the passed array, up
* to the number passed.
* @return double for sum of all elements
* @param Nums int for the number of values to assess in the passed
 * array
* @param array integer array to sum*/
public static double SumArray(double[] array, int Nums) 
{
    double sum=0;
    for (int i =0; i < Nums; i++)sum=sum+ array[i];
    return sum;
}
/**Returns the sum of the first Nums elements of the array.
* @param array float [] to sum
* @param Nums int for the number of elements to sum
* @return float sum of the Nums elements of the array*/
public static float SumArray(float[] array, int Nums)
{
    float sum=0f;
    for (int i =0; i < Nums; i++)sum=sum+ array[i];
    return sum;
}
 
    
}

