package GUI;
//  Notice //
//  The contents of this file are subject to the  same license as the 
// source code for ImageJ.  FracLac is in the public domain.  If you 
// use it to publish research, please cite the source.  The initial 
// developer of the original code is Audrey Karperien, Charles Sturt 
// University.
//=======================================
//=======================================
//Author:     Audrey Karperien
//Project:    FracLac
//Version:    2.4e; needs ImageJ1.37c
//Files:      FL_.java; FracLac.java; FLAnalyzer; FLUtilities
//            FracLac_.jar; FracLacJ1.4_.jar; FracLac_J1.5.jar
//Program:    Masters (Honours) Research
//Department: Neuroscience, School of Community Health,
//         Faculty of Health Studies, Charles Sturt University, Australia
//Date:       2002-Apr 2006
//Purpose:    Plugin for ImageJ analyzes digital images using
//            grid scans.
//Returns:    Fractal dimensions, the generalized
//            dimension spectrum over a set of values for Q, the
//            frequency distribution of pixels,
//            lacunarity, cell span, and circularity for binary images.
//            Also delivers regression statistics, graphs,
//            a colour coded scan of the image for the fractal dimension,
//            and graphics showing grid placement.
//Features:   The algorithm adds to standard box
//            counting techniques
//            options to find global and local dimensions, average
//            dimensions over shifted grid positions, and a most efficicient
//            cover dimension.  Aa an additional feature,
//            the plugin can automatically optimize scans based
//            on each image (alternatively, it lets the user manually
//            specify features of the scan). Offers a practical solution
//            to multifractal analysis with box counting by scanning
//            to find an optimized multifractal spectrum.  Includes
//            a sliding box lacunarity feature. Different types of scaling
//            are available from the user interface, as well.//
//History and background:
//            The algorithm is modified originally from ImageJ's box
//            counting algorithm and H. Jelinek's Image plugin.
//            A full description of the
//            basic techniques represented can be found in:
//            T.G. Smith, Jr., G.D. Lange and W.B. Marks,
//            Fractal Methods and
//            Results in Cellular Morphology, J. Neurosci. Methods,
//            69:1123-126, 1996.  See also
//            E. Fernandez et al., Are neurons multifractals?,
//            J. Neurosci. Methods, 89:151157, 1999
//            and R.E. Plotnick, R.H. Gardner, and R.V. O'Neil,
//            Lacunarity indices as measures of landscape texture,
//            in Landscape Ecology 8(3):201-211, 1993 and Innaconne,
//            Geometry in Biological Systems.  The multifractal
//            spectrum is calculated based on the ideas of
//            A. Chhabra and R.V. Jensen, Direct
//            Determination of the f(alpha)
//            singularity spectrum in Phys. Rev . Lett. 62: 1327, 1989.
//            The convex hull algorithm
//            was provided by Thomas Roy, University of Alberta, Canada.
//            I referred also to Costa and Cesar, Shape Analysis
//            and Classification, CRC Press, 2001, for other info.
//Public Domain:
//            I wrote this plugin for ImageJ.  It is provided without
//            warranty and is forever freely available
//            to anyone to use or modify, in the same way that
//            ImageJ is.  Please acknowledge it if  you use it for published 
//            research.
//==================================
//==================================

import FLAnalyzer.FLGraphics;
import FLAnalyzer.FLMain;
import FLAnalyzer.FLVars;
import FLAnalyzer.FLsetup;
import ij.*;
import ij.gui.*;
import ij.io.Opener;
import ij.measure.Measurements;
import ij.measure.ResultsTable;
import ij.plugin.PlugIn;
import ij.plugin.filter.Analyzer;
import ij.plugin.filter.ParticleAnalyzer;
import ij.plugin.frame.RoiManager;
import ij.process.ByteProcessor;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import ij.text.TextWindow;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.awt.image.MemoryImageSource;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Random;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;


/**User interface drives the workhorse
 * {@link FLAnalyzer.FLMain FracLac}class, putting up a JFrame
 * housing buttons to set up and run analyses without invoking the 
 * plugin repeatedly from the ImageJ menu. Buttons have action 
 * listeners to set up {@link #BoxCountListener nonoverlapping 
 * scans}, {@link #subListener subarea scans} (using an internal 
 *{@link #SetUpRectangleSubscans analyzer} or the
 * ij.plugin.filter.ParticleAnalyzer), potentially {@link #lacListener 
 * overlapping scans}, and {@link #mfListener multifractal scans}. 
 * When a setup button is clicked, a {@link FLAnalyzer.FLsetup 
 * popup}gets inputs for the {@link FLAnalyzer.FLVars variables}
 * of the the FracLac instance invoked in this class.  The inputs are 
 * not saved if the popup is cancelled by the user. Saved inputs 
 * specify the properties of the analysis that is done.  There is 
 * {@link #HelpListener help button} which displays a page of basic
 * instructions.  Other buttons signal when to do an action: a scan of 
 * an {@link #DoActiveImage open image or selection on it}, of 
 * {@link #OpenAndScanFilesWithoutShowingThem image files}, 
 * or of {@link #DoRois rois in the RoiManager}. The methods called
 * here invoke {@link FLAnalyzer.FLMain the working} class's 
 * methods to do the actual scan and analyze the data.  Some of the 
 * methods call scans then  {@link #DrawParticles draw graphics} 
 * including the {@link #DrawHullandCircle bounding circle and 
 * convex hull}. The action methods also {@link #WriteScanData 
 * display the data writer} after the scanners return.  The 
 * calculations, metrics, and graphics are all explained in more detail
 * in the overview for the overall package.<p>This program is free 
 * software distributed in the same way that ImageJ is. <p>A. 
 * Karperien, MSc, Charles Sturt University, 2001-2006
 * <H2>Related Documentation</H2><UL>
 * <LI><A href="http://rsb.info.nih.gov/ij/plugins/frac-lac.html">
 * User's Guide.</A></LI>
 * <li> or see {@link <a href=
 * "http://www.geocities.com/akarpe@sbcglobal.net/box.html">
 * FracLac page</a>} </LI>For futher information or to send 
 * comments, bugs, and feature requests contact {@link 
 * <a href="mailto:akarpe@postoffice.csu.edu.au"> the 
 * author</a>}</UL><p>IDE: NetBeans IDE 4.1</p>
 * @version 2.4e FracLac_ May 2006
 * @version 1.37g ImageJ
 * @author Audrey Karperien (akarpe01@postoffice.csu.edu.au)
 * @author Thomas R. Roy, University of Alberta, Canada (convex 
 * hull)
 * @author ImageJ code and borrowed code from Wayne Rasband of
 * the NIH
 * @author H.F. Jelinek, Charles Sturt University, Australia (idea 
 * developer)
 * @since jdk 1.5.0_04*/
public class FL_ implements PlugIn, ActionListener, WindowListener
{    
    //////////////////////////////////////////////////////////////////////
    ////////////      VARIABLES FOR INTERFACES       ////////////
    ////////////////////////////////////////////////////////////////////
    /**Panel holds {@link #DoActiveImageButton} and
     * {@link #DoManyFilesButton} for
     * invoking another analysis. */
    public JPanel panel=new  JPanel ();
     /**int for scan type
      * @see #SLACSCAN =1
      * @see #BCSCAN =0
      * @see #MFSCAN =2
      * @see #GBCSCAN =3
      * @see  #SUBSCAN =4
      * @see #ScanType*/
    public final static int SLACSCAN=1, BCSCAN=0,
            MFSCAN=2, GBCSCAN=3, SUBSCAN=4;
    /**int for scan type
      * @see #SLACSCAN =1
      * @see #BCSCAN =0
      * @see #MFSCAN =2
      * @see #GBCSCAN =3
      * @see  #SUBSCAN =4*/
    int ScanType = 0; 
    String BoxCountTitle="FracLac Box Count Results";
    String SlidingLacTitle="FracLac Sliding Box Lacunarity Results";
    String GSlidingLacTitle=
            "FracLac Sliding Box Lacunarity (grayscale) Results";
    String MultifractTitle="FracLac Multifractal Analysis Results";
    String GBoxCountTitle="FracLac Grayscale Box Count Results";
    String RawDataTitle="Data";
    String GRawDataTitle="Grayscale Data";
    String fail ="Conversion to gray scale failed.", 
            click = "Click to change ";
    /**ImageStack variable that is used for showing
     * colour coded images.
     * @see #DrawParticles
     * @see #DrawManyRectangles*/
    public ImageStack ColouredRoisStack;
    
    /**Used in {@link #applyThreshold}*/
    private double minThreshold;
     
    /**Used in {@link #applyThreshold}*/
    private double maxThreshold; 
 
    boolean FirstBCScan=true;
    boolean FirstSLScan=true, FirstGSLScan=true;
    boolean FirstMFScan=true;
    boolean FirstGBCScan=true;
   
/**Text Windows displaying results for the different types of 
* analyses. Some windows display summary information:
* <li>{@link #BoxCountWindow} displays results for {@link 
* FLMain#BoxCountDataWriter box counting},<li>{@link 
* #SlidingLacWindow} displays results for {@link 
* FLMain#StoreSLACData sliding box lacunarity},<li>{@link 
* #MultifractWindow}displays results for a {@link 
* FLMain#MultifractalDataProcessor multifractal analysis}, 
* <li> {@link #GBoxCountWindow} displays grayscale 
* boxcounting data,<li> {@link #GSlidingLacWindow} displays
* grayscale {@link FLMain#GetGSLACData sliding box lacunarity} 
* data. Some display data for box size, count, mass, etc.: <li>
* {@link #GDataWindow} displays data from grayscale scans, and
* <li>{@link #DataWindow} for binary image scans. Raw data are
* displayed in windows private to the {@link 
* FLAnalyzer.FLDataProcessor#recordRawData data printer}.*/
public   TextWindow BoxCountWindow,  SlidingLacWindow, 
        GSlidingLacWindow,MultifractWindow ,  GBoxCountWindow,
        GDataWindow, DataWindow;

    /**Buttons for running the analysis.*/
    public JButton HelpButton;
    
    /**Button for setting up scans.     */
    public JButton SetUpButton;
    
    /** Button for setting up subscans.     */
    public JButton SubButton;
    /** Button for opening and scanning many
     * images without viewing them. */
    
    public JButton DoManyFilesButton;
    /**Button for  scanning many rois from the roi manager. */
    
    public JButton RoisButton;        
    
    /**Button to select the active image or roi and process it. */    
    public JButton DoActiveImageButton;
    
    /**Button to select a sliding lacunarity scan.
     * {@link FLAnalyzer.FLsetup#getSLacInputs 
     * sliding lacunarity option} */
    public JButton SlidingBoxLacunarityButton;
    
    /**Button to select a multifractal scan.
     * {@link FLAnalyzer.FLsetup#getMFInputs
     * multifractal options}*/
    public JButton MultiFractalButton;    
        
    /**Pane to display the colour choices for sub scans. */
    JScrollPane treeView;
    
    /**Displays the type of scan that is currently set. */
    JLabel statuslabel;
    
    /**Instance of {@link FLAnalyzer.FLMain main class}
     * that does the analyses.*/
    public FLMain F;
    
    /**Instance of {@link FLVars variables class}*/
    public FLVars MFVars=new FLVars(), SLACVars=new FLVars(), 
            BCVars=new FLVars(), subVars=new FLVars();
  
    /**Maximum number of box sizes to use in a series.
  * Is set by {@link FLAnalyzer.FLsetup#getBCInputs}.
  * Default is 0.
  * @see FLAnalyzer.FLMain#MakeNewBoxSizes */
     private static int MAXMFSizes = 20;
     
/**The number of grid positions used in
* the multifractal analysis
* as set in the
* {@link FLAnalyzer.FLsetup#getMFInputs get
* multifractal inputs} method, to temporarily supersede
* {@link FLVars#GridPositions}.
* The default is 4; these first four values 
* are different than
* in the regular analysis.  The position changes
* only if a
* {@link FLAnalyzer.FLMain#moveGridalloverCountPixels
* multifractal}
* analysis is being done. The change orients the grid so
* that it falls completely at each of the four corners of the rectangle
* encompassing the image. This emulates 
* rotating the image 90&deg; to all four positions,
* whereas the usual shift is a translation.  For calculations of
* the positions, see {@link 
* FLAnalyzer.FLMain#moveGridalloverCountPixels moving} 
* function.*/
public static int  NUMBEROFGRIDSFORMF = 4;
   
/**Constructor that accepts no parameters*/
    public FL_ ()
    {    }

    /**Roi for scans.
     * @see #StoreRoiType     */
     public OvalRoi op=null;
     
     /**Roi for scans.
     * @see #StoreRoiType
     */public PolygonRoi pp=null;
     
     /**Roi for scans.
     * @see #StoreRoiType
     */public ShapeRoi sp=null;
     
      /** Main FracLac frame for buttons. */
    JFrame frame = new JFrame ("FracLac v2.4e");
    /** Frame for colours for sub scans.     */
    JFrame treeframe = new JFrame ();
    /** Help frame. */
    JFrame hframe = new JFrame ("FracLac Instructions");
     /** For help files.    */
    JScrollPane textScrollPane = new JScrollPane ();    
    
    
     /**Runs FracLac_ by invoking a new FracLac instance and
     * initializing variables for FLVars instances used to hold
      * options for the different scan types.
      * If the passed argument is "about",
     * calls the {@link #showAbout showAbout method} 
     * then terminates.
     * Calls the {@link #ShowPanelOfButtons ShowFL} method.
     * @see #ShowPanelOfButtons
      *@see #initializeVars
     * @param str String for starting by double clicking; leave blank */
    public void run (String str)
    {
        if (str.equals ("about"))showAbout ();
        else
        {
            F= new FLMain ();
            F.V.Qs = FLsetup.MakeAnArrayOfQs
                    (F.V.getQmin (), F.V.getQmax (),
                    F.V.getQinc (), F.V.getQinc ());
            initializeVars();
            ShowPanelOfButtons ();
        }
    }
    
    void initializeVars()
    {
            initializeMF();
            initializeSLAC();
            initializeSubs();
    }
    
    void initializeMF()
    {
        MFVars.GridPositions
                    =NUMBEROFGRIDSFORMF;
            MFVars.GraphFA=true;
            MFVars.setMaxNumBoxSizes (MAXMFSizes);
            MFVars.maxpercent=60;
            MFVars.percentsubboxsize=60;
            MFVars.RandomMassMaxSubpercent=70;
            MFVars.subsamples=15;
            MFVars.UseGreater=true;
            MFVars.Series=2;
            MFVars.setUserMinBoxSize(10); 
            MFVars.setUserMaxBoxSize(0);
            MFVars.setMF(true);
            MFVars.GraphLac=false;
            MFVars.grayscantype=MFVars.DIFFERENTIALBC;
    }
    
    void initializeSubs()
    {
        this.subVars.setGraphLac (false);
        this.subVars.setDocircul (false);
        this.subVars.DRAWCircle=false;
        this.subVars.DRAWHull=false;
        this.subVars.GridPositions=1;
        this.subVars.GraphLac=false;
    }
    
    void initializeSLAC()
    {
        SLACVars.setImageType (FLVars.BINARY);
        SLACVars.Series=0;
        SLACVars.setUserMinBoxSize(5);
        SLACVars.setUserMaxBoxSize(0);  
        SLACVars.setMaxNumBoxSizes(10);
        SLACVars.SLAC=true;
        SLACVars.maxpercent=100;
        SLACVars.setUserWantsToSpecifyNumberOfBoxes(true);
        SLACVars.GraphLac=true;
    }
    
   
    
    
    /** The font for buttons.     */
    Font font = new Font ("Tahoma", 1, 10);
    
   
    
    
    
    /** Puts up the panel with buttons and 
     * action listeners, and a colour tree.
     *  Buttons call up panels to change 
     * parameters and scan type, start
     * scans, call up some help, or close all
     * the open windows except the results tables.
     * @see #DoManyFilesButton
     * @see #DoActiveImageButton
     * @see #SlidingBoxLacunarityButton
     * @see #SetUpButton     */
    public void ShowPanelOfButtons ()
    {       
        //              make a tree for colour choices,
        //and frames for showing buttons for
        //              controlling the plugin
        
        makeTree ();
        
        frame.getContentPane ().setLayout (new FlowLayout ());
        panel = new JPanel ();
        panel.setLayout (new GridLayout (12,0, 5 , 0));
        JLabel optionslabel = new JLabel ("Options");
        JLabel actionslabel = new JLabel ("Actions");
                
        BevelBorder bevborder =
                new
                BevelBorder (BevelBorder.RAISED);
        //              add a button for opening new image files
        //              and automatically analyzing them
        DoManyFilesButton = new JButton ();
        DoManyFilesButton.setText ("Select and Scan Files");
        DoManyFilesButton.setToolTipText ("Click to do more files "
                +"without changing any settings");
        DoManyFilesButton.setFont (font);
        DoManyFilesButton.setBorder (bevborder);
        DoManyFilesButton.setBackground (new Color (45, 119, 176));
        DoManyFilesButton.setActionCommand ("new");
        DoManyFilesButton.addActionListener (this);
        DoManyFilesButton.setEnabled (false);
        DoManyFilesButton.setVisible (false);
        //make a button for scanning rois from the Roi Manager
        RoisButton = new JButton ();
        RoisButton.setText ("Scan Roi Manager");
        RoisButton.setToolTipText ("Click to scan rois in Roi Manager");
        RoisButton.setFont (font);
        RoisButton.setBorder (bevborder);
        RoisButton.setBackground (new Color (45, 119, 176));
        RoisButton.setActionCommand ("rois");
        RoisButton.addActionListener (DoRois);
        RoisButton.setEnabled (false);
        RoisButton.setVisible (false);
        //              add a button to analyze the active image
        DoActiveImageButton = new JButton ();
        DoActiveImageButton.setFont (font);
        DoActiveImageButton.setBorder (bevborder);
        DoActiveImageButton.setBorderPainted (true);
        DoActiveImageButton.setText ("Scan Current" +
                " Image or Roi");
        DoActiveImageButton.
                setIcon (imageloader.LoadImage ("scan.jpg"));
        DoActiveImageButton.setBorder (bevborder);
        DoActiveImageButton.setFont (font);
        DoActiveImageButton.setToolTipText ("Click to " +
                "analyze the active image "
                +" without changing any settings");
        DoActiveImageButton.setBackground (
                new Color (45, 119, 176));
        
        DoActiveImageButton.setActionCommand ("again");
        DoActiveImageButton.addActionListener
                (DoActiveImageListener);
        DoActiveImageButton.setEnabled (false);
        DoActiveImageButton.setVisible (false);
        //make a label for displaying the scan type
        statuslabel = new JLabel ("Click a purple " +
                "button to select a scan");
        statuslabel.setFont (font);
        statuslabel.setForeground (Color.red.darker ());
        //make a button for setting up a sliding box lacunarity scan
        SlidingBoxLacunarityButton = new JButton ();
        SlidingBoxLacunarityButton.setFont (font);
        SlidingBoxLacunarityButton.setBorder (bevborder);
        SlidingBoxLacunarityButton.setIcon (imageloader.
                LoadImage ("slide.jpg"));
        SlidingBoxLacunarityButton.setText 
                ("Plan a Sliding Box Lacunarity Scan");
        SlidingBoxLacunarityButton.setToolTipText (click
                +"sliding box lacunarity settings");
        SlidingBoxLacunarityButton.setBackground (
                new Color (135, 100, 176));
        SlidingBoxLacunarityButton.setActionCommand ("Lac");
        SlidingBoxLacunarityButton.addActionListener (lacListener);
        //make a button for setting up a multifractal analysis scan
        MultiFractalButton = new JButton ();
        MultiFractalButton.setText
                (" Plan a Multifractal Analysis Scan  ");
               
        MultiFractalButton.setToolTipText (click
                +"multifractal analysis settings");
        MultiFractalButton.setBorder (bevborder);
        MultiFractalButton.setFont (font);
        MultiFractalButton.setBackground (new Color (125, 100, 176));
        MultiFractalButton.setActionCommand ("mf");
        MultiFractalButton.setIcon
                (imageloader.LoadImage ("henon.jpg"));
        MultiFractalButton.addActionListener (mfListener);
        
        //              add a button to set up the parameters
        //for a regular box count scan by invoking the getInputs dialog
        SetUpButton = new JButton ();
        SetUpButton.setIcon (imageloader.LoadImage ("flicon.jpg"));
        SetUpButton.setFont (font);
        SetUpButton.setBorder (bevborder);
        SetUpButton.addActionListener (BoxCountListener);
        SetUpButton.setText 
                ("  Plan a Standard Box Count Scan  ");
               
        SetUpButton.setToolTipText (click
                +"box counting fractal and lacunarity analysis settings");
        SetUpButton.setBackground (new Color (145, 119, 176));              
        //make a button for setting up subscans
        SubButton = new JButton ();
         SubButton.setIcon (imageloader. LoadImage ("sub.jpg"));
        SubButton.setFont (font);
        SubButton.setBorder (bevborder);
       
        SubButton.setText
                ("          Plan a  Subarea Scan             ");
              
        SubButton.setToolTipText (click
                +"sub scan settings");
        SubButton.setBackground (
                new Color (135, 100, 176));
        SubButton.setActionCommand ("Sub");
        SubButton.addActionListener (subListener);
        //              add a button to close the open windows
        //              because sometimes many graphs are generated
        JButton close = new JButton ();
        close.setFont (font);
        close.setIcon (imageloader.LoadImage ("window.jpg"));
        close.setText ("Close windows");
        close.addActionListener (CloseWindows);        
        
        HelpButton=new JButton ("Instructions");
        HelpButton.addActionListener (HelpListener);
        HelpButton.setFont (font);
        HelpButton.setIcon (imageloader.LoadImage ("blog.jpg"));
        HelpButton.setText ("Instructions");
        //add the buttons and labels to the panel
        panel.add (optionslabel);        
        panel.add (SetUpButton);
        panel.add(SubButton);
        panel.add (SlidingBoxLacunarityButton);
        panel.add (MultiFractalButton);        
        panel.add (actionslabel);
        panel.add (DoActiveImageButton);
        panel.add (DoManyFilesButton);
        panel.add (RoisButton);
        panel.add (close);
        panel.add (HelpButton);        
        panel.add (statuslabel);
        
        frame.getContentPane ().add (panel);
        //set up the colour tree
        treeframe.setTitle ("Colours for Sub Scans");
        treeframe.getContentPane ().add (treeView);        
        //finalize it all for display
        frame.pack ();        
        treeframe.pack ();
        treeframe.setVisible (true);
        frame.addWindowListener (this);        
        frame.setVisible (true);        
    }
    
    
    
    
   
    
    /** Window event gets rid of the tree and help frame on closing.
     *@param e Window Event
     *@see #hframe
     *@see #treeframe
     */
    public void windowClosing (WindowEvent e)
    {
        if(treeframe.isVisible ())treeframe.dispose ();
        if (hframe.isVisible ())hframe.dispose ();
    }
    
  
   
    
    
    /**ActionListener attached to the button
     * for setting up a regular box counting fractal analysis.
     * Gets {@link FLAnalyzer.FLsetup#getBCInputs user inputs}.     
     * Resets
     * FracLac instance's boolean {@link FLAnalyzer.FLVars#abort}
     * to false before call.  Calls internal method 
     * {@link #reveal()} to ensure that the
     * action buttons are visible once the settings have been chosen.*/
    public ActionListener BoxCountListener = new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {            
            F.V.setAbort (false);
            //Store the user's inputs for scan settings in a temp instance 
            BCVars=FLsetup.getBCInputs (BCVars);
            boolean accept = BCVars.proceed;
            if(accept)
            {//if the user wants to go on, record the inputs
                //and get the panel ready for action                
                F.V=BCVars;                
                DoManyFilesButton.setEnabled (true);               
                statuslabel.setText ("Box Count" +setlabel());
                ScanType = F.V.isGray ()?GBCSCAN:BCSCAN;
                reveal ();
            }
            else {F.V.abort=true; return;}           
        }
    };//////END/////////
 
    
    
    /**ActionListener
     * attached to the button for setting up a sub scan
     * fractal analysis.
     * Gets 
     * {@link FLAnalyzer.FLsetup#GetSubScanInputs for subscans}. 
     * Resets
     * FracLac instance's boolean {@link FLAnalyzer.FLVars#abort}
     * to false before call.  The buttons for opening files and
     * using the ROI manager are disabled for subscans, and the 
     * number of grid positions is limited to 1.
     * Calls internal method {@link #reveal} to ensure that the
     * action buttons are
     * visible once the settings have been chosen.*/
    public ActionListener subListener = new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {            
            F.V.setAbort (false);
            //Store the user's inputs for scan settings in a temp instance 
            subVars=FLsetup.GetSubScanInputs (subVars);
            boolean accept = subVars.proceed;
            if(accept)
            {//if the user wants to go on, record the inputs
                //and get the panel ready for action                
                F.V=subVars;     
                statuslabel.setText ("FRACTAL/LACUNARITY Subscans");
                ScanType = F.V.isGray ()?GBCSCAN:SUBSCAN;
                if (!treeframe.isVisible ())treeframe.setVisible (true);                
                if (F.V.DisableRoiManager)RoisButton.setEnabled (false);
                else RoisButton.setEnabled (true);
                DoManyFilesButton.setEnabled (false);                
                statuslabel.setText (gettype () +setlabel());
                reveal ();
                } else F.V.abort=true;            
        }
    };//////END/////////
    
    
   
    
    /** Listener for the Help button, which displays instructions from
     * the stored file help.html.  
     * See internal class GUI.FL.imageloader#LoadHtml
     */
    public ActionListener HelpListener = new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {            
            boolean notloaded=false;
            
            URL url = imageloader.LoadHtml ("help.html");
            if (url==null)IJ.showMessage ("error loading instructions");
            else try
            {                
                JEditorPane hpane=new JEditorPane (url);
                textScrollPane = new JScrollPane (hpane);
                textScrollPane.setVerticalScrollBar (new JScrollBar ());
                hframe.setSize (280, 480);//.setPage(url);
                hframe.setVisible (true);
                hframe.getContentPane ().add (textScrollPane, 0);
                hframe.validate ();
                
            }
            catch(IOException ioe)
            {notloaded=false;}
            
            
        }
    };//////END/////////
    
    
    /**Returns the type of subscan*/
    String gettype ()
    {
        String s=FLsetup.SubScanTypes[F.V.SubScanType];
        return s;
    }
    
    
    
    /**Ensures the button that
     * opens new files is active at the right time only.*/
    void reveal ()
    {
        if(F.V.notsetup)
            
        {
            DoManyFilesButton.setVisible (true);
            DoActiveImageButton.setVisible (true);
            RoisButton.setVisible (true);
            F.V.notsetup=false;
            DoManyFilesButton.setEnabled (true);
            DoActiveImageButton.setEnabled (true);
            RoisButton.setEnabled (true);
            
            
        }
    }
    
   
    
    /**Sets the scan method to sliding box.
     * Prepares for a sliding box 
     * {@link FLVars#SLAC lacunarity} scan by calling the SLAC
     * {@link FLsetup#getSLacInputs input getter}.       
     * @see #DoManyFilesButton is enabled here
     * @see #ScanType (is set to SLACSCAN)*/     
    public ActionListener lacListener = new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {            
            F.V.setAbort (false);
            SLACVars=FLsetup.getSLacInputs (SLACVars);
            if(SLACVars.proceed)
            {                
                F.V=SLACVars;                
                DoManyFilesButton.setEnabled (true);                
                statuslabel.setText ("Sliding Box"
                        +setlabel());
                ScanType=SLACSCAN;
            }
            reveal ();
        }
    };//////END/////////
     
    
    String setlabel()
    {
        String s="-binary";
        if (F.V.isGray())
        {
            s="-2d variation";
            if(F.V.grayscantype==F.V.DIFFERENTIALBC)
                s="-differential box count";
        }
        return s;
    }
    
    
    
    /**Sets up multifractality options.
     Sets the {@link #ScanType}
     * to {@link #MFSCAN} and sets the {@link #statuslabel}
     * for a multifractal scan.*/
    ActionListener mfListener = new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {
            
            F.V.setAbort (false);
            MFVars = FLsetup.getMFInputs (MFVars);
            if(MFVars.proceed)
            {                
                F.V=MFVars;//FLsetup.SetMFOptions(F.V, MFVars);
                DoManyFilesButton.setEnabled (true);                
                statuslabel.setText ("Multifractal"
                        +setlabel()); 
                ScanType=MFSCAN;
            }
            reveal ();
        }
    };//////END/////////
        
    
    /**Clicking the {@link #DoActiveImageButton attached button}
     * on the panel starts
     * an analysis of the currently displayed window or
     * roi using the variables
     * currently set up for it.
     * Resets the
     * {@link FLVars#abort} to false and passes the active image
     * window to {@link #DoActiveImage method} to scan it.    
     *Returns and shows a message if there is no image. Returns
     *without doing anything if the image fails the 
     * {@link #IsNotCompatible compatibility test}.
     */
    public ActionListener DoActiveImageListener =            
            new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {
            F.V.setAbort (false);
            F.V.UseRoiManager=false; 
            ImageWindow wm = WindowManager.getCurrentWindow ();
            if(wm==null) { IJ.noImage (); return;}
            if(IsNotCompatible (wm.getImagePlus ())) return;
            else
            {                
                DoActiveImage(wm);
            }

        }
    };//////END/////////
    
   
     
     
    
/**Action listener that controls scans using the IJ RoiManager.
 Stops and gives a message if there is no RoiManager or no image.
 Calls {@link #IsNotCompatible compliance detector}.  If it
 runs, gets the rois from RoiManager then sets each one 
 on the image and calls {@link #DoActiveImage method} for 
 scanning the active image for each roi.*/
//public ActionListener DoRois=new ActionListener ()
//{
//    public void actionPerformed (ActionEvent e)
//    {
//            F.V.setAbort (false);
//            ImageWindow wm = WindowManager.getCurrentWindow ();
//            ImagePlus img = wm.getImagePlus ();
//            RoiManager Rm=RoiManager.getInstance ();
//             if(Rm==null){IJ.error ("No Roi Manager");return;}
//            if(wm==null){ IJ.noImage (); return;}
//            if(IsNotCompatible (img)) return;           
//            List list = Rm.getList ();
//            Hashtable rois=Rm.getROIs ();     
//            int nResults=list.getItemCount ();
//            Roi [] ROI = new Roi[nResults];
//            //for each ROI, scan the image
//        for (int k=0; k<nResults; k++)
//        {
//            String label = list.getItem (k);
//            Roi roi = (Roi)rois.get (label);
//            img.setRoi ((Roi)roi.clone ());                    
//            ROI[k]=
//                    new PolygonRoi (
//                    roi.getPolygon ().xpoints,
//                    roi.getPolygon ().ypoints,
//                    roi.getPolygon ().npoints,Roi.TRACED_ROI);
//            DoActiveImage(wm);                    
//        }
//    }
//};//////END/////////
   
public ActionListener DoRois=new ActionListener ()
{
    public void actionPerformed (ActionEvent e)
    {
            F.V.setAbort (false);
            ImageWindow wm = WindowManager.getCurrentWindow ();
            ImagePlus img = wm.getImagePlus ();
            RoiManager Rm=RoiManager.getInstance ();
             if(Rm==null){IJ.error ("No Roi Manager");return;}
            if(wm==null){ IJ.noImage (); return;}
            if(IsNotCompatible (img)) return;   
            F.V.UseRoiManager=true; 
            DoActiveImage(wm);
            F.V.UseRoiManager=false; 
    }
};//////END/////////
   
    
/**Initiates scans on the passed ImageWindow or else returns if the
 * ImageWindow is null or has no image. {@link #ResetRoiType 
 * Resets} and {@link #StoreRoiType stores} the roi type if it is not
 * a rectangle or the entire image. The scan initiated depends on the 
 * roi.  If it is null because the entire image is to be scanned, calls the
 *{@link #DoEntireImage image scanner} on the current window.  
 * If the roi is a rectangle, calls a {@link #DoARectangularRoi 
 * rectangular roi method}, or else  calls the {@link 
 * #DoIrregularRoi irregular roi} scanner. If the image is a stack, 
 * the scan is repeated for each stack, setting the ImagePlus being 
 * scanned to the current slice each time.  At the end of each stack, 
 * if none of the calls has set the {@link FLVars#abort abort} option
 * to true, calls the {@link #WriteScanData data writer} before 
 * going on to the next slice.
* @param wm ImageWindow for scanning*/
public void DoActiveImage (ImageWindow wm)
{
         ImagePlus workingcopy=wm.getImagePlus ();
         if(workingcopy==null){ IJ.noImage (); return;}
         else
         { 
             ResetRoiType ();
             workingcopy.setSlice (1);
             Roi roi =workingcopy.getRoi ();
             int roitype = (roi!=null)? roi.getType (): Roi.RECTANGLE;
             if(roitype!=Roi.RECTANGLE&&roi!=null)
                 StoreRoiType (roitype, roi, workingcopy) ;
             //if its not a rectangle or the whole image
             //find out what type of roi it is
             int lastslice=workingcopy.getStackSize ();
             for (int thisslice = 1; thisslice <= lastslice; ++thisslice)
             {//for each slice in the stack
                workingcopy.setSlice (thisslice);
                F.V.abort=false;
                if (roi==null) 
                    DoEntireImage (workingcopy, thisslice, lastslice);
                else  if (roitype==Roi.RECTANGLE)
                    DoARectangularRoi (workingcopy, thisslice, lastslice, roi);
                else if (workingcopy !=null&&roi!=null)
                    DoIrregularRoi (workingcopy, roi, roitype, thisslice, 
                            lastslice);
                if(!F.V.abort)WriteScanData (ScanType);
             }//end for each slice in the stack
             workingcopy.trimProcessor ();
         }//end scan if the image is not null
}//end function
     
     
     
     /**Further directs scans on the roi on the passed image
      * at the passed slice.
      *Scans {@link #ScanOneRectangleRoi one rectangle} 
      * by default, or if 
      * {@link 
      * FLVars#UseRectAnalyzer the rectangular sub scan option}
      * is set calls the
      * 
      * {@link #SetUpRectangleSubscans rectangular scanner} , 
      *or else if the
      * {@link FLVars#UseParticleAnalyzer particle analyzer option}
      * is selected, calls the {@link #ScanParticles particle scanner}.  
      * Or, if the {#link FLVars#RandomSubs random scanner}
      * option is chosen, , 
      * calls a function to 
      * {@link #ScanRandomRectangleSubscans scan
      * random rectangular samples} over the image.  This function
      *resets the {@link FLMain#Rois roi array} to null if the 
      *scan is the default single rectangle scan, but not the other two.
      @param workingcopy ImagePlus to scan
      @param thisslice int for the current slice to analyze
      @param lastslice int for the number of slices to analyze
      @param roi Roi on the image to scan*/
      public  boolean DoARectangularRoi (ImagePlus workingcopy,
                 int thisslice, 
                 int lastslice,
                 Roi roi)
         {
             F.V.DrawingForRectangularRoi =workingcopy;
             initializeStrings();
             if (roi!=null)workingcopy.setRoi (roi);
             if (F.V.UseParticleAnalyzer)
             {if(!ScanParticles (workingcopy, thisslice,
                      lastslice))F.V.abort=true; return false;}
             else if (F.V.UseRoiManager) 
             {if(!ScanRoiManager(workingcopy, thisslice, lastslice))
                  F.V.abort=true; return false;}
             else if (F.V.UseRectAnalyzer)
             { SetUpRectangleSubscans (workingcopy,
                       thisslice, lastslice);}
              else if (F.V.ScanSeparateRandomSubAreas)
             { if(!ScanRandomRectangleSubscans
                       (workingcopy, thisslice, lastslice))
                   F.V.abort=true;return false;}
             else
             {
                    if(thisslice==1)
                   {   F.Rois= new Roi [1];
                        F.Rois[0]= roi==null? new Roi (0, 0,
                               workingcopy.getWidth (),
                               workingcopy.getHeight ())
                               :roi;
                     }
                   F.V.IsASingleRectangularRoi =true;
                   ScanOneRectangleRoi (workingcopy, thisslice, lastslice);
                   F.V.IsASingleRectangularRoi =false;               
             }
             return true;
         }
      
      
void initializeStrings()
{
    F.V.BCdata="";
    F.V.MFdata="";
    F.V.SLACdata="";    
    F.V.GSLACdata="";
    F.V.DataString="";
    F.V.GDataString="";
    F.V.GBCdata="";
}        
         
         
/**Scans the passed irregular roi on the passed ImagePlus. The 
 * {@link #StoreRoiType stored roi} is reset on the passed image 
 * before scanning. Stores the centre x,y coordinates in the image 
 * title and stores {@link FLVars#NonRectX the x,y} of the 
 * rectangular boundary of the roi.  Resets {@link FLVars#BCdata 
 * data string} and {@link FLVars#DataString} to null;  Calls
 * {@link #copyROInow copier} to make a small copy of the roi of 
 * the image only, and stores another image for  {@link 
 * FLVars#DrawingForRectangularRoi drawing} later. Calls one of 
 * four functions to do the scan on 
 * {@link #SetUpRectangleSubscans multiple rectangles},
 * {@link #ScanParticles the particle analyzer},
 *{@link #ScanRoiManager the RoiManager},
 * or {@link #ScanOneRectangleRoi one rectangle} of the copy.
 * Closes the roi copy  when all is done.
 * @param workingcopy ImagePlus to scan
* @param roi Roi to scan
* @param roitype int for the roi.getType
* @param thisslice int for the current slice to scan
* @param lastslice int for total number of slices being scanned**/ 
public void DoIrregularRoi (ImagePlus workingcopy,
     Roi roi, int roitype, int thisslice, int lastslice)
{//if this is not a rectangular roi or the entire image
             String imgtitle="", x="", y="";
             switch (roitype)
             {  //set the roi type
                 case (Roi.OVAL):
                 {workingcopy.setRoi (op);      break;}
                 case (Roi.POLYGON):
                 {workingcopy.setRoi (pp);      break;}
                 case (Roi.COMPOSITE):
                 {workingcopy.setRoi (sp);      break;}
                 default:
                 {workingcopy.setRoi (pp);break;}
             }//end switch to find out what type of roi is selected
             if(true)
                 
             {
                 x=java.lang.Double.toString (
                         roi.getBoundingRect ().getCenterX ());
                 y=java.lang.Double.toString (
                         roi.getBoundingRect ().getCenterY ());
                 imgtitle=
                         workingcopy.getTitle ()+
                         thisslice+" roi "+x+"/" + y;
                 F.V.NonRectX=
                         (int)roi.getBounds ().getBounds2D ().getX ();
                 F.V.NonRectY=
                         (int)roi.getBounds ().getBounds2D ().getY ();
             }

             ImagePlus img2=copyROInow (workingcopy, imgtitle);
             img2.killRoi ();
             img2.setTitle (imgtitle);
             if(img2!=null)
             {
                 workingcopy.setSlice (thisslice);
                 workingcopy.updateAndRepaintWindow ();
                 F.V.DrawingForRectangularRoi=
                         workingcopy.createImagePlus ();
                 ImageProcessor ipp=workingcopy.getProcessor ();
                 F.V.DrawingForRectangularRoi.
                         setProcessor ("New", ipp);
                 
                 F.V.IsIrregularRoi =true;
                 initializeStrings();
                 
                 if (F.V.UseParticleAnalyzer)
                 {
                     if(!ScanParticles (img2, thisslice, lastslice))
                         F.V.abort=true;
                     return;
                 }
                 else if (F.V.UseRoiManager) 
                 {
                     if(!ScanRoiManager(workingcopy, thisslice, lastslice))
                        F.V.abort=true; 
                     return;
                 }
                 else if (F.V.UseRectAnalyzer)
                     SetUpRectangleSubscans (img2, thisslice, lastslice);
                 else if (F.V.ScanSeparateRandomSubAreas)
                 {
                     if(!ScanRandomRectangleSubscans
                             (img2, thisslice, lastslice))   F.V.abort=true; 
                     return;
                 }
                 else
                 {
                     if(thisslice==1)
                     {
                         F.Rois=new Roi [1];
                         img2.setRoi (0,0,img2.getWidth(), img2.getHeight());
                         F.Rois[0]=img2.getRoi ();
                     }
                     F.V.IsASingleRectangularRoi =true;
                     img2.setRoi (F.Rois[0]);
                     ScanOneRectangleRoi (img2, thisslice, lastslice);
                     F.V.IsASingleRectangularRoi =false;
                  }
                 img2.changes=false;
                 img2.getWindow ().close ();
                 F.V.IsIrregularRoi =false;                 
             }
             else IJ.showMessage ("No image");
}
        
        
         
/**Directs the scan of an entire image by calling the appropriate
* function depending on variables. Always  resets 
* {@link #initializeStrings data strings}. Calls the {@link 
* #ScanParticles particle scanner function} if {@link 
* FLVars#UseParticleAnalyzer} is true. Alternatively, calls 
* {@link #SetUpRectangleSubscans rectangular sub scan}
* starter if {@link FLVars#UseRectAnalyzer} is true. If neither
* is true, but  {@link FLVars#ScanSeparateRandomSubAreas}
* is true, calls the {@link #ScanRandomRectangleSubscans 
* random}scanner to find random rectangular subareas. 
* {@link #SetUpScans plain starter for the scanner}. In this 
* case, remakes the {@link FLMain#Rois roi array} on the first
* slice for a nonparticle scan, and stores the entire image as an 
* roi in {@link FLMain#Rois the array}. */
public void DoEntireImage (ImagePlus workingcopy,
     int thisslice, int lastslice)                 
{
             initializeStrings();
             workingcopy.setRoi (0, 0, workingcopy.getWidth (),
                     workingcopy.getHeight ());
             F.Rois=new Roi [1];
             F.Rois[0]= workingcopy.getRoi ();             
             if (F.V.UseParticleAnalyzer)
              {if(!ScanParticles (workingcopy, thisslice, lastslice))
                   F.V.abort=true; return;}
             else if (F.V.UseRoiManager) 
             {if(!ScanRoiManager(workingcopy, thisslice, lastslice))
                  F.V.abort=true; return;}
             else if (F.V.UseRectAnalyzer)
              {SetUpRectangleSubscans (workingcopy,
                       thisslice, lastslice);}
             else if (F.V.ScanSeparateRandomSubAreas)
             {if(!ScanRandomRectangleSubscans
                         (workingcopy, thisslice, lastslice))return;}
             else
             { if(thisslice==1)
               {  
                   F.Rois=new Roi [1];
                   F.Rois[0]=new Roi (0, 0,  workingcopy.getWidth (),
                           workingcopy.getHeight ());                   
               } 
               SetUpScans (workingcopy, 
                       0, 0,
                       workingcopy.getWidth (), 
                       workingcopy.getHeight (),
                       thisslice, lastslice, 
                       false, 
                       false, 
                      false,
                       0); 
             }
         }
         
         
         
      /**Resets the internal rois to null.
       @see #op
       @see #pp
       @see #sp */          
     public void ResetRoiType()
     {
         op=null;
         pp=null;
         sp=null;;
     }
     
    
     
     /**Stores the passed roi in an internal variable, chosen
      *because it matches the type of the passed roi, 
      * or if there is no match,
      *picked as the default, which is a {@link #pp polygon roi}.
      *
      *@param roitype int for the roi.gettype
      *@param roi the roi
      *@param workingcopy the ImagePlus the roi is on
      *@see #op
      @see #pp
      @see #sp */
     public void StoreRoiType(int roitype, Roi roi,
             ImagePlus workingcopy)
     {
         switch (roitype)
                        {
                            case (Roi.OVAL):
                            {op=
                                     (OvalRoi) workingcopy.getRoi (); break;}
                            case (Roi.POLYGON):
                            {pp=(PolygonRoi) workingcopy.getRoi (); break;}
                            case (Roi.POLYLINE):
                            {
                                IJ.showMessage ("Select an area (e.g. " +
                                        "use freehand selection instead" +
                                        " of freehand line).");
                                F.V.abort=true;return;}
                            case (Roi.FREELINE):
                            {
                                IJ.showMessage ("Select an area (e.g. " +
                                        "use freehand selection " +
                                        "instead of freehand line).");
                                F.V.abort=true;return;}
                            case (Roi.COMPOSITE):
                            {sp =
                                     (ShapeRoi)roi;break;}
                            default:
                            {pp = new PolygonRoi (roi.getPolygon (),
                                     roi.getType ());break;}
                        }
}
    
    /**
     * Writes the data from each scan 
     * to the appropriate results window.
     * @see FLAnalyzer.FLVars#BCdata
     * @see FLAnalyzer.FLVars#GBCdata
     * @see FLAnalyzer.FLVars#GSLACdata
     * @see FLAnalyzer.FLVars#SLACdata
     * @see FLAnalyzer.FLVars#MFdata
     * @see FLAnalyzer.FLVars#DataString
     * @see FLAnalyzer.FLVars#GDataString
     * @see #BoxCountWindow
     * @see #MultifractWindow
     * @see #SlidingLacWindow
     * @see #GBoxCountWindow
     * @see #GSlidingLacWindow
     * @see #GDataWindow
     * @see #DataWindow
     * @param TYPE int for the scan type */
    public void WriteScanData (int TYPE)
    {
        if(TYPE==GBCSCAN)
        {
            if(FirstGBCScan)
            {
                FirstGBCScan=false;
                GBoxCountWindow
                        = new TextWindow (GBoxCountTitle,
                        (F.V.GBoxCountHeadings), F.V.GBCdata, 500, 500);
            }
            else if(GBoxCountWindow.isShowing ())
                GBoxCountWindow.append (F.V.GBCdata);
            else  GBoxCountWindow =
                    new TextWindow
                    (GBoxCountTitle, (F.V.GBoxCountHeadings),
                    F.V.GBCdata, 500, 500);            
            GBoxCountWindow.setVisible (true);
        }
     
        if(TYPE==BCSCAN||TYPE==SUBSCAN)
        {
            if(FirstBCScan)
            {
                FirstBCScan=false;
                BoxCountWindow = new TextWindow (BoxCountTitle,
                        (F.V.BoxCountHeadings), F.V.BCdata, 500, 500);
            }
            else if(BoxCountWindow.isShowing ())
                BoxCountWindow.append (F.V.BCdata);
            else  BoxCountWindow =
                    new TextWindow
                    (BoxCountTitle, (F.V.BoxCountHeadings),
                    F.V.BCdata, 500, 500);            
            BoxCountWindow.setVisible (true);
        }
        
      if(TYPE==SLACSCAN&&(!F.V.isGray()))
        {
         String H=F.V.SLACHeadings;
            if(FirstSLScan)
            {
                FirstSLScan=false;
                SlidingLacWindow
                        = new TextWindow (SlidingLacTitle,
                        (H),
                        F.V.SLACdata, 500, 500);
            }
            else if(SlidingLacWindow.isShowing ())
                SlidingLacWindow.append (F.V.SLACdata);
            else  SlidingLacWindow =
                    new TextWindow ( SlidingLacTitle, H,
                    F.V.SLACdata, 500, 500);
            
            SlidingLacWindow.setVisible (true);
        }
        
      if(TYPE==SLACSCAN&&(F.V.isGray()))
        {String H=F.V.GRAYSLACHeadings;
            if(FirstGSLScan)
            {
                FirstGSLScan=false;
                GSlidingLacWindow
                        = new TextWindow (GSlidingLacTitle,
                        (H),
                        F.V.GSLACdata, 500, 500);
            }
            else if(GSlidingLacWindow.isShowing ())
                GSlidingLacWindow.append (F.V.GSLACdata);
            else  GSlidingLacWindow =
                    new TextWindow ( GSlidingLacTitle, H,
                    F.V.GSLACdata, 500, 500);
            
            GSlidingLacWindow.setVisible (true);
        }
        
     if(TYPE==MFSCAN)
        {
            if(FirstMFScan)
            {
                FirstMFScan=false;
                MultifractWindow
                        = new TextWindow (MultifractTitle,
                        (F.V.MFHeadings),
                        F.V.MFdata, 500, 500);
            }
            else if(MultifractWindow.isShowing ())
                MultifractWindow.append (F.V.MFdata);
            else  MultifractWindow =
                    new TextWindow
                    (MultifractTitle, (F.V.MFHeadings),
                    F.V.MFdata, 500, 500);
            
            MultifractWindow.setVisible (true);
        }
     
        if(F.V.ShowData)
        {
            
            if(F.V.isGray())
            {GDataWindow =
            
                    new TextWindow
                    ( GRawDataTitle, (F.V.GDataHeadings),
                    F.V.GDataString, 500, 500);
            
            GDataWindow.setVisible (true);}
            else
            {DataWindow =
            
                    new TextWindow
                    ( RawDataTitle, (F.V.DataFileHeadings),
                    F.V.DataString, 500, 500);
            
            DataWindow.setVisible (true);}
        }        
    }
    
    

/**Returns true if the passed image for the type of scanning is  
* noncompliant with the {@link FLVars#ImageType image type} 
* and cannot be made compliant, or false if it is compliant or is 
* changed to be so.  If the scan is for {@link FLVars#isGray 
* grayscale }but the image has only black and white pixels, an 
* error is shown, true is returned,and the scan is stopped.  This is 
* because the grayscale algorithm works on the max-min of the 
* histogram, rather than the absolute presence or absence of 
* foreground pixels as in a binary scan.  If the image is not GRAY8,
* GRAY16, or GRAY32, it is converted to gray8  then RGB (still gray;
* required for processing in {@link 
 * FLMain#ScanBlockOrRelativeOrGray 
* scanner}). If this fails, true is returned and an error is shown.  If
 * this  works or the image is already gray scale and the conversion 
* succeeds, then false is returned.  The conversions are done using
* IJ.run commands.  If the type of is set to {@link FLVars#isBinary 
* binary}, then if the passed image is not binary, an error is shown 
* and true returned. If the image is binary and is compliant with the
* scan type, then the background and foreground are {@link 
* FLVars#setForegroundandBackground set}  according to the 
* histogram of the passed image, and false is returned.  If the image 
* is not binary but {@link FLVars#isAutoThresh auto thresholding}
*is true, the image is {@link #Threshhold converted to binary}, the
* background and foreground value are {@link
* FLVars#setForegroundandBackground set} and false is returned.
* If this fails, an error is shown and true is returned. The test for 
* binary is stats = img.getStatistics () and if = stats.histogram[255]+
* stats.histogram[0]= stats.pixelCount it is binary.
* @param img ImagePlus to check for compliance
*@return true if image is noncompliant with scanner or false if it is 
* compliant or is made compliant */
public boolean IsNotCompatible (ImagePlus img)
    {
        boolean abort=false;
        ImageStatistics stats = img.getStatistics ();
        int BlackPlusWhite = stats.histogram[255]+stats.histogram[0];
        if(F.V.isGray())
            { 
                int t=img.getType ();
                if (t!=img.GRAY16&&t!=img.GRAY8&&t!=img.GRAY32)
                {     
                    WindowManager.setTempCurrentImage (img);
                    if(WindowManager.getTempCurrentImage()==img)
                        IJ.run("8-bit");
                    else 
                    {
                        IJ.showMessage(fail);
                        F.V.abort=true;
                        return true;
                    }            
                    t=img.getType();
                    if(t!=img.GRAY8)
                    {
                        IJ.showMessage(fail);
                         F.V.abort=true;
                         return true;
                    }
                }
                if (t==img.GRAY16||t==img.GRAY8||t==img.GRAY32)
                {                
                        WindowManager.setTempCurrentImage (img);
                        if(WindowManager.getTempCurrentImage ()==img)
                            IJ.run("RGB Color");
                        else 
                        {
                            IJ.showMessage(
                                    "Image conversion to gray RGB failed");
                            F.V.abort=true;
                            return true;
                        }
                       if(img.getType ()!=img.COLOR_RGB) 
                       {
                            F.V.abort=true;
                            IJ.showMessage("Image conversion failed");
                            return true;
                       }
                       else return false;                       
                }           
        }
        else      
        {    int t=img.getType ();         
            if (BlackPlusWhite!=stats.pixelCount&&F.V.isBinary())
            {                
                IJ.showMessage ("Binary image required.");
                F.V.abort=true;
                abort=true;
                return abort;
            }
            else  
                if (BlackPlusWhite==stats.pixelCount&& F.V.isBinary())
            {
                F.V.setForegroundandBackground (stats);
                return false;
            }
            else if
                     (BlackPlusWhite!=stats.pixelCount
                     &&F.V.isAutoThresh ())
            {
                //abort=To8 (img);
                WindowManager.setTempCurrentImage (img);
                Roi roi = img.getRoi ();
                  if (t!=img.GRAY16&&t!=img.GRAY8&&t!=img.GRAY32)
                {     
                    WindowManager.setTempCurrentImage (img);
                    if(WindowManager.getTempCurrentImage()==img)
                        IJ.run("8-bit");
                    else 
                    {
                        IJ.showMessage(fail);
                        F.V.abort=true;
                        return true;
                    }            
                    t=img.getType();
                    if(t!=img.GRAY8)
                    {
                        IJ.showMessage(fail);
                         F.V.abort=true;
                         return true;
                    }
                }
               abort=Threshhold(img);
                //if(WindowManager.getTempCurrentImage ()==img)
                //IJ.run("Threshold", "stack", "Black");
                //else {IJ.showMessage ("Error thresholding");return true;}
                if (roi!=null)img.restoreRoi ();
                stats = img.getStatistics ();
                BlackPlusWhite =stats.histogram[255]+stats.histogram[0];
                if (BlackPlusWhite!=stats.pixelCount)
                {
                    IJ.showMessage ("Error "+stats.pixelCount+" vs "
                            +BlackPlusWhite);
                    F.V.abort=true;
                    abort=true;
                    return true;
                } else 
                {
                    F.V.setForegroundandBackground(stats);
                    return false;
                }
            }
                        
        }return abort;
    }
    
     
    
    /**
     * Action listener attached to the
     * button that opens new files to automatically
     * analyze them without displaying them.  Calls
     * {@link #OpenAndScanFilesWithoutShowingThem openAndrun}
     * method to do analyses on
     * a group of images without opening them or changing the
     * settings. Resets the internal
     * variable "abort" to a value of false.
     * @param e ActionEvent from {@link #DoManyFilesButton}*/    
    public void actionPerformed (ActionEvent e)
    {        
        F.V.setAbort (false);       
        try{OpenAndScanFilesWithoutShowingThem  ();}
        catch (Throwable ev){}        
    }
        

    /**List list = Rm.getList ();
            Hashtable rois=Rm.getROIs ();     
            int nResults=list.getItemCount ();
            Roi [] ROI = new Roi[nResults];
            //for each ROI, scan the image
        for (int k=0; k<nResults; k++)
        {
            String label = list.getItem (k);
            Roi roi = (Roi)rois.get (label);
            img.setRoi ((Roi)roi.clone ());                    
            ROI[k]=
                    new PolygonRoi (
                    roi.getPolygon ().xpoints,
                    roi.getPolygon ().ypoints,
                    roi.getPolygon ().npoints,Roi.TRACED_ROI);
            DoActiveImage(wm);                    
        }*/
/**Gets the rois for all particles in a passed ImagePlus, then calls a
 * method to scan them, returning nothing. Invokes the ImageJ 
 * ParticleAnalyzer to segment the image, then passes the image 
 * and the ResultsTable containing roi coordinates to the {@link  
 * #SetUpParticles scanning method}to scan each particle. The 
 * data from the scans are accessed through the ImageJ Results 
 * window eventually; refer to the scanning method for the next 
 * step.
 * @param thisslice int for the current slice or 1 if not a stack
 * @param lastslice int for the number of slices in this stack or 1
 * if this is not a stack
 * @param img ImagePlus to scan
 * @return boolean true if not cancelled; false if the call to the
 * {@link #setupPA particle anlayzer} set up is cancelled     */
public boolean ScanRoiManager (ImagePlus img, int thisslice,
        int lastslice)
{
        boolean quit=false;
        boolean goOn=true;
        //store information
        String title = img.getTitle ();
        ImageProcessor ip = img.getProcessor ();
        Roi roi = img.getRoi ();
        if(roi==null)
        {
            img.setRoi (0,0, img.getWidth (), img.getHeight ());
            roi=img.getRoi ();
        }
        goOn=true;             
        {          
            {
                img.setRoi (roi);                
                ip.setRoi (roi);
                SetUpRM ( img, thisslice, lastslice, title);                
            }
        }return true;
    }
    
/**Gets the rois for all particles in a passed ImagePlus, then calls a
 * method to scan them, returning nothing. Invokes the ImageJ 
 * ParticleAnalyzer to segment the image, then passes the image 
 * and the ResultsTable containing roi coordinates to the {@link  
 * #SetUpParticles scanning method}to scan each particle. The 
 * data from the scans are accessed through the ImageJ Results 
 * window eventually; refer to the scanning method for the next 
 * step.
 * @param thisslice int for the current slice or 1 if not a stack
 * @param lastslice int for the number of slices in this stack or 1
 * if this is not a stack
 * @param img ImagePlus to scan
 * @return boolean true if not cancelled; false if the call to the
 * {@link #setupPA particle anlayzer} set up is cancelled     */
public boolean ScanParticles (ImagePlus img, int thisslice,
        int lastslice)
{
        boolean quit=false;
        boolean goOn=true;
        //store information
        String title = img.getTitle ();
        ImageProcessor ip = img.getProcessor ();
        Roi roi = img.getRoi ();
        if(roi==null)
        {
            img.setRoi (0,0, img.getWidth (), img.getHeight ());
            roi=img.getRoi ();
        }//get user's options for the particle analyzer
        goOn=true;
        if(thisslice==1)  { goOn=setupPA ();}//quit if cancelled here
        if (!goOn) return false;
        else//if not cancelled, go on
        {
            ImageStatistics stats = img.getStatistics ();
            if ((stats.histogram[0]+ stats.histogram[255])
            !=stats.pixelCount)
            {
                IJ.showMessage ("Binary (thresholded) image required.");
                quit=true;
            }//quit if the image is not binary
            else if (!quit)//or else go on
            {
                if (F.V.background==F.V.BLACK)
                {
                    img.killRoi ();
                    invert (ip);
                    img.updateAndRepaintWindow ();
                }                
                //get the rois
                img.setRoi (roi);
                Analyzer ana=new Analyzer ();
                int measurements = Analyzer.getMeasurements ();
                ResultsTable rt = ana.getResultsTable ();
                int options=
                        (ParticleAnalyzer.EXCLUDE_EDGE_PARTICLES|
                        ParticleAnalyzer.RECORD_STARTS
                        |ParticleAnalyzer.SHOW_NONE);
                measurements =  (ana.PERIMETER|ana.AREA);
                ParticleAnalyzer pa=new ParticleAnalyzer
                        (options, measurements, rt, F.V.minparticle,
                        F.V.maxparticle);
                Analyzer.resetCounter ();
                img.setRoi (roi);
                pa.analyze (img);
                ip.setRoi (roi);
                SetUpParticles (rt, img, thisslice, lastslice, title);                
            }
        }return true;
    }


/**Sets up several rectangular rois on the passed image.  Stores 
* the roi or sets it to the entire image before starting. The size of 
* the rectangles is {@link FLVars#Subboxsize a user variable}
* set in the {@link GUI GUI}. Determines the locations of the 
* rectangular areas of the image to scan using the image 
* dimensions and stores  the {@link FLVars#subsamples 
* number} of subareas (which is never allowed to be less than 1).
*If an roi was selected, only that roi is covered, otherwise 
*rectangular areas covering the entire image are created. These 
* rois and their locations are stored in {@link FLMain#Rois } and
* {@link FLVars#ParticleRectangleXY}. After making the 
* rectangular subscan areas, resets the roi  and calls
* {@link #SetUpEachRectanglesRoi a scanner set up} method to
* prepare to scan the multiple rectangles.
* @param img ImagePlus to analyze
* @param thisslice int for current slice
* @param lastslice int for total slices of stack to process
* @return boolean true if it executes//FIXME     */
    public boolean SetUpRectangleSubscans (ImagePlus img,
            int thisslice, int lastslice)
    {        
        //store information
        int track=0;
        String title = img.getTitle ();
        ImageProcessor ip = img.getProcessor ();
        Roi roi = img.getRoi ();
        if(roi==null)
        {
            img.setRoi (0,0, img.getWidth (), img.getHeight ());
            roi=img.getRoi ();
        }//get user's options for the particle analyzer        
        
              //get the rois
                img.setRoi (roi);
                int BigRoiWidth=roi.getBounds ().width;
                int BigRoiHeight=roi.getBounds ().height;
                int BigX = (int)roi.getBounds ().getX ();
                int BigY = (int)roi.getBounds ().getY ();
                int LeftLimit=BigX+BigRoiWidth;
                int BottomLimit=BigY+BigRoiHeight;                
                if (thisslice==1)
                {
                    int numacross =  (int)(Math.ceil ((float)
                            BigRoiWidth/(float)F.V.Subboxsize));
                    int numhigh = (int)(Math.ceil ((float)BigRoiHeight/
                            (float)F.V.Subboxsize));                    
                    if (numhigh<1) numhigh=1;
                    if (numacross<1) numacross=1;
                    int numrois = numhigh*numacross;
                    F.V.subsamples=numrois;
                    F.Rois= new PolygonRoi [F.V.subsamples];
                    F.wandx=new float[F.V.subsamples];
                    F.wandy=new float [F.V.subsamples];                    
                    F.V.ParticleRectangleXY=new int [2][F.V.subsamples];
                    //initialize this array for holding the 
                    //ips for circle drawings
                    track = 0;
                    int a = 0, b=0;
                    for (int i=0; i<numacross; i++)
                    { 
                        for(int j = 0; j < numhigh; j++)
                        { 
                            F.wandx[a] =BigX+ i*F.V.Subboxsize;
                            F.wandy[b] = BigY+j*F.V.Subboxsize;
                            int roiwidth = F.V.Subboxsize;
                            //if the roi is bigger than the image,
                            //the end will extend past the right side
                            if(((int)F.wandx[a]+roiwidth)>LeftLimit)
                                roiwidth = LeftLimit-(int)F.wandx[a];
                            int roiheight = F.V.Subboxsize;
                            //if the roi is bigger than the image,
                            //the end will extend past the bottom
                            if(((int)F.wandy[b]+roiheight)>BottomLimit)
                                roiheight = BottomLimit-(int)F.wandy[b];
                            int[]x = {(int)F.wandx[a], (int)F.wandx[a],
                                    (int)F.wandx[a]+roiwidth,
                                    (int)F.wandx[a]+roiwidth};
                            int [] y = {(int)F.wandy[b], (int)F.wandy[b]+
                                    roiheight,  (int)F.wandy[b]+roiheight, 
                                    (int)F.wandy[b]};
                            Polygon rectangle = new Polygon (x,y,4);
                            Roi R=
                                    new PolygonRoi(rectangle, Roi.TRACED_ROI);
                            boolean goodtogo=true;
                            if(!F.V.isGray ())
                            {
                                img.setRoi (R);
                                int [] H =img.getStatistics ().histogram;
                                if (H[F.V.foreground]==0)
                                    goodtogo=false;
                            }
                            if(goodtogo)
                            { if(b<numhigh)b++;  else {b=0;a++;}
                            F.Rois[track]=R;
                            F.V.ParticleRectangleXY[0][track]=
                                    (int)F.Rois [track].
                                    getBounds ().getBounds2D ().getX ();
                            F.V.ParticleRectangleXY[1][track]=
                                    (int)F.Rois [track].
                                    getBounds ().getBounds2D ().getY ();
                            track++;}
                        }
                    }
                
                if(track<F.V.subsamples)
                {//remake arrays to the proper length
                    F.V.subsamples=track;
                    int [][] temp = new int [2][track];
                    Roi [] r = new Roi [track];
                    float [] wx = new float [track], wy=new float[track];
                    for (int i = 0; i < track; i++)
                    {
                        temp[0][i]=F.V.ParticleRectangleXY[0][i];
                        temp[1][i]=F.V.ParticleRectangleXY[1][i];
                        r[i]=F.Rois[i];
                        wx[i]=F.wandx[i];
                        wy[i]=F.wandy[i];
                    }
                    F.Rois= new PolygonRoi [F.V.subsamples];
                    F.wandx=new float[F.V.subsamples];
                    F.wandy=new float [F.V.subsamples];                    
                    F.V.ParticleRectangleXY=new int [2][F.V.subsamples];
                    for (int i = 0; i < track; i++)
                    {
                        F.V.ParticleRectangleXY[0][i]=temp[0][i];
                        F.V.ParticleRectangleXY[1][i]=temp[1][i];
                        F.Rois[i]=r[i];
                        F.wandx[i]=wx[i];
                        F.wandy[i]=wy[i];
                    }                    
                    
                }
                F.V.HCImageProcessors = new
                            ImageProcessor [F.V.subsamples][lastslice];
                }
            img.setRoi (roi); 
            ip.setRoi (roi);
            SetUpEachRectanglesRoi (img, thisslice, lastslice, title);                
            return true;
    }
    
    
    
    /**Calls scans on random rectangular areas over the image.
     *First calculates rois for the 
     * {@link FLVars#subsamples requested number} of
     *subscans, and stores them in {@link FLMain#Rois rois array}.
     *@param img ImagePlus to analyze
     *@param thisslice int for the current slice
     *@param lastslice int for the number of slices to process
     *@return boolean true if it executes     */
    public boolean ScanRandomRectangleSubscans (ImagePlus img,
            int thisslice, int lastslice)
    {        
        //store information
        String title = img.getTitle ();
        ImageProcessor ip = img.getProcessor ();
        Roi roi = img.getRoi ();
        if(roi==null)
        {
            img.setRoi (0,0, img.getWidth (), img.getHeight ());
            roi=img.getRoi ();
        }        
        {
            {  //get the rois
                img.setRoi (roi);
                int BigRoiWidth=roi.getBounds ().width;
                int BigRoiHeight=roi.getBounds ().height;
                int BigX = (int)roi.getBounds ().getX ();
                int BigY = (int)roi.getBounds ().getY ();
                int LeftLimit=BigX+BigRoiWidth;
                int BottomLimit=BigY+BigRoiHeight;
                
                if (thisslice==1)
                {//make random locations                    
                    F.Rois= new PolygonRoi [F.V.subsamples];
                    F.wandx=new float[F.V.subsamples];
                    F.wandy=new float [F.V.subsamples];
                    F.V.HCImageProcessors = new
                            ImageProcessor [F.V.subsamples][lastslice];
                    F.V.ParticleRectangleXY=new int [2][F.V.subsamples];
                    //initialize this array for holding the 
                    //ips for circle drawings
                    int track = 0;
                    int []x=null, y=null;
                    Random r = new Random();
                    int j=0;
                    boolean toosparse=true;
                    for(int i = 0; i < F.V.subsamples; i++)
                    { long start = System.currentTimeMillis ();                       
                        do{
                            F.wandx[i] =BigX+ r.nextInt (BigRoiWidth);
                            F.wandy[j] = BigY+r.nextInt (BigRoiHeight);
                            int roiwidth = F.V.Subboxsize;
                            //if the roi is bigger than the image,
                            //the end will extend past the right side
                            if(((int)F.wandx[i]+roiwidth)>LeftLimit)
                                roiwidth = LeftLimit-(int)F.wandx[i];
                            int roiheight = F.V.Subboxsize;
                            //if the roi is bigger than the image,
                            //the end will extend past the bottom
                            if(((int)F.wandy[j]+roiheight)>BottomLimit)
                                roiheight = BottomLimit-(int)F.wandy[j];                            
                   
                            x =new int[] {(int)F.wandx[i], (int)F.wandx[i],
                                    (int)F.wandx[i]+roiwidth,
                                    (int)F.wandx[i]+roiwidth};                                    
                            y = new int[]{(int)F.wandy[j], (int)F.wandy[j]+
                                            roiheight,
                                            (int)F.wandy[j]+roiheight, (int)F.wandy[j]};
                           ip.setRoi((int)F.wandx[i],
                                                    (int)F.wandy[j],roiwidth, roiheight);
                            int [] h=ip.getHistogram ();
                            int z = h[F.V.getForeground()];
                            if (z==0)toosparse=true;
                            else if(((float)z/Math.pow(F.V.Subboxsize, 2))
                                  <F.V.getFACTOR ()) toosparse=true ;
                            else toosparse=false;
                            if (toosparse)
                            {long now = System.currentTimeMillis ();
                             if (now-start>10000) 
                      {Object[] options =
                      {"Click YES to reduce the samples"};
                      {
                          if (JOptionPane.showConfirmDialog (null,
                                  options, "Pixel ratio is too low. Reduce samples?",
                                  JOptionPane.YES_NO_OPTION)
                                  ==JOptionPane.YES_OPTION)
                              ;
                          toosparse=false;
                           F.V.subsamples--;
                      if (F.V.subsamples<=0)return false;}
                        }}}while (toosparse);
                            
                        Polygon rectangle = new Polygon (x,y,4);
                                            F.Rois[track]=
                                                    new PolygonRoi (rectangle, 
                                                    Roi.TRACED_ROI);
                                            F.V.ParticleRectangleXY[0][track]=
                                                    (int)F.Rois
                                                    [track].
                                                    getBounds ().getBounds2D ().getX ();
                                            F.V.ParticleRectangleXY[1][track]=
                                                    (int)F.Rois
                                                    [track].
                                                    getBounds ().getBounds2D ().getY ();
                                            track++;
                        }
                }
                img.setRoi (roi);
                ip.setRoi (roi);
                SetUpEachRectanglesRoi (img, thisslice, lastslice, title);                
            }
        }return true;
    }


/**Sets up scans of  the rois from the passed particle analyzer's 
* results table.Uses an undisplayed {@link #copyROInow 
* cropped copy}of the passed image for each {@link #SetUpScans
* scan} and makes a colour coded {@link #DrawParticles image}
*  showing the fractal dimension over each particle.  Uses the 
* ImageJ ij.gui.Wand class to automatically outline the rois using 
* x,y coordinates obtained from the 
* ij.plugin.filter.ParticleAnalyzer.
* @see FLGraphics#ColourRois colouring method
* @param thisslice int for the current slice or 1 if is not a stack
* @param lastslice int for the number of slices or 1 if not a stack
* @param rt ImageJ ResultsTable from ParticleAnalyzer scan
* @param img ImageJ ImagePlus to scan particles on
* @param title String for the image title to record     */
    public void SetUpParticles (ResultsTable rt, ImagePlus img,
            int thisslice,  int lastslice, String title)
    {
        ImageWindow win = img.getWindow ();
        int nResults = rt.getCounter ();
        ImageProcessor ip = img.getProcessor ();
        Image StoredCopyOfOriginal = ip.createImage ();
        F.wandx=new float[nResults];
        F.wandy=new float [nResults];
        F.V.colourarray = new float [nResults];
        F.V.particlenumber=0;
        F.Rois=new PolygonRoi [nResults];
        {F.V.PAHulls=new PolygonRoi[nResults];
         F.V.PACircs=new float[nResults][];}
        F.V.ParticleRectangleXY=new int[2][nResults];
        
        for (int i=0; i<nResults; i++)
        {
            F.wandx[i] =(float) rt.getValue ("XStart", i);
            F.wandy[i] =(float) rt.getValue ("YStart", i);
            Wand w = new Wand (img.getProcessor ());            
            w.autoOutline (
                    (int)F.wandx[i],(int)F.wandy[i]);
            F.Rois[i]=
                    new PolygonRoi (w.xpoints, w.ypoints,
                    w.npoints, Roi.TRACED_ROI);
            
            {F.V.ParticleRectangleXY[0][i]=
                     (int)F.Rois[i].
                     getBounds ().getBounds2D ().getX ();
             F.V.ParticleRectangleXY[1][i]=
                     (int)F.Rois[i].
                     getBounds ().getBounds2D ().getY ();}                    
        }
       
        for (int i=0; i<nResults; i++)
        {
            WindowManager.setCurrentWindow (win);
            img.setRoi (F.Rois[i]);            
            ImagePlus copied=copyROInow (img, title);
            F.V.actualx=(int)F.wandx[i];
            F.V.actualy=(int)F.wandy[i];         
            F.V.setAbort (false);
            F.V.dontclose=true;
            copied.killRoi ();
            
            SetUpScans (copied,
                    (int)F.Rois[i].getBounds ().getBounds2D ().getX (),
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getY (),
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getWidth (),
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getHeight (),
                    thisslice,
                    lastslice, 
                    true, 
                    false, 
                    true, 
                    i);
            int wOld, hOld, wNew, hNew;/*variables for borders*/
            wOld = F.Rois[i].getBounds ().width;
            /*get the image's width and height*/
            hOld = F.Rois[i].getBounds ().height;
            wNew = wOld + F.V.border;
            /* set up a new value for the added border*/
            hNew = hOld + F.V.border;
            /*set up a new value for the added border*/
            int xOff = (wNew - wOld)/2;	// offset for centering
            int yOff = (hNew - hOld)/2;	// offset for centering
            F.V.ParticleRectangleXY[0][i]=
                    F.V.ParticleRectangleXY[0][i]-xOff;
            F.V.ParticleRectangleXY[1][i]=
                    F.V.ParticleRectangleXY[1][i]-yOff;
            //get a copy of the hull and circle using the bounding box
            //of the roi, and make sure that the
            //hull and circle are not made for the
            //particle scan
            copied.changes=false;
            copied.close ();            
        }
        DrawParticles(img, thisslice, lastslice, StoredCopyOfOriginal,
                ip, title); 
    }
    
    
public void SetUpRM (ImagePlus img,  int thisslice,  int lastslice, 
   String title)
{
        RoiManager Rm=RoiManager.getInstance ();
        List list = Rm.getList ();
        Hashtable rois=Rm.getROIs ();     
        int nResults=list.getItemCount ();
        Roi [] ROI = new Roi[nResults];                   
        ImageProcessor ip = img.getProcessor ();
        Image StoredCopyOfOriginal = ip.createImage ();        
        F.V.colourarray = new float [nResults];
        F.V.particlenumber=0;
        F.Rois=new PolygonRoi [nResults];
        F.V.PAHulls=new PolygonRoi[nResults];
        F.V.PACircs=new float[nResults][];        
        F.V.ParticleRectangleXY=new int[2][nResults];
        
        for (int i=0; i<nResults; i++)
        {
            String label = list.getItem (i);
            Roi roi = (Roi)rois.get (label);
            img.setRoi ((Roi)roi.clone ());                    
            F.Rois[i]=
                    new PolygonRoi (
                    roi.getPolygon ().xpoints,
                    roi.getPolygon ().ypoints,
                    roi.getPolygon ().npoints,Roi.TRACED_ROI);    
            
            {F.V.ParticleRectangleXY[0][i]=
                     (int)F.Rois[i].
                     getBounds ().getBounds2D ().getX ();
             F.V.ParticleRectangleXY[1][i]=
                     (int)F.Rois[i].
                     getBounds ().getBounds2D ().getY ();}                    
        }
       
        for (int i=0; i<nResults; i++)
        {            
            img.setRoi (F.Rois[i]);            
            ImagePlus copied=copyROInow (img, title);
            F.V.actualx=
                    (int)F.Rois[i].getBounds ().getBounds2D ().getX ();                    
            F.V.actualy=
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getY ();     
            F.V.setAbort (false);
            F.V.dontclose=true;
            copied.killRoi ();
            
            SetUpScans (copied,
                    (int)F.Rois[i].getBounds ().getBounds2D ().getX (),
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getY (),
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getWidth (),
                    (int)F.Rois[i]. getBounds ().getBounds2D ().getHeight (),
                    thisslice,
                    lastslice, 
                    true, 
                    false, 
                    true, 
                    i);
            int wOld, hOld, wNew, hNew;/*variables for borders*/
            wOld = F.Rois[i].getBounds ().width;
            /*get the image's width and height*/
            hOld = F.Rois[i].getBounds ().height;
            wNew = wOld + F.V.border;
            /* set up a new value for the added border*/
            hNew = hOld + F.V.border;
            /*set up a new value for the added border*/
            int xOff = (wNew - wOld)/2;	// offset for centering
            int yOff = (hNew - hOld)/2;	// offset for centering
            F.V.ParticleRectangleXY[0][i]=
                    F.V.ParticleRectangleXY[0][i]-xOff;
            F.V.ParticleRectangleXY[1][i]=
                    F.V.ParticleRectangleXY[1][i]-yOff;
            //get a copy of the hull and circle using the bounding box
            //of the roi, and make sure that the
            //hull and circle are not made for the
            //particle scan
            copied.changes=false;
            copied.close ();            
        }
        DrawParticles(img, thisslice, lastslice, StoredCopyOfOriginal,
                ip, title); 
} 
      
    /**Draws particles scanned using the 
     * {@link #SetUpParticles 
     * Particle Analyzer}.  
     @param img ImagePlus to draw on
     @param thisslice int for which slice to draw on
     @param lastslice int for the last slice to work on
     @param StoredCopyOfOriginal Image stored previously,
     which will be drawn over 
     @param ip ImageProcessor of img
     @param title String for the image title*/
public void DrawParticles(ImagePlus img,  int thisslice, int lastslice,
        Image StoredCopyOfOriginal, ImageProcessor ip, String title)
{
    if (F.V.background==F.V.BLACK)
    {
            img.killRoi ();
            invert (ip);
            img.setProcessor ("Inverted", ip);
            img.updateAndDraw ();
    }   
   // if(!F.V.UseRoiManager)
    {
        img.setSlice (thisslice);
        img.updateAndRepaintWindow ();
        BufferedImage bi = new BufferedImage (img.getWidth (),
                img.getHeight (), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D)bi.getGraphics ();
        g.drawImage (StoredCopyOfOriginal, 0, 0, null);
        boolean makescale=thisslice==lastslice?true:false;
        FLGraphics.ColourRois (ip,  F.V.RoiColouringMethod,
                F.V.isGray(),    g,    F.Rois,   F.V.colourarray,
                F.V.colourcode,  F.V.LocalAlpha, F.V.comptype,
                F.V.foreground, makescale  );            
        ImagePlus imgg = new ImagePlus ("new"+title, bi);            
        if(thisslice==1)
        {
            ColouredRoisStack = new ImageStack (img.getWidth (),
                    img.getHeight ());
            ColouredRoisStack.addSlice (title, imgg.getProcessor ());
        }
        else ColouredRoisStack.addSlice (title, imgg.getProcessor ());
        if(thisslice==lastslice)
        {
            StackWindow iww = new StackWindow (new ImagePlus
                    (F.V.TitleAndSlice  + " SubScans", ColouredRoisStack));
            iww.setVisible (true);
        }
    }
    if(F.V.DRAWCircle||F.V.DRAWHull)
    {
        img.setSlice (thisslice);
        img.updateAndRepaintWindow ();
        BufferedImage ThisSliceWithHullsAndCirclesOnIt
                = new BufferedImage (img.getWidth (),
                img.getHeight (), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = (Graphics2D)
            ThisSliceWithHullsAndCirclesOnIt.getGraphics ();
        g2.drawImage (StoredCopyOfOriginal, 0, 0, null);
        F.drawshapesArrays (F.V.DRAWCircle,  F.V.DRAWHull,                   
                F.V.PAHulls,  F.V.PACircs, F.V.ParticleRectangleXY, g2 );             
        ImagePlus TSWHCOI = new ImagePlus ("new" +title, 
                ThisSliceWithHullsAndCirclesOnIt);
        if(thisslice==1)
        {
            HCImagesStack = 
                    new ImageStack (img.getWidth (), img.getHeight ());
            HCImagesStack.addSlice(title,TSWHCOI.getProcessor());
        }
        else HCImagesStack.addSlice(title,TSWHCOI.getProcessor());
        if(thisslice==lastslice)
        {
            StackWindow iww = new StackWindow (new ImagePlus 
                    (F.V.TitleAndSlice+ "Hull/Circle", HCImagesStack));
            iww.setVisible (true);
        }
    }
}     
    
 public static final float DUD=-9999.99f;
  
 
/**Is called by {@link #SetUpRectangleSubscans scanner set up
* method} for subscans. Sets each roi from {@link FLMain#Rois 
* roi array}on a {@link #copyROInow copy} of the roi and 
* {@link #SetUpScans sets up scans} for it.  If the image scan
*type is not grayscale, and if the options for finding the 
* {@link FLVars#DRAWCircle circle} and {@link 
* FLVars#DRAWHull hull} are set, calls {@link 
* FLGraphics#drawshapes drawing} function to store a dummy
* value for blocks with no scan values; this function is called by a 
* later function for scans that have a value for the fractal 
* dimension. Makes a new {@link #ColouredRoisStack} on the first
* slice, then after all of the rectanglar subareas in the array are 
* scanned, calls {@link FLGraphics#ColourRois method} to 
* colour-code according to the fractal dimensions of the sub areas,
* then shows the stack if thisslice=lastslice. Also calls {@link 
* FLMain#drawshapesArrays method} to draw the {@link 
* FLVars#DRAWCircle boundingcircle} and {@link 
* FLVars#DRAWHull convex hull} if these options are selected.
*Makes new {@link #HCImagesStack} on the first slice and shows 
* the stack of images on the last.
* @param img ImagePlus to analyze
* @param thisslice int for the current slice in the stack
* @param lastslice int for total slices to analyze
* @param title String used in {@link #copyROInow copier}  */
public void SetUpEachRectanglesRoi( ImagePlus img, int thisslice,
     int lastslice, String title)
{
        ImageWindow win = img.getWindow ();
        int nResults = F.V.subsamples;
        ImageProcessor ip = img.getProcessor ();
        Image StoredCopyOfOriginal = ip.createImage ();
        F.V.colourarray = new float [nResults];
        F.V.particlenumber=0;
        {F.V.PAHulls=new PolygonRoi[nResults];
         F.V.PACircs=new float[nResults][];}
        
        for (int i=0; i<nResults; i++)
        {
            WindowManager.setCurrentWindow (win);
            img.setRoi (F.Rois[i]);
            ImagePlus copied=copyROInow (img, title);
            F.V.actualx=(int)F.wandx[i];
            F.V.actualy=(int)F.wandy[i];         
            F.V.setAbort (false);
            F.V.dontclose=true;
            copied.killRoi ();
            ImageStatistics stats = copied.getStatistics ();
            if((!F.V.isGray())&&
                    stats.histogram[F.V.foreground]==0)
            {
                F.V.colourarray[i]=DUD; F.V.particlenumber++;
                if((F.V.DRAWCircle||F.V.DRAWHull))
                    FLGraphics.drawshapes (
                            F.V.DRAWCircle,
                            F.V.DRAWHull,
                            thisslice, 
                            lastslice,
                            null,//polyhull,
                            null,//BoundingCircle,
                            copied,//V.drawing,                          
                            //i,//thisrectangle,
                            true,//isRectangleSubScan,
                            i, 
                            null,
                            F.V.TitleAndSlice,
                            F.V);   
            }//make a blank copy of the roi if there
            //are too few pixels to analyze
            else
                SetUpScans (copied,
                        (int)F.Rois[i].getBounds ().getBounds2D ().getX (),
                        (int)F.Rois[i].getBounds ().getBounds2D ().getY (),
                        (int)F.Rois[i].getBounds ().getBounds2D ().getWidth (),
                        (int)F.Rois[i].getBounds ().getBounds2D ().getHeight (),
                        thisslice,
                        lastslice,
                        true, 
                        false, 
                        true, 
                        i);
            int wOld, hOld, wNew, hNew;/*variables for borders*/
            wOld = F.Rois[i].getBounds ().width;
            /*get the image's width and height*/
            hOld = F.Rois[i].getBounds ().height;
            wNew = wOld + F.V.border;
            /* set up a new value for the added border*/
            hNew = hOld + F.V.border;
            /*set up a new value for the added border*/
            int xOff = (wNew - wOld)/2;	// offset for centering
            int yOff = (hNew - hOld)/2;	// offset for centering
            F.V.ParticleRectangleXY[0][i]=
                    (int)F.Rois[i].getBounds ()
                    .getBounds2D ().getX ()-xOff;
            F.V.ParticleRectangleXY[1][i]=
                    (int)F.Rois[i].getBounds ()
                    .getBounds2D ().getY ()-yOff;            
            copied.changes=false;
            copied.close ();
        }
       DrawManyRectangles(img, ip, thisslice, lastslice, 
                StoredCopyOfOriginal, title);                
    }
     
    /**Draws colour coded images and hull and circle images
     *on the passed images. Is called by
     *  {@link #SetUpEachRectanglesRoi 
     *rectangle scanner}.
     *@param img ImagePlus
     *@param ip ImageProcessor
     *@param thisslice int for the slice being worked on
     *@param lastslice int for the last slice to work on
     *@param StoredCopyOfOriginal Image that will be drawn over
     *@param title String for the image title
     *@see FLMain#drawshapesArrays*/
      public void DrawManyRectangles(
              ImagePlus img, ImageProcessor ip,
              int thisslice,
              int lastslice, Image StoredCopyOfOriginal,
              String title){
            img.setSlice (thisslice);
            img.updateAndRepaintWindow ();
            BufferedImage bi
                    = new BufferedImage (img.getWidth (),
                    img.getHeight (), BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = (Graphics2D)bi.getGraphics ();
            g.drawImage (StoredCopyOfOriginal, 0, 0, null);
            boolean makescale=thisslice==lastslice?true:false;
            FLGraphics.ColourRois
                    (ip,
                    F.V.RoiColouringMethod,
                    F.V.isGray(),
                    g,
                    F.Rois,
                    F.V.colourarray,
                    F.V.colourcode,
                    F.V.LocalAlpha, F.V.comptype,
                    F.V.foreground, makescale
                    );
            
            ImagePlus imgg = new ImagePlus ("new"+title, bi);
            
            if(thisslice==1)
            {
                ColouredRoisStack = new ImageStack (img.getWidth (),
                        img.getHeight ());
                ColouredRoisStack.addSlice (title, imgg.getProcessor ());
            }
            else ColouredRoisStack.addSlice (title, imgg.getProcessor ());
            if(thisslice==lastslice)
            {
                StackWindow iww = new StackWindow (new ImagePlus (
                        F.V.TitleAndSlice+" Subscans",
                        ColouredRoisStack));
                iww.setVisible (true);
            }            
            
            if(F.V.DRAWCircle||F.V.DRAWHull)
            {img.setSlice (thisslice);
             img.updateAndRepaintWindow ();
             BufferedImage ThisSliceWithHullsAndCirclesOnIt
                     = new BufferedImage (img.getWidth (),
                     img.getHeight (), BufferedImage.TYPE_INT_ARGB);
             Graphics2D g2 =
                     (Graphics2D)ThisSliceWithHullsAndCirclesOnIt.
                     getGraphics ();
             g2.drawImage (StoredCopyOfOriginal, 0, 0, null);
             F.drawshapesArrays (
                     F.V.DRAWCircle,
                     F.V.DRAWHull,
                     F.V.PAHulls,
                     F.V.PACircs,
                     F.V.ParticleRectangleXY,
                     g2 );             
             
             ImagePlus TSWHCOI =
                     new ImagePlus ("new"+title,
                     ThisSliceWithHullsAndCirclesOnIt);
             
             if(thisslice==1)
             {
                 HCImagesStack = new ImageStack (img.getWidth (),
                         img.getHeight ());
                 HCImagesStack.addSlice
                         (title, TSWHCOI.getProcessor ());
             }
             else HCImagesStack.addSlice
                     (title, TSWHCOI.getProcessor ());
             if(thisslice==lastslice)
             {
                 StackWindow iww =
                         new StackWindow (new
                         ImagePlus (F.V.TitleAndSlice+
                         " Hull/Circle", HCImagesStack));
                 iww.setVisible (true);
             }} 
        }
      
      
      
    /**Stores coordinates then 
     * calls {@link #SetUpOneRectangularRoi method}
     *to scan a rectangular roi.  If there 
     *is no roi on the passed ImagePlus, then the entire image is used.
     * 
     * @param img ImagePlus to scan
     * @param thisslice int for the current slice
     * @param lastslice int for the number 
     * of slices to process in the stack
     * @return boolean true when done */
    public boolean ScanOneRectangleRoi (ImagePlus img,
            int thisslice, int lastslice)
    {
        boolean quit=false;
        boolean goOn=true;
        //store information
        String title = img.getTitle ();
        ImageProcessor ip = img.getProcessor ();
        Roi roi = img.getRoi ();
        if(roi==null)
        {
            img.setRoi (0,0, img.getWidth (), img.getHeight ());
            roi=img.getRoi ();
        }        
        {
            {                
                img.setRoi (roi);
                int BigRoiWidth=roi.getBounds ().width;
                int BigRoiHeight=roi.getBounds ().height;
                int BigX = (int)roi.getBounds ().getX ();
                int BigY = (int)roi.getBounds ().getY ();
                int LeftLimit=BigX+BigRoiWidth;
                int BottomLimit=BigY+BigRoiHeight;                
                if (thisslice==1)
                {
                    int numacross =1;                   
                    int numhigh =1;
                    int numrois = 1;
                    F.wandx=new float[numrois];
                    F.wandy=new float [numrois];
                    F.V.HCImageProcessors =
                            new ImageProcessor [numrois][lastslice];
                    F.V.ParticleRectangleXY=new int [2][numrois];                  
                    int track = 0;
                    for (int i=0; i<numacross; i++)
                    {
                        for(int j = 0; j < numhigh; j++)
                        {
                            F.wandx[i] =BigX+ i*BigRoiWidth;
                            F.wandy[j] = BigY+j*BigRoiHeight;
                            int roiwidth = BigRoiWidth;
                            //if the roi is bigger than the image,
                            //the end will extend past the right side
                            if(((int)F.wandx[i]+roiwidth)>LeftLimit)
                                roiwidth = LeftLimit-(int)F.wandx[i];
                            int roiheight = BigRoiHeight;
                            //if the roi is bigger than the image,
                            //the end will extend past the bottom
                            if(((int)F.wandy[j]+roiheight)>BottomLimit)
                                roiheight = BottomLimit-(int)F.wandy[j];                            
                            F.V.ParticleRectangleXY[0][track]=
                                    (int) F.Rois[track].
                                    getBounds ().getBounds2D ().getX ();
                            F.V.ParticleRectangleXY[1][track]=
                                    (int) F.Rois[track].
                                    getBounds ().getBounds2D ().getY ();
                            track++;}}
                }
                img.setRoi (roi);
                ip.setRoi (roi);
                SetUpOneRectangularRoi (img, thisslice, lastslice, title);                
            }
        }return true;
    }
    
    
    
    
    /**Scans the passed image as a rectangular roi.
     * Sets the roi
     * using a {@link FLMain#Rois stored roi},
     * then {@link #copyROInow copies}
     * the roi to an image and {@link #SetUpScans sets up scans}
     * on it.  Resets
     * {@link FLVars#colourarray the colours storage array}
     * before scanning, so stored results are lost.  Does the same for
     * {@link FLVars#PAHulls hulls array} and
     * {@link FLVars#PACircs circles array}.
     * Stores the roi coordinates and size.     
     * Calls {@link FLMain#drawshapesArrays} to draw
     * the {@link FLVars#DRAWCircle boundingcircle} and
     * {@link FLVars#DRAWHull convex hull}
     * if these options are selected.
     * Makes new {@link #HCImagesStack} on the first slice and
     * shows the stack of images on the last.  Stores offsets
     * for drawing. If its an
     * {@link FLVars#IsIrregularRoi irregular roi}
     * , calls a different
     * {@link #DrawSingleRectangleDrawings drawing} method.
     *  @param img ImagePlus to scan
     *  @param thisslice int for current slice
     *  @param lastslice int for total slices to scan on stack
     *  @param title String for describing the image and roi */
    public void SetUpOneRectangularRoi
            ( ImagePlus img, int thisslice, int lastslice, String title)
    {
        ImageWindow win = img.getWindow ();
        int nResults = 1;
        ImageProcessor ip = img.getProcessor ();
        Image StoredCopyOfOriginal = ip.createImage ();
        
        F.V.colourarray = new float [nResults];
        F.V.particlenumber=0;        
        {F.V.PAHulls=new PolygonRoi[nResults];
         F.V.PACircs=new float[nResults][];}
        
        for (int i=0; i<nResults; i++)
        {
            WindowManager.setCurrentWindow (win);
            img.setRoi (F.Rois[i]);
            ImagePlus copied=copyROInow (img, title);
            F.V.actualx=(int)F.wandx[i];
            F.V.actualy=(int)F.wandy[i];         
            F.V.setAbort (false);
            F.V.dontclose=true;
            copied.killRoi ();            
            SetUpScans (copied,
                    (int)F.Rois[i].
                    getBounds ().getBounds2D ().getX (),
                    (int)F.Rois[i].
                    getBounds ().getBounds2D ().getY (),
                    (int)F.Rois[i].
                    getBounds ().getBounds2D ().getWidth (),
                    (int)F.Rois[i].
                    getBounds ().getBounds2D ().getHeight (),
                    thisslice, lastslice, true, false, true, i);
            copied.changes=false;
            copied.close ();
        }
        DrawSingleRectangleDrawings(img, thisslice, 
                lastslice, StoredCopyOfOriginal, title);   
    }
    
    
    
    
    /**Sets up and calls for bounding circles and convex hulls 
     *to be drawn on the image.
     @param img ImagePlus     
     @param thisslice int for the current slice
     @param lastslice int for the last slice to process
     @param StoredCopyOfOriginal Image to draw on
     @param title String for the image's title*/
    public void DrawSingleRectangleDrawings(ImagePlus img,            
              int thisslice,
              int lastslice, Image StoredCopyOfOriginal,
              String title){            
            
            img.setSlice (thisslice);
            img.updateAndRepaintWindow ();
            int [][] offsets=F.V.ParticleRectangleXY;
            int [][] newoffsets = 
                    new int [2][F.V.ParticleRectangleXY[0].length];
            for (int i = 0; i < offsets[0].length; i++)
            {newoffsets[0][i]=offsets[0][i]+F.V.NonRectX;
             newoffsets[1][i]=offsets[1][i]+F.V.NonRectY;
            }
            if (F.V.IsIrregularRoi)
            {
                int wOld, hOld, wNew, hNew;/*variables for borders*/
                wOld = F.Rois[0].
                        getBounds ().width;
                /*get the image's width and height*/
                hOld =  F.Rois[0].
                        getBounds ().height;
                wNew = wOld + F.V.border;
                /* set up a new value for the added border*/
                hNew = hOld + F.V.border;
                /*set up a new value for the added border*/
                int xOff = (wNew - wOld)/2;	// offset for centering
                int yOff = (hNew - hOld)/2;	// offset for centering
                int XXX=(int)F.Rois[0].
                        getBounds ().getX ()-xOff;
                int YYY=(int)F.Rois[0].
                        getBounds ().getY ()-yOff;
                offsets=new int [] []{{XXX}, {YYY}};
                for (int i = 0; i < offsets[0].length; i++)
                {newoffsets[0][i]=offsets[0][i]+F.V.NonRectX;
                 newoffsets[1][i]=offsets[1][i]+F.V.NonRectY;
                }
            }
            
            else if(!F.V.IsIrregularRoi)
            {
                int wOld, hOld, wNew, hNew;/*variables for borders*/
                wOld = F.Rois[0].getBounds ().width;
                hOld =  F.Rois[0].getBounds ().height;
                wNew = wOld + F.V.border;
                /* set up a new value for the added border*/
                hNew = hOld + F.V.border;
                /*set up a new value for the added border*/
                int xOff = (wNew - wOld)/2;	// offset for centering
                int yOff = (hNew - hOld)/2;	// offset for centering
                float XXX= (int)F.Rois[0].getBounds ().getX ()-xOff;
                float YYY=(int)F.Rois[0].getBounds ().getY ()-yOff;
                newoffsets[0][0]=(int)XXX;
                newoffsets[1][0]=(int)YYY;
            }
            
            if(F.V.DRAWCircle||F.V.DRAWHull)
            {
                BufferedImage ThisSliceWithHullsAndCirclesOnIt
                        = new BufferedImage (img.getWidth (),
                        img.getHeight (), BufferedImage.TYPE_INT_ARGB);
                Graphics2D g2 = (Graphics2D)
                        ThisSliceWithHullsAndCirclesOnIt.getGraphics ();
                g2.drawImage (StoredCopyOfOriginal, 0, 0, null);
                F.drawshapesArrays (
                        F.V.DRAWCircle,
                        F.V.DRAWHull,
                        F.V.PAHulls,
                        F.V.PACircs,
                        F.V.IsIrregularRoi?offsets:newoffsets,
                        g2 );                
                
                ImagePlus TSWHCOI =
                        new ImagePlus ("new"+title,
                        ThisSliceWithHullsAndCirclesOnIt);                
             
                {if(thisslice==1)
                 {
                 HCImagesStack =
                             new ImageStack(img.getWidth (), img.getHeight ());
                 HCImagesStack.addSlice(title,TSWHCOI.getProcessor());
                 }
                 else
                 HCImagesStack.addSlice (title, TSWHCOI.getProcessor ());
                 if(thisslice==lastslice)
                 {
                     StackWindow iww = new StackWindow 
                             (new ImagePlus (F.V.TitleAndSlice+
                             " Hull/Circle", HCImagesStack));
                     iww.setVisible (true);
                 }}
                        
                        if(F.V.IsIrregularRoi)
                            DrawHullandCircle (F.V.DrawingForRectangularRoi,
                                    thisslice, lastslice, newoffsets);
            }
        }
    
    
    
    
    /**Draws the hull and circle.
     * If neither {@link FLVars#DRAWCircle circle} nor 
     * {@link FLVars#DRAWHull hull}
     * option is not set, does nothing.  Otherwise, calls 
     * {@link FLMain#drawshapesArrays drawing} method.
     * Makes a new {@link #BigHCImagesStack} if thisslice=1, then
     * adds a new slice every time thisslice is not one, and
     * shows the stack when thisslice=lastslice.
     * @param img ImagePlus to work on
     * @param thisslice int for current slice in the stack
     * @param lastslice int for total slices to use
     * @param offset offset for drawing */
    public void DrawHullandCircle (ImagePlus img, 
            int thisslice, int lastslice,
            int [][] offset)
    {
        if(F.V.DRAWCircle||F.V.DRAWHull)
        {
            BufferedImage ThisSliceWithHullsAndCirclesOnIt
                    = new BufferedImage (img.getWidth (),
                    img.getHeight (), BufferedImage.TYPE_INT_ARGB);
            Graphics2D g2 =
                    (Graphics2D)
                    ThisSliceWithHullsAndCirclesOnIt.getGraphics ();
            g2.drawImage (img.getImage (), 0, 0, null);
            F.drawshapesArrays (
                    F.V.DRAWCircle,
                    F.V.DRAWHull,
                    F.V.PAHulls,
                    F.V.PACircs,
                    offset,
                    g2 );
            
            String title = "newpics";
            ImagePlus TSWHCOI =
                    new ImagePlus (title, ThisSliceWithHullsAndCirclesOnIt);
            
            if(thisslice==1)
            {
                BigHCImagesStack
                        = new ImageStack (img.getWidth (), img.getHeight ());
                
            }
            BigHCImagesStack.addSlice (title, 
                    TSWHCOI.getProcessor ());
            if(thisslice==lastslice)
            {
                StackWindow iww =
                        new StackWindow (new ImagePlus 
                        (F.V.TitleAndSlice+
                        "Hull/Circle", BigHCImagesStack));
                iww.setVisible (true);
            }}}
    
    
    
    
    /**Stack for showing hull and circle.
     @see #DrawParticles
     @see #DrawHullandCircle*/
    public ImageStack HCImagesStack, BigHCImagesStack;
        
    
    
    /**Main method in case user wants to double click. Starts ImageJ.
     * @param args String array; if "about" shows about method. */
    public static void main (String args[])
    {
        String  s = "txt";
        new ImageJ ();
        IJ.run ("FracLac 2.4e");
    }
    
    
    /**Main method in case user wants to double click. Starts ImageJ.
     * @param args String array; if "about" shows about method. */
    public  void run (String args[])
    {
        String  s = "txt";
        new ImageJ ();
        IJ.run ("FracLac 2.4e");
    }
    
    
    
    /**Describes FracLac when invoked by the ImageJ Help menu. */
    public void showAbout ()
    {
        IJ.showMessage ("FracLac 2.4e (FracLac_.jar)\n"
                + "This plugin analyzes binary or grayscale images\n"
                + "using multiple grid " 
                + "scans, subscans, and mass analysis. \n"
                + "It delivers box counting and mass\n"
                + "fractal dimensions (D) as well " 
                +  "as lacunarity ("+FLMain.Lambda
                + "), the convex \n"
                + " hull and bounding circle, "
                + " and other morphometrics. \n"
                + "FracLac scans using overlapping \n"
                + "and nonoverlapping "
                + "algorithms and delivers smoothing \n"
                + " and minimal"
                + " covering data. It is suitable for "
                + " analyzing subtle differences \n"
                + "in textures and patterns, such as those obtained from " 
                + "biological tissue and cells. \n"
                + "It generates colour coded graphics showing the \n"
                + "distribution of Ds " 
                + "over images segmented \n"
                + "in grids or using the particle analyzer \n"
                + "and delivers the mass distribution as well as the \n"
                + "Generalized Dimension and  "+FLMain.falpha
                + " spectra for multifractal analysis. \n"
                + "Public Domain: \n"
                + "This plugin for ImageJ is freely available\n"
                + "to anyone to use or modify, in the same way that\n"
                + "ImageJ is.  Please acknowledge it if\n"
                + "you use it for published research (see  \n"
                + "FracLac " 
                + "Instructions for references. JavaDoc can be\n"
                + "generated from the source files.  Request a \n"
                + "User's Guide or fix from or please report bugs\n"
                + "to the author, A. Karperien, Charles Sturt University\n"
                + "at akarpe@postoffice.csu.edu.au."
                );
    }
    
    
    
    
    /**Code implemented is from ImageJ.
     * @param title String for the images title
     * @param IPlus ImagePlus to copy
     * @return ImagePlus that is the copied and binarized roi */
    public ImagePlus copyROInow (ImagePlus IPlus, String title)
    {
        int width=(int)IPlus.getRoi ().getBoundingRect ().getWidth () ;
        int height=(int)IPlus.getRoi ().getBoundingRect ().getHeight ();
        WindowManager.setTempCurrentImage (IPlus);
        IJ.run ("Copy");
        String colour=F.V.background==F.V.BLACK?"Black":"White";
        String TITLE="FL-image";
         if(!F.V.isGray())
             IJ.newImage (TITLE, "8-bit "+colour, width, height, 1);
         else   IJ.newImage (TITLE, "RGB", width, height, 1);
        ImagePlus imp = WindowManager.getImage (TITLE);
        imp.setRoi (0, 0,  width, height);
        if(F.V.isGray())
        {
            imp.setColor (F.V.filler);    
            WindowManager.setTempCurrentImage (imp);
            IJ.run("Fill");
        }
       
        imp.killRoi ();
       WindowManager.setTempCurrentImage (imp);
       IJ.run ("Paste");
        imp.setTitle (title+"roi");       
        ImageProcessor ip = imp.getProcessor ();        
        return imp;
        
    }    
     
    
/**Sets up then sends the area to be scanned to the scan controller.
 *@see FLMain#SendImage
 *@param img ImagePlus the scan is to be on
 *@param XForCheckedArea int x coordinate of area to scan
 *@param YForCheckedArea int y coordinates of area to scan
 *@param WidthOfCheckedArea int for scan area's width
 *@param HeightOfCheckedArea int for scan area's height
 *@param thisslice int for the current slice
 *@param lastslice int for the last slice to process
 *@param SHOW boolean false if scanning entire image, false
 *if scanning subscans or an roi
 *@param file boolean true if scanning from the file opener
 *@param isRectangleSubScan boolean true if this is a rectangle
 *sub scan
 *@param ThisRectangleSubScan int for which scan it is when
 *doing multiple areas on an image; 0 for entire image  */
public boolean SetUpScans (
        ImagePlus img,
        int XForCheckedArea,
        int YForCheckedArea,
        int WidthOfCheckedArea,
        int HeightOfCheckedArea,
        int thisslice, int lastslice,
        boolean SHOW, //is false if called from DoEntireImage
        boolean file, 
        boolean isRectangleSubScan,
        int ThisRectangleSubScan
        )
{
        ImagePlus imgg=null;
        boolean worked=false;
        boolean grabbed=false;
        F.V.TitleAndSlice=img.getTitle ()+"S"+thisslice;
        Roi roi=null;
        if (img.getRoi ()!=null) roi = img.getRoi ();      
        if(!file)
        {if (!SHOW)//SHOW is false when called from DoEntireImage
         {if (roi!=null)//it is not null when called from DoEntireImage
          {//for an entire image
//at this point, an open image will be scanned, with its roi set to all
              //because the roi is
              //set to all if there is no roi on an active image and there is
              //no particle analyzer or rectangle analyzer being used
              String x=java.lang.Double.toString (
                      roi.getBoundingRect ().getCenterX ());
              String y=java.lang.Double.toString (
                      roi.getBoundingRect ().getCenterY ());             
              imgg=copyROInow (img, F.V.TitleAndSlice);
              grabbed=true;
          }}
         if(SHOW)//show is true for roi manager scans
         {//roi manager scans here
             if (roi!=null)
             {
                 String x=java.lang.Double.toString (
                         roi.getBoundingRect ().getX ());
                 String y=java.lang.Double.toString (
                         roi.getBoundingRect ().getY ());
                 F.V.TitleAndSlice=img.getTitle ()+
                        "S"+thisslice+" roi x,y="+x+"/" + y;               
                 imgg=WindowManager.getCurrentImage ();
                 grabbed=true;
             }
         }
         //              grab the roi instead of the entire image if
         //              an roi is selected...nice feature of ImageJ 
        }
        ImageProcessor ip =
                grabbed? imgg.getProcessor ():img.getProcessor (); 
        if (ip!=null)
        {
            F.V.setAbort (false);            
            worked =F.SendImage ( 
                    F.V.TitleAndSlice, 
                    grabbed?imgg:img,
                    XForCheckedArea,
                    YForCheckedArea,
                    WidthOfCheckedArea,
                    HeightOfCheckedArea,
                    thisslice, lastslice,                      
                    isRectangleSubScan, 
                    ThisRectangleSubScan);
          
            if(!worked)F.V.setAbort (true);
            //do the analysis if there is an  image to work on
            IJ.showStatus ("Done");
            if(!F.V.isAbort ()) if(grabbed)imgg.updateAndDraw ();
            img.updateAndDraw ();
            
        }        
       else {IJ.showMessage ("No Image");return false;}
       if(imgg!=null){ imgg.changes=false;  imgg.close ();}
        return worked;
    }
  

/**Wayne Rasband's ImageJ code to invert an image, specifically 
 * for binary files.
 *@param ip ImageProcessor on which to work */
public void invert (ImageProcessor ip)
{
        byte[] pixels = (byte[])ip.getPixels ();
        int width = ip.getWidth ();
        Rectangle r = ip.getRoi ();
        int offset, i;
        for (int y=r.y; y<(r.y+r.height); y++)
        {
            offset = y*width;
            for (int x=r.x; x<(r.x+r.width); x++)
            {
                i = offset + x;
                //	pixels[i] = (byte)(255-pixels[i]);
                
                if(pixels[i]==(byte)(0))pixels[i]=(byte)255;
                
                else if(pixels[i]==(byte)255)pixels[i]=(byte)0;
            }
        }
    }
    
     
/**Sets up the ij.plugin.filter.ParticleAnalyzer using
* a ij.gui.GenericDialog to get values for the minimum and
* maximum particle size to find, and the background and
* foreground arrangement (i.e., which is white and which is black).
* Changes these variables in the {@link FLAnalyzer.FLVars variable
 * class}.
* @return boolean false if cancelled or else true */
public boolean setupPA ()
{
        GenericDialog gd
                = new GenericDialog ("Particle Options", IJ.getInstance ());
        String [] C =
        {"Black", "White"};
        String ch = C[0];
        if (F.V.background==F.V.WHITE)ch=C[1];
        gd.addChoice ("Background Colour", C, ch);
        gd.addNumericField ("Minimum particle", F.V.minparticle, 0);
        gd.addNumericField ("Maximum particle", F.V.maxparticle, 0);
        gd.showDialog ();
        if (gd.wasCanceled ())
        {
            return false;}
        else
        {
            int t=gd.getNextChoiceIndex ();
            if (t==0)
            {F.V.ChooseBackground=F.V.USEBLACK;
             F.V.background=F.V.BLACK; F.V.foreground=F.V.WHITE;}
            else
            {F.V.ChooseBackground=F.V.USEWHITE;
             F.V.background=F.V.WHITE;F.V.foreground=F.V.BLACK;}
            
            Toolbar.setBackgroundColor (F.V.background
                    ==F.V.BLACK?Color.black:Color.white);
            F.V.minparticle=(int)gd.getNextNumber ();
            F.V.maxparticle=(int)gd.getNextNumber ();
        }
        return true;
    }

    /**Makes a tree displaying colour 
     * schemes for subscans.  Has an internal
     * selection listener and cellRenderer
     * that changes the colour code
     * variable in the global FLVars instance 
     * {@link FLVars#colourcode} according
     * to the node selected.  The rendere shows .*/
    public void makeTree ()
    {
        //make top node        
        DefaultMutableTreeNode top =
                new DefaultMutableTreeNode ("Colours");
        createNodes (top);
        
//        //make tree and its viewing pane
        final JTree tree = new JTree (top);
        tree.setToolTipText ("choose colour scheme");
        treeView = new JScrollPane (tree);
        
        tree.getSelectionModel ().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
//        //Listens for when the type of image selected changes.
        tree.addTreeSelectionListener (
                new TreeSelectionListener ()
        {
            boolean quit=false;
            public void valueChanged (TreeSelectionEvent e)
            {
                DefaultMutableTreeNode node = 
                        (DefaultMutableTreeNode)               
                tree.getLastSelectedPathComponent ();
                if (node == null) return;
                Object nodeInfo = node.getUserObject ();
                if (node.isLeaf ())
                {
                    try
                    {
                        F.V.colourcode
                                =node.getParent ().getIndex (node);
                    }
                    catch (NumberFormatException ex)
                    {F.V.colourcode=0;}
                    if (F.V.colourcode==11)
                        FLsetup.getUserColourGuides (F.V);
                }
            }
        }
        );
        tree.setCellRenderer (new MyRenderer ());
    }
    
    
    /**Creates nodes for the colour code tree that
     * the gui raises.  The tree is used to set the
     * value of the colour coding scheme
     * in {@link FLAnalyzer.FLVars#colourcode}.
     *
     * @param   top DefaultMutableTreeNode for the tree */
    public void createNodes (DefaultMutableTreeNode top)
    {
        DefaultMutableTreeNode A = null;
        A=new DefaultMutableTreeNode ("Colours");
        DefaultMutableTreeNode a = null;
        top.add (A);
        
        for (int i = 0; i <FLGraphics.NUMCOLOURCODES; i++)
        {
            a = 
                    new DefaultMutableTreeNode 
                    ((Object)Integer.toString (i));
            A.add (a);
        }
    }
    
    
   
    /**
     * Opens and processes selected files
     * without showing them on screen.
     * Reuses image variable for each opened image.  Calls
     * {@link FLAnalyzer.FLMain#SendImage method} 
     * using a global
     * instance of FLAnalyzer.FracLac invoked by its class's
     * {@link #run run method}
     * once the image is open.       
     * @throws java.lang.Throwable if file is null
     */
    public void OpenAndScanFilesWithoutShowingThem
            () throws Throwable
    {
        //ask if the user wants to use the roi manager if this
        //scan is not a subscan
        if (ScanType!=SUBSCAN)
        {
            if (!IJ.showMessageWithCancel ( "Use Roi Manager", 
            "Use the Roi Manager for these images?"));
            else F.V.UseRoiManager=true;
                     
        }
        ////////////////////variables///////////////////////////
        JFileChooser fc = null;
        double returnVal;/*for file chooser*/
        File[] files;/*array of files to be opened*/
        Opener opener;/*opener for opening images*/
        String data="";
        //                          select files with a file chooser
        try
        {fc = new JFileChooser ();}
        catch (Throwable e)
        {
            IJ.error ("This plugin requires Java 2 or Swing.");
            return;
        }
        fc.setMultiSelectionEnabled (true);
        
        returnVal = fc.showOpenDialog (IJ.getInstance ());
        if (returnVal!=JFileChooser.APPROVE_OPTION)return;
        files = fc.getSelectedFiles ();
        
        
        //                            store the list of files to be opened
        if (files.length==0)
        {
            //                            getSelectedFiles does not
            //                              (Aug 2, 2005)work on some JVMs FIXME
            files = new File[1];
            files[0] = fc.getSelectedFile ();
        }
        
        String sdir = fc.getCurrentDirectory ().toString ();
        //                            save the name of the directory for later
        
        
        opener = new Opener ();
        //                  make a file opener to use with the analysis routines
        
        
        for (int i=0; i<files.length; i++)
            //                 work through the list of files one by one
            
        {
            
            ImagePlus img = opener.openImage (sdir, files[i].getName ());
            
            String title=img.getTitle ();
            F.V.abort=IsNotCompatible (img);
            if(!F.V.abort)
            {
                int lastslice=img.getStackSize ();
                for (int thisslice = 1; thisslice <= lastslice; ++thisslice)
                {
                    img.setSlice (thisslice);
                    F.V.TitleAndSlice=title+"S"+thisslice;
                   initializeStrings();
                    if(thisslice==1)
                    {   F.Rois=
                                new Roi [1];
                        F.Rois[0]=
                                new Roi (0, 0,
                                img.getWidth (),
                                img.getHeight ())
                                ;
                    }
                                       
                    F.SendImage (F.V.TitleAndSlice, img,
                            0,0,
                            img.getWidth (),
                            img.getHeight (),
                            thisslice, lastslice, 
                            false, 0);
                   
                    if(!F.V.isAbort ())WriteScanData (ScanType);
                }}}//                  end loop through list of images
        
    }
    
    /**
     * a window event
     * @param windowEvent not specified
     */
    public void windowActivated (WindowEvent windowEvent)
    {
    }
    /**
     * not specitifed
     * @param windowEvent not specified */
    public void windowClosed (WindowEvent windowEvent)
    {
    }
    /**
     * a window event
     * @param windowEvent not specified
     */
    public void windowDeactivated (WindowEvent windowEvent)
    {
    }
    /**
     * a window event
     * @param windowEvent not specified
     */
    public void windowDeiconified (WindowEvent windowEvent)
    {
    }
    /**
     * a window event
     * @param windowEvent not specified
     */
    public void windowIconified (WindowEvent windowEvent)
    {
    }
    /**
     * a window event
     * @param windowEvent not specified
     */
    public void windowOpened (WindowEvent windowEvent)
    {    }
    
    
    
    /**Closes all open windows except the
     * {@link #BoxCountWindow hull and circle results Windows}
     *
     * Does not check to save other  
     * ResultsWindows that may be open.
     * This method calls 
     * {@link ij.WindowManager#closeAllWindows window 
     * closing method}.
     * Before calling it, temporary copies
     * of the Hull and Circle and Test results windows
     * are stored, then after they are remade.*/
    ActionListener CloseWindows= new ActionListener ()
    {
        public void actionPerformed (ActionEvent e)
        {
            boolean closeTextToo = IJ.showMessageWithCancel (
                    "Close results windows", "Close all results " +
                    "without saving?");
            Frame[] list = ij.ImageJ.getFrames ();              
		for (int i=0; i<list.length; i++) {
			Frame frame = list[i];
                        if (frame instanceof ImageWindow)
                            ((ImageWindow)frame).close ();
                        if(closeTextToo)if(frame instanceof TextWindow)
                            ((TextWindow)frame).close();
		}
        }
    };//////END/////////
    
    /**Code implemented is from ImageJ.
     * @param imp ImagePlus to convert to 8-bit.
     * @return boolean true if there is a problem when 
     * {@link #runit} is called, otherwise false     */
    public boolean Threshhold (ImagePlus imp)
    {     
        Roi r = imp.getRoi ();
        if( runit (imp))return true;
         if(r==null);
         else imp.setRoi (r);
        return false;
    }
   
    
    
    
    /**
     * Code implemented is from ImageJ.
     * @see #Threshhold
     * @param imp ImagePlus to continue converting to binary.
     * @return boolean true if it fails, false if it succeeds in calling
     *{@link #applyThreshold}
     */
    public boolean runit (ImagePlus imp)
    {
        if (imp==null)
        {
            IJ.noImage (); return true;}
        if (imp.getStackSize ()==1)
        {
            Undo.setup (Undo.COMPOUND_FILTER, imp);
            applyThreshold (imp);
            Undo.setup (Undo.COMPOUND_FILTER_DONE, imp);
        }
        else
        {
            Undo.reset ();
            applyThreshold (imp);
        }
        return false;
    }
    /**Code implemented is from ImageJ.*/
    void applyThreshold (ImagePlus imp)
    {
        int n  = imp.getStackSize ();
        for (int j = 0; j <= n; j++)
        {
        if (!imp.lock ()) return;
        imp.setSlice (j); 
                imp.killRoi ();
        ImageProcessor ip = imp.getProcessor ();
        ip = imp.getProcessor ();
            autoThreshold (imp);        
        int fcolor =Color.black.getRGB (), bcolor=Color.white.getRGB();
        ip.resetThreshold ();
        int[] lut = new int[256];
        for (int i=0; i<256; i++)
        {
            if (i>=minThreshold && i<=maxThreshold)
                lut[i] = fcolor//:(byte)i
            ;
            else
            {
                lut[i] =bcolor;//:(byte)i                      
            }
        }
            ip.applyTable (lut);
        imp.updateAndDraw ();
        imp.unlock ();
        }
    }
    
    /**Code implemented is from ImageJ.*/
    void autoThreshold (ImagePlus imp)
    {
        ImageStatistics stats =
                imp.getStatistics (Measurements.MIN_MAX+
                Measurements.MODE);
        ImageProcessor ip = imp.getProcessor ();
        int threshold = ((ByteProcessor)ip).getAutoThreshold ();
        if ((stats.max-stats.mode)<(stats.mode-stats.min))
        {
            minThreshold = stats.min;
            maxThreshold = threshold;
        }
        else
        {
            minThreshold = threshold;
            maxThreshold = stats.max;
        }
    }
    
}/*end plugin*/

                //===============//
    //======    NEW CLASS  =======//
                 //==============//
class imageloader
{
    /**Returns an ImageIcon from a passed URL or else 
     *makes a gradient image if the image file fails to load.
     * @param urlString String is url for an image file to use as icon
     * @return ImageIcon based on image file passed to function
     */
    public static ImageIcon LoadImage (String urlString)
    {
        Image image1=null;
        boolean error = false;
        URL url = null;
        try
        {
            url = FL_.class.getResource (urlString);
            if (url.toString ()==null)error=true;}
        catch(AWTError awterror)
        {
            error = true;}
        catch (Error r)
        {
            error = true;}
        catch (java.lang.NullPointerException rrr)
        {
            error = true;}
        finally{}
       // {//url=null; IJ.error ("Error loading "+urlString);        }
        try
        {
            image1=Toolkit.getDefaultToolkit ().createImage (url);}
        catch(AWTError e)
        {
            error = true;}
        catch (java.lang.NullPointerException tr)
        {
            error = true;}
        catch (Error r)
        {
            error = true;}
        finally        {        }//FIXME what about it, then?
       
        int w = 16;
        int h = w;
        int pix[] = new int[w * h];
        
        if (error==true)
        {
            int index = 0;
            for (int y = 0; y < h; y++)
            {
                int red = (y * 255) / (h - 1);
                for (int x = 0; x < w; x++)
                {
                    int blue = (x * 255) / (w - 1);
                    pix[index++] = (255 << 24) | (red << 16) | blue;
                }
            }
            JFrame j = new JFrame ();
            image1 =
                    j.createImage (new 
                    MemoryImageSource (w, h, pix, 0, w));
        }        
        
        return new ImageIcon (image1);
    }
    
    /**Loads an html file from the passed string.
     *@param urlString string for the file to load
     @return URL a url for the file*/
    public static URL LoadHtml (String urlString)
    {     
        boolean error = false;
        URL url = null;
        try
        {
            url = FL_.class.getResource (urlString);
            if (url.toString ()==null)error=true;}
        catch(AWTError awterror)
        {
            error = true;}
        catch (Error r)
        {
            error = true;}
        catch (java.lang.NullPointerException rrr)
        {
            error = true;}
        finally
        {}        
        return url;
    }  
}

                //===============//
    //======    NEW CLASS  =======//
                 //==============//
/**Puts icons on the tree displayed in
 * the {@link GUI.FL_#makeTree tree} in
 * the image type panel of the
 * {@link GUI.FL_ GUI}, to offer the user
 * different configurations.
 *
 * The images for the icons are stored in the source package
 *<A HREF="../GUI/doc-files">see samples.</A> .
 *
 */
class MyRenderer extends DefaultTreeCellRenderer
{
    
    //////////////////////DECLARE VARIABLES////////////////////
  
    public ImageIcon[] makeGradientIcons
            ()
    {
        float inc=FLGraphics.increment;
        int h=(int)(FLGraphics.MaxDf/inc);
        int w = 20;
        int length = FLGraphics.NUMCOLOURCODES;
        ImageIcon [] I
                = new ImageIcon[length];
        
        //get an array of colours and Dfs
        float [] dfarray = new float[(int)(h)];
        float v = 0f;
        for (int i = 0; i < dfarray.length; i++)
        {
            dfarray[i]=v;
            v=v+inc;
        }
        
        //LOAD ICONS IN ARRAY AND RETURN IT
        for (int i = FLGraphics.NUMCOLOURCODES-1; i >=0; i--)
        {
            Color [] gcolours =
                    FLGraphics.MakeColours (dfarray, i);
            
            int pix[] = new int[w * h];
            
            
            int index = 0;
            for (int y = 0; y < h; y++)
            {
                for (int x = 0; x < w; x++)
                    pix[index++] =gcolours[y].getRGB ();
            }
            
            I[i] = new ImageIcon ( this.createImage (new
                    MemoryImageSource (w, h, pix, 0, w)));
        }
        return I;
    }
    /** Default constructor accepts no parameters.
     */
    public MyRenderer ()
    {
        
    }    

    
    public Component getTreeCellRendererComponent (
            JTree tree, Object value, boolean sel,
            boolean expanded, boolean leaf, int row, boolean hasFocus)
    {
        
        super.getTreeCellRendererComponent (
                tree,
                value,
                sel,
                expanded,
                leaf,
                row,
                hasFocus);
        
        ImageIcon [] icons = //makeIcons(urls,  iconwidth);
                icons = makeGradientIcons ();//20, 20, 21);
        int j=0;
        if (leaf)
        {
            DefaultMutableTreeNode node
                    =(DefaultMutableTreeNode)value;
            
            String title = node.toString ();
            //get the image that corresponds to the node's name
            for (int k = 0; k<icons.length;//urls.length;
            k++)
            {
                if (title.toLowerCase ().indexOf (Integer.toString (k))
                // (urls[k].substring(0, (urls[k].length()-4)).toLowerCase())
                >= 0 )
                    
                {//Set that icon for that node
                    setIcon (icons[k]);
                }
            }
            
        }return this;
    }
}




